<?php
class Auth {
	const ENC_COUNT   =  7;
	const SALT_LENGTH = 32;
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct(){
	}
	
	/***************************************************************
	 * 名前:instance
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function instance( $name, $instanceParam = NULL ){
		$file = sprintf("Auth_%s", $name);
		require_once("class/Auth/{$file}.php");
		
		$class = $file;
		return new $class($instanceParam);
	}
	
	
###################################################
# ログイン
###################################################
	
	/***************************************************************
	 * 名前:encPass
	 * 説明:パスワード暗号化
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function encPass( $pass, $salt = NULL ){
		// salt作成
		$salt = is_null($salt) ? substr(md5(uniqid(rand(), TRUE)), 0, self::SALT_LENGTH) : $salt;
		// パスワードにsalt付加
		$enc = $salt.$pass;
		// ENC_COUNT分の暗号化実行
		for ( $i=0; $i<self::ENC_COUNT; $i++ ){
			$enc = sha1($enc);
		}
		//暗号化されたパスワードにsalt付加
		return $salt.$enc;
	}
	
	/***************************************************************
	 * 名前:checkPass
	 * 説明:パスワード比較
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function checkPass( $pass, $encPass ){
		// salt取得
		$salt = substr($encPass, 0, self::SALT_LENGTH);
		// パスワード比較
		return (strcmp($encPass, self::encPass($pass, $salt)) === 0);
	}
	
	/***************************************************************
	 * 名前:login
	 * 説明:ログイン
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function login( $name, $password ){
		$result      = RES_OK;
		$res         = NULL;
		$userID      = INIT_ID;
		
		$objDAOUser  = DAO::instance("user");
		$objDAOLogin = DAO::instance("login");
		
		if ( $result === RES_OK ){
			$resDAOUser = $objDAOUser->get(NULL, $name, DAO::T_USER_STATE_ACTIVE, NULL, NULL, NULL, 1, NULL, TRUE);
			$result = $resDAOUser["result"];
			
			if ( $result === RES_OK ){
				$dataUser = $resDAOUser["data"][0];
				
				$userID       = $dataUser["t_user_id"];
				$userPassword = $dataUser["t_user_password"];
				
				// パスワード比較
				if ( self::checkPass($password, $userPassword) ){
					$_SESSION["logged"] = $userID;
					$objDAOLogin->insert($userID);
				} else {
					$result = RES_NG;
				}
			} else {
				$result = RES_NG;
			}
		}
		
		return ($result === RES_OK);
	}
	
	/***************************************************************
	 * 名前:logout
	 * 説明:ログアウト
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function logout(){
		// セッション変数を全て解除
		$_SESSION = array();
		unset($_SESSION["logged"]); 
		// 最終的に、セッションを破壊する
		session_destroy();
	}
	
	/***************************************************************
	 * 名前:isLogin
	 * 説明:ログインチェック
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isLogin(){
		return isset($_SESSION["logged"]) ? $_SESSION["logged"] : FALSE;
	}
	
###################################################
# ユーザータイプ
###################################################
	
	/***************************************************************
	 * 名前:isType
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	private static function isType( $targetType, $type = NULL, $userID = NULL ){
		if ( is_null($type) && is_null($userID) ){
			$type = User::get("type");
		}
		
		if ( is_null($type) ){
			$objDAOUser = DAO::instance("user");
			$resDAOUser = $objDAOUser->get($userID);
			return ($targetType === $resDAOUser["data"][0]["t_user_type"]);
		} else {
			return ($type === $targetType);
		}
	}
	
	/***************************************************************
	 * 名前:isSysmgr
	 * 説明:主システム管理者か？
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isSysmgr( $type = NULL, $userID = NULL ){
		return self::isType(DAO::T_USER_TYPE_SYSMGR, $type, $userID);
	}
	
	/***************************************************************
	 * 名前:isOrgmgr
	 * 説明:組織管理者か？
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isOrgmgr( $type = NULL, $userID = NULL ){
		return self::isType(DAO::T_USER_TYPE_ORGMGR, $type, $userID);
	}
	
	/***************************************************************
	 * 名前:isInsttr
	 * 説明:インストラクターか？
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isInsttr( $type = NULL, $userID = NULL ){
		return self::isType(DAO::T_USER_TYPE_INSTTR, $type, $userID);
	}
	
	/***************************************************************
	 * 名前:isNormal
	 * 説明:一般ユーザーか？
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isNormal( $type = NULL, $userID = NULL ){
		return self::isType(DAO::T_USER_TYPE_NORMAL, $type, $userID);
	}
	
	/***************************************************************
	 * 名前:isLogoff
	 * 説明:ログオフ中か？
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isLogoff( $type = NULL, $userID = NULL ){
		if ( $type === DAO::T_USER_TYPE_LOGOFF ){
			return TRUE;
		}
		
		return ! array_key_exists($type, DAO::T_USER_TYPE_TABLE());
	}
	
	/***************************************************************
	 * 名前:isUserTypeUpper
	 * 説明:権限が上か？
	 * 引数:
	 * 返値:
	****************************************************************/
	public static function isUserTypeUpper( $userType, $userTypeMine ){
		return ($userType < $userTypeMine);
	}
	
	
}
