<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_bbs extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:isSystemBbs
	 * 説明:プロジェクト掲示板かどうか
	 * 引数: $id = 掲示板ID
	 * 返値:
	****************************************************************/
	private function isSystemBbs($id, $projectId){
		$objDAOBbs = DAO::instance("bbs");
		$resDAOBbs = $objDAOBbs->get($id);
		if ( $resDAOBbs["result"] === RES_OK ){
			return ($resDAOBbs["data"][0]["t_project_id"] === 0);
		} else if ( $projectId === DAO::T_BBS_PROJECT_ID_SYSTEM ){
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:getProjectIdByBbsId
	 * 説明:掲示板IDからプロジェクトIDを取得する（システム掲示板の場合は 0 ）
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getProjectIdByBbsId($id, $projectId){
		if ( is_numeric($projectId) && ($projectId > 0) ){
			return $projectId;
		} else {
			$objDAOBbs = DAO::instance("bbs");
			$resDAOBbs = $objDAOBbs->get($id);
			if ( $resDAOBbs["result"] === RES_OK ){
				return $resDAOBbs["data"][0]["t_project_id"];
			} else {
				return NULL;
			}
		}
	}
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:掲示板一覧のSQL文
	 *      [Project] プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 *      [System]  ログインユーザー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array();
		if ( $this->isManager() ){
			// 管理者はOK
		} else {
			$arrSql = array();
			// システム掲示板
			$arrSql[] = "(t_project_id = 0)"; 
			// プロジェクト作成者
			$arrSql[] = "(get_user_id_by_project_id(t_project_id) = {$this->userId})";
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(t_project_id, {$this->userId})";
			// インストラクター
			if ( $this->isInstructor() ){
				// プロジェクト作成者と同じ組織
				$arrSql[] = "(get_organization_id_by_user_id(get_user_id_by_project_id(t_project_id)) = {$this->organizationId})";
			}
			
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => $this->getSqlOr($arrSql),
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:掲示板一覧閲覧権限があるか？
	 * 引数:
	 * 返値:
	 ****************************************************************/
	/*
	public function readList($projectId = NULL){
		if ( $this->isSystemBbs() ){
			// システム掲示板
			return FALSE;
		} else {
			// プロジェクト掲示板
			return FALSE;
		}
	}
	 */
	
	/***************************************************************
	 * 名前:read
	 * 説明:掲示板詳細閲覧権限があるか？
	 *      [System]  ログインユーザー
	 *      [Project] プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function read($id, $projectId = NULL){
		if ( $this->isSystemBbs($id, $projectId) ){
			// システム掲示板
			return Auth::isLogin($this->userType, $this->userId);
		} else {
			// プロジェクト掲示板
			if ( ! $this->validID($id) ){
				return FALSE;
			}
			$projectId = $this->getProjectIdByBbsId($id, $projectId);
			return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
		}
	}
	
	/***************************************************************
	 * 名前:raedItem
	 * 説明:掲示板投稿閲覧権限があるか？
	 *      [System]  ログインユーザー
	 *      [Project] プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $itemId = 掲示板ポストNo
	 *       $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function readItem($itemId, $id, $projectId = NULL){
		return $this->read($id, $projectId);
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:掲示板書込権限があるか？
	 *      [Create-System]  システム管理者
	 *      [Create-Project] 管理者 or プロジェクトメンバー
	 *      [Update] FALSE 
	 * 引数: $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function write($id, $projectId = NULL){
		if ( $this->writeIsCreate($id) ){
			// 追加
			if ( $this->isSystemBbs($id, $projectId) ){
				// システム掲示板
				return Auth::isSysmgr($this->userType, $this->userId);
			} else {
				// プロジェクト掲示板
				//$projectId = $this->getProjectIdByBbsId($id, $projectId);
				return FALSE;
			}
		} else {
			// 編集
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:writeItem
	 * 説明:掲示板投稿書込権限があるか？
	 *      [Create-System]  ログインユーザー
	 *      [Create-Project] 管理者 or インストラクター or プロジェクト有権者
	 *      [Update] FALSE 
	 * 引数: $itemId = 掲示板ポストNo
	 *       $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function writeItem($itemId, $id, $projectId = NULL){
		if ( $this->writeIsCreate($itemId) ){
			// 追加
			if ( $this->isSystemBbs($id, $projectId) ){
				// システム掲示板
				return Auth::isLogin($this->userType, $this->userId);
			} else {
				// プロジェクト掲示板
				$projectId = $this->getProjectIdByBbsId($id, $projectId);
				return ( $this->isProjectVoter($projectId) || $this->isRole($projectId, Task::ROLE_ROBS) ); // 掲示板投稿の場合のみ、オブザーバーでも投稿できる
			}
		} else {
			// 編集
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:additionItem
	 * 説明:掲示板追記権限があるか？
	 *      [Create-System]  FALSE
	 *      [Create-Project] 管理者 or 作成者
	 *      [Update] FALSE 
	 * 引数: $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function additionItem($itemId, $id, $projectId = NULL){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( $this->writeIsCreate($itemId) ){
			return FALSE;
		}
		
		// 追加
		if ( $this->isSystemBbs($id, $projectId) ){
			// システム掲示板
			return FALSE;
		} else {
			// プロジェクト掲示板
			$projectId = $this->getProjectIdByBbsId($id, $projectId);
			return ( $this->isCreateManager("project", $projectId) || $this->isCreator("bbs_post", array($itemId, $id)) );
		}
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:掲示板削除権限があるか？
	 *      FALSE
	 * 引数: $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function remove($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return FALSE;
	}
	
	/***************************************************************
	 * 名前:removeItem
	 * 説明:掲示板投稿削除権限があるか？
	 *      [Create-System]  FALSE
	 *      [Create-Project] 管理者 or 作成者
	 *      [Update] FALSE 
	 * 引数: $itemId = 掲示板ポストNo
	 *       $id = 掲示板ID
	 * 返値:
	****************************************************************/
	public function removeItem($itemId, $id, $projectId = NULL ){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( $this->writeIsCreate($itemId) ){
			return FALSE;
		}
		
		// 追加
		if ( $this->isSystemBbs($id, $projectId) ){
			// システム掲示板
			return FALSE;
		} else {
			// プロジェクト掲示板
			$projectId = $this->getProjectIdByBbsId($id, $projectId);
			return ( $this->isCreateManager("project", $projectId) || $this->isCreator("bbs_post", array($itemId, $id)) );
		}
	}
	
	
}
