<?php
abstract class Auth_common {
	var $userId = NULL;
	var $userType = NULL;
	var $organizationId = NULL;
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $__abstract ){
		$this->userId         = User::get("id");
		$this->userType       = User::get("type");
		$this->organizationId = User::get("organization_id");
	}
	
	//-------------------------------------------------------
	// 拡張クラスにメソッドの定義を強制する
	//-------------------------------------------------------
	abstract public function getSqlCondition();
	
	/***************************************************************
	 * 名前:getSqlOr
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function getSqlOr($arr){
		if ( is_array($arr) && count($arr)>0 ){
			$sql = implode(" OR ", $arr);
			return "({$sql})";
		} else {
			return "TRUE";
		}
	}
	
	/***************************************************************
	 * 名前:getCreatorUserID
	 * 説明: getCreatorUserID($table, $id) か getCreatorUserID($table, $classSuffix, $id) で呼ぶ
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getCreatorUserID( $table, $id, $arg = NULL ){
		$classSuffix = NULL;
		if ( ! is_null($arg)) {
			$classSuffix = $id;
			$id = $arg;
		}
		
		$objDAOXXX = DAO::instance($table, $classSuffix);
		if ( is_array($id) ){
			// ↓ $resDAOXXX = $objDAOXXX->get($id[0], $id[1], ... );
			$resDAOXXX = call_user_func_array( array($objDAOXXX, "get"), $id );
		} else {
			$resDAOXXX = $objDAOXXX->get($id);
		}
		
		if ( $resDAOXXX["result"] === RES_OK ){
			$dataXXX = $resDAOXXX["data"][0];
			return $dataXXX["t_user_id"];
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:isCreator
	 * 説明: isCreator($table, $id) か isCreator($table, $classSuffix, $id) で呼ぶ
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isCreator( $table, $id, $arg = NULL ){
		if ( ($createUserID = $this->getCreatorUserID($table, $id, $arg)) === FALSE ){
			return FALSE;
		}
		
		return ( $this->userId === $createUserID );
	}
	
	/***************************************************************
	 * 名前:isManager
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isManager(){
		return ( Auth::isSysmgr($this->userType, $this->userId) || Auth::isOrgmgr($this->userType, $this->userId) );
	}
	
	/***************************************************************
	 * 名前:isInstructor
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isInstructor(){
		return ( Auth::isInsttr($this->userType, $this->userId) );
	}
	
	/***************************************************************
	 * 名前:isCreateManager
	 * 説明: isCreateManager($table, $id) か isCreateManager($table, $classSuffix, $id) で呼ぶ
	 *       作成ユーザーの管理者かどうか
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isCreateManager( $table, $id, $arg = NULL ){
		// システム管理者は無条件
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			return TRUE;
			
		// 組織管理者は作成ユーザーの組織であるか確認
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) ){
			if ( ($createUserID = $this->getCreatorUserID($table, $id, $arg)) === FALSE ){
				return FALSE;
			}
			$createUserOrgID = DAO::storedFunction("get_organization_id_by_user_id", $createUserID);
			return ( $createUserOrgID === $this->organizationId );
			
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:isCreateInstructor
	 * 説明: isCreateInstructor($table, $id) か isCreateInstructor($table, $classSuffix, $id) で呼ぶ
	 *       作成ユーザーの組織のインストラクターかどうか
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isCreateInstructor( $table, $id, $arg = NULL ){
		// インストラクターは作成ユーザーの組織であるか確認
		if ( $this->isInstructor() ){
			if ( ($createUserID = $this->getCreatorUserID($table, $id, $arg)) === FALSE ){
				return FALSE;
			}
			$createUserOrgID = DAO::storedFunction("get_organization_id_by_user_id", $createUserID);
			return ( $createUserOrgID === $this->organizationId );
			
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function validID( $id ){
		return ( ! ($id === INIT_ID || $id === NULL));
	}
	
	/***************************************************************
	 * 名前:writeIsCreate
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function writeIsCreate( $id ){
		return ($id === INIT_ID || $id === NULL);
	}
	
	/***************************************************************
	 * 名前:isProjectVoter
	 * 説明:プロジェクトメンバーで役割を持っているか？（プロジェクト有権者）
	 *      管理者 or プロジェクトで役割を持つユーザー(オブザーバーを除く)
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isProjectVoter( $projectId ){
		if ( $this->isCreateManager("project", $projectId) ){
			return TRUE;
		} else {
			$objDAOProjectUser = DAO::instance("project_user");
			$resDAOProjectUser = $objDAOProjectUser->get($projectId, $this->userId);
			
			if ( $resDAOProjectUser["result"] === RES_OK ){
				foreach ( $resDAOProjectUser["data"] as $dataProjectUser ){
					$projectUserRole = $dataProjectUser["t_project_user_role"];
					if ( $projectUserRole === Task::ROLE_INIT ){
						return FALSE;
					} else if ( $projectUserRole === TASK::ROLE_ROBS ){
						// オブザーバー
					} else {
						return TRUE;
					}
				}
				return FALSE;
			}
			
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:isProjectMember
	 * 説明:プロジェクトメンバーか？
	 *      管理者 or プロジェクトのユーザー
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isProjectMember( $projectId ){
		if ( $this->isCreateManager("project", $projectId) ){
			return TRUE;
		} else {
			$objDAOProjectUser = DAO::instance("project_user");
			$resDAOProjectUser = $objDAOProjectUser->get($projectId, $this->userId);
			
			return ($resDAOProjectUser["result"] === RES_OK);
		}
	}
	
	/***************************************************************
	 * 名前:isRoleRPMG
	 * 説明: 指定の役割を持っているか？
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isRole( $projectId, $role ){
		$objDAOProjectUser = DAO::instance("project_user");
		$resDAOProjectUser = $objDAOProjectUser->get($projectId, $this->userId, $role);
		
		return ($resDAOProjectUser["result"] === RES_OK);
	}
	
	/***************************************************************
	 * 名前:isRoleRPMG
	 * 説明: 役割::RPMG(プロジェクトマネージャー)を持っているか？
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function isRoleRPMG( $projectId ){
		return $this->isRole($projectId, Task::ROLE_RPMG);
	}
}
