<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_data extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:getProjectIdByDataId
	 * 説明:データIDからプロジェクトIDを取得する
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getProjectIdByDataId($id, $projectId){
		if ( is_numeric($projectId) && ($projectId > 0) ){
			return $projectId;
		} else {
			$objDAOData = DAO::instance("data");
			$resDAOData = $objDAOData->get($id);
			if ( $resDAOData["result"] === RES_OK ){
				return $resDAOData["data"][0]["t_project_id"];
			} else {
				return NULL;
			}
		}
	}
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:データ一覧のSQL文
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array(
			"t_data_state" => DAO::T_DATA_STATE_NORMAL,
		);
		if ( $this->isManager() ){
			// 管理者はOK
		} else {
			$arrSql = array();
			// プロジェクト作成者
			$arrSql[] = "(get_user_id_by_project_id(t_project_id) = {$this->userId})";
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(t_project_id, {$this->userId})";
			// 公開範囲：全体
			$arrSql[] = "(t_data_public = ".DAO::T_DATA_PUBLIC_OPN.")";
			// 公開範囲：組織内
			$arrSql[] = "((t_data_public = ".DAO::T_DATA_PUBLIC_ORG.") AND (get_project_organization_id(t_project_id) = {$this->organizationId}))";
			// インストラクター
			if ( $this->isInstructor() ){
				// プロジェクト作成者と同じ組織
				$arrSql[] = "(get_organization_id_by_user_id(get_user_id_by_project_id(t_project_id)) = {$this->organizationId})";
			}
			
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => $this->getSqlOr($arrSql),
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:データ一覧閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList($projectId){
		return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:データ詳細閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $id = データID
	 * 返値:
	****************************************************************/
	public function read($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( $this->isManager() ){
			return TRUE;
		}
		$refProjectId = $this->getProjectIdByDataId($id, NULL);
		if ( $this->isCreateInstructor("project", $refProjectId) || $this->isCreator("project", $refProjectId) || $this->isProjectMember($refProjectId) ){
			return TRUE;
		} else {
			// データの公開状態の確認
			$objDAOData = DAO::instance("data");
			$resDAOData = $objDAOData->get($id, DAO::T_DATA_STATE_NORMAL);
			
			if ( $resDAOData["result"] === RES_OK ){
				$dataData = $resDAOData["data"][0];
				switch ( $dataData["t_data_public"] ){
					// 一般公開
					case DAO::T_DATA_PUBLIC_OPN:
						return TRUE;
					// 組織内公開
					case DAO::T_DATA_PUBLIC_ORG:
						return ($dataData["t_organization_id"] === $this->organizationId);
					// プロジェクト内公開(公開しない)
					case DAO::T_DATA_PUBLIC_PRJ:
						return FALSE;
				}
			}
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:データ書込権限があるか？
	 *      [Create] 管理者 or インストラクター or プロジェクト有権者
	 *      [Update] 管理者 or インストラクター or プロジェクト有権者
	 * 引数: $id = データID
	 * 返値:
	****************************************************************/
	public function write($id, $projectId){
		if ( $this->writeIsCreate($id) ){
			// 追加
			return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) );
		} else {
			// 編集
			$refProjectId = $this->getProjectIdByDataId($id, NULL);
			if ( $projectId !== $refProjectId ){
				return FALSE;
			}
			return ( $this->isCreateManager("project", $refProjectId) || $this->isCreateInstructor("project", $refProjectId) || $this->isProjectVoter($refProjectId) );
		}
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:データ削除権限があるか？
	 *      管理者 or インストラクター or プロジェクト有権者
	 * 引数: $id = データID
	 * 返値:
	****************************************************************/
	public function remove($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$refProjectId = $this->getProjectIdByDataId($id, NULL);
		if ( $projectId !== $refProjectId ){
			return FALSE;
		}
		return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) );
		//return ( $this->isCreateManager("data", $id) || $this->isCreator("data", $id) );
	}
	
	/***************************************************************
	 * 名前:refrence
	 * 説明:データ参照追加(削除)権限があるか？
	 *      - 追加するプロジェクトの 管理者 or インストラクター or プロジェクト有権者
	 *      - 参照元データの閲覧権限
	 * 引数: $id = 用語集ID
	 * 返値:
	****************************************************************/
	public function refrence($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) ){
			$refProjectId = $this->getProjectIdByDataId($id, NULL);
			return $this->read($id, $refProjectId);
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:processDownload
	 * 説明:データダウンロード権限があるか？
	 *      読込可能 and ファイルが存在
	 * 引数: $id = データID
	 * 返値:
	****************************************************************/
	public function processDownload($id, $projectId, $fileCheck = FALSE){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( ! $this->read($id, $projectId) ){
			return FALSE;
		}
		
		if ( $fileCheck ){
			$file = Setting::get("PATH_DATA_DATA") . "/{$id}";
			return file_exists($file);
		} else {
			return TRUE;
		}
	}
	
}
