<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_document extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:getProjectIdByDocumentId
	 * 説明:文書IDからプロジェクトIDを取得する
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getProjectIdByDocumentId($id, $projectId){
		if ( is_numeric($projectId) && ($projectId > 0) ){
			return $projectId;
		} else {
			$objDAODocument = DAO::instance("document");
			$resDAODocument = $objDAODocument->get($id);
			if ( $resDAODocument["result"] === RES_OK ){
				return $resDAODocument["data"][0]["t_project_id"];
			} else {
				return NULL;
			}
		}
	}
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:文書一覧のSQL文
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array(
			"t_document_state" => array("type" => DAO::TYPE_IN, "value" => array_keys(DAO::T_DOCUMENT_STATE_VALID_TABLE())),
		);
		if ( $this->isManager() ){
			// 管理者はOK
		} else {
			$arrSql = array();
			// プロジェクト作成者
			$arrSql[] = "(get_user_id_by_project_id(t_project_id) = {$this->userId})";
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(t_project_id, {$this->userId})";
			// インストラクター
			if ( $this->isInstructor() ){
				// プロジェクト作成者と同じ組織
				$arrSql[] = "(get_organization_id_by_user_id(get_user_id_by_project_id(t_project_id)) = {$this->organizationId})";
			}
			
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => $this->getSqlOr($arrSql),
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:文書一覧閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList($projectId){
		return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:文書詳細閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function read($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:文書書込権限があるか？
	 *      [Create] 管理者 or インストラクター or プロジェクト有権者
	 *      [Update] 管理者 or インストラクター or プロジェクト有権者
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function write($id, $projectId){
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		if ( $this->writeIsCreate($id) ){
			// 追加
			return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) );
		} else {
			// 編集
			return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) );
		}
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:文書削除権限があるか？
	 *      文書を作成したユーザー or 管理者 or (インストラクター and プロジェクト作成者)
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function remove($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		if ( $this->isCreateManager("project", $projectId) || $this->isCreator("document", $id) ){
			return TRUE;
		} else if ( $this->isCreateInstructor("project", $projectId) && $this->isCreator("project", $projectId) ){
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:processDiff
	 * 説明:比較処理権限があるか？
	 *      管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function processDiff($projectId) {
		return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:processExport
	 * 説明:ファイルエクスポート権限があるか？
	 *      管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function processExport($id, $projectId) {
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:processEditor
	 * 説明:QREdit起動権限があるか？
	 *      管理者 or プロジェクト有権者
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function processEditor($id, $projectId) {
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isProjectVoter($projectId) );
	}
	
	/***************************************************************
	 * 名前:processInsertSourceText
	 * 説明:訳文に原文挿入権限があるか？
	 *      管理者 or プロジェクト有権者
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function processInsertSourceText($id, $projectId) {
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isProjectVoter($projectId) );
	}
	
	/***************************************************************
	 * 名前:processInsertSimilarity
	 * 説明:訳文に類似文挿入権限があるか？
	 *      管理者 or プロジェクト有権者
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function processInsertSimilarity($id, $projectId) {
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isProjectVoter($projectId) );
	}
	
	/***************************************************************
	 * 名前:processRunMT
	 * 説明:機械翻訳権限があるか？
	 *      管理者 or プロジェクト有権者
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function processRunMT($id, $projectId) {
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isProjectVoter($projectId) );
	}
	
	/***************************************************************
	 * 名前:processWorkflowRead
	 * 説明:文書ごとのワークフロー詳細を見る権限があるか
	 *      管理者 or プロジェクトメンバー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function processWorkflowRead($id, $projectId) {
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByDocumentId($id, $projectId);
		
		$authTask = Auth::instance("task");
		return $authTask->readList($projectId);
	}
	
	/***************************************************************
	 * 名前:processCopy
	 * 説明:コピー権限があるか？
	 * 引数: $id = 用語集ID
	 * 返値:
	 ****************************************************************/
	public function processCopy($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return $this->read($id, $projectId) && $this->write(INIT_ID, $projectId);
	}
	
}
