<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_organization extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:組織一覧のSQL文
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array();
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			// システム管理者はOK
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
			// 組織管理者,インストラクターはOK
		} else {
			// 自組織のみ
			$arrCondition["auth"] = array(
				"type" => DAO::TYPE_SQL,
				"value" => "t_organization_id = {$this->organizationId}",
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:組織一覧閲覧権限があるか？
	 *      システム管理者 or 組織管理者
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList(){
		return ( Auth::isSysmgr($this->userType, $this->userId) || Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) );
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:組織詳細閲覧権限があるか？
	 *      システム管理者 or 組織管理者（自分の組織のみ） or インストラクター
	 * 引数: $id = 組織ID
	 * 返値:
	****************************************************************/
	public function read($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			return TRUE;
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
			return TRUE;
		} else {
			return ( $id === $this->organizationId);
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:組織書込権限があるか？
	 *      [Create] システム管理者
	 *      [Update] システム管理者
	 * 引数: $id = 組織ID
	 * 返値:
	****************************************************************/
	public function write($id){
		if ( $this->writeIsCreate($id) ){
			// 追加
			return ( Auth::isSysmgr($this->userType, $this->userId) );
		} else {
			// 編集
			return ( Auth::isSysmgr($this->userType, $this->userId) );
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:組織情報書込権限があるか？(名前は変更不可)
	 *      [Create] -
	 *      [Update] システム管理者、組織管理者、インストラクター
	 * 引数: $id = 組織ID
	 * 返値:
	****************************************************************/
	public function write_isInfo($id){
		if ( $this->validID($id) ){
			if ( Auth::isSysmgr($this->userType, $this->userId) ){
				return TRUE;
			} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
				return ( $id === $this->organizationId);
			} else {
				return FALSE;
			}
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:組織削除権限があるか？
	 * 引数: $id = 組織ID
	 * 返値:
	****************************************************************/
	public function remove($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return FALSE;
	}
	
}
