<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_project extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:プロジェクト一覧のSQL文
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array();
		
		$arrSql = array();
		// 管理者
		if ( $this->isManager() ){
			
		// インストラクター
		} else if ( $this->isInstructor() ){
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(t_project_id, {$this->userId})";
			// プロジェクト作成者と同じ組織
			$arrSql[] = "(t_organization_id = {$this->organizationId})";
		} else {
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(t_project_id, {$this->userId})";
		}
		
		$arrCondition["auth"] = array(
			"type"  => DAO::TYPE_SQL,
			"value" => $this->getSqlOr($arrSql),
		);
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:プロジェクト一覧閲覧権限があるか？
	 *      ログインユーザー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList(){
		return Auth::isLogin($this->userType, $this->userId);
	}
	
	public function readList_isCreate(){
		return ( $this->isInstructor() || $this->isManager() );
	}
	
	public function readList_isOrg(){
		return ( $this->isInstructor() || $this->isManager() );
	}
	
	public function readList_isAll(){
		return ( $this->isManager() );
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:プロジェクト詳細閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function read($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isManager() || $this->isCreateInstructor("project", $id) || $this->isCreator("project", $id) || $this->isProjectMember($id) );
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:プロジェクト書込権限があるか？
	 *      [Create] 管理者 or インストラクター
	 *      [Update] 管理者 or インストラクター
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function write($id){
		if ( $this->writeIsCreate($id) ){
			// 追加
			return ( $this->isManager() || $this->isInstructor() );
		} else {
			// 編集
			return ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) );
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:プロジェクトメンバー書込権限があるか？
	 *      管理者 or インストラクター or プロジェクトマネージャー
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function write_isMember($id, $organizationId){
		if ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) ){
			return TRUE;
		} else if ( $this->isRoleRPMG($id) ){
			if ( $this->validID($organizationId) ){
				return ( $this->organizationId === $organizationId);
			} else {
				return TRUE;
			}
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:プロジェクト状態書込権限があるか？
	 *      管理者 or インストラクター
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function write_isState($id){
		return ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) );
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:プロジェクト削除権限があるか？
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function remove($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) || $this->isCreator("project", $id) );
	}
	
	/***************************************************************
	 * 名前:processImport
	 * 説明:プロジェクトインポート権限があるか？
	 *      管理者 or インストラクター
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function processImport($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) );
	}
	
	/***************************************************************
	 * 名前:processExport
	 * 説明:プロジェクトエクスポート権限があるか？
	 *      管理者 or インストラクター
	 * 引数: $id = プロジェクトID
	 * 返値:
	****************************************************************/
	public function processExport($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) );
	}
	
	/***************************************************************
	 * 名前:processCopy
	 * 説明:コピー権限があるか？
	 * 引数: $id = プロジェクトID
	 * 返値:
	 ****************************************************************/
	public function processCopy($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isCreateManager("project", $id) || $this->isCreateInstructor("project", $id) || $this->isCreator("project", $id) );
	}
	
	/***************************************************************
	 * 名前:processDSearch
	 * 説明:対訳検索(文書)権限があるか？
	 *      管理者 or インストラクター or プロジェクトメンバ
	 * 引数: $id = プロジェクトID
	 * 返値:
	 ****************************************************************/
	public function processDSearch($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isManager() || $this->isCreateInstructor("project", $id) || $this->isProjectMember($id) );
	}
	
	/***************************************************************
	 * 名前:processBSearch
	 * 説明:対訳検索(翻訳メモリ)権限があるか？
	 *      管理者 or インストラクター or プロジェクトメンバ
	 * 引数: $id = プロジェクトID
	 * 返値:
	 ****************************************************************/
	public function processBSearch($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isManager() || $this->isCreateInstructor("project", $id) || $this->isProjectMember($id) );
	}
	
	/***************************************************************
	 * 名前:processSimilarity
	 * 説明:対訳検索権限があるか？
	 *      管理者 or インストラクター or プロジェクトメンバ
	 * 引数: $id = プロジェクトID
	 * 返値:
	 ****************************************************************/
	public function processSimilarity($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return ( $this->isManager() || $this->isCreateInstructor("project", $id) || $this->isProjectMember($id) );
	}
	
}
