<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_task extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:getProjectIdByTaskId
	 * 説明:タスクIDからプロジェクトIDを取得する
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getProjectIdByTaskId($id, $projectId){
		if ( is_numeric($projectId) && ($projectId > 0) ){
			return $projectId;
		} else {
			$objDAOTask = DAO::instance("task");
			$resDAOTask = $objDAOTask->get($id);
			if ( $resDAOTask["result"] === RES_OK ){
				return $resDAOTask["data"][0]["t_project_id"];
			} else {
				return NULL;
			}
		}
	}
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:タスク一覧のSQL文
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array();
		if ( $this->isManager() ){
			// 管理者はOK
		} else {
			$arrSql = array();
			// 作成者
			$arrSql[] = "(get_user_id_by_project_id(get_project_id_by_workflow_id(t_workflow_id)) = {$this->userId})";
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(get_project_id_by_workflow_id(t_workflow_id), {$this->userId})";
			// インストラクター
			if ( $this->isInstructor() ){
				// 作成者と同じ組織
				$arrSql[] = "(get_organization_id_by_user_id(get_user_id_by_project_id(get_project_id_by_workflow_id(t_workflow_id))) = {$this->organizationId})";
			}
			
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => $this->getSqlOr($arrSql),
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:タスク一覧閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList($projectId){
		return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:タスク詳細閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数: $id = タスクID
	 * 返値:
	****************************************************************/
	public function read($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:タスク書込権限があるか？
	 *      [Create] 管理者 or インストラクター or プロジェクトの役割でプロジェクトマネージャー
	 *      [Update] 管理者 or インストラクター or プロジェクトの役割でプロジェクトマネージャー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function write($id, $projectId){
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		if ( $this->writeIsCreate($id) ){
			// 追加
			return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isRoleRPMG($projectId) );
		} else {
			// 編集
			return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isRoleRPMG($projectId) );
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:ワークフロー設定書込権限があるか？
	 *      管理者 or インストラクター or プロジェクトマネージャー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function write_isWorkflow($id, $projectId){
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isRoleRPMG($projectId) );
	}
	
	/***************************************************************
	 * 名前:write_isWorkflow_getTypeGroup
	 * 説明:ワークフロー設定で変更可能なタスクタイプグループ
	 * 引数:
	 * 返値:
	****************************************************************/
	public function write_isWorkflow_getTypeGroup($id, $projectId){
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		$arrNonTypeGroup = array();
		if ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) ){
			$arrNonTypeGroup = array();
		} else if ( $this->isRoleRPMG($projectId) ){
			$arrNonTypeGroup = array( Task::TYPE_GROUP_WORKFLOW );
		} else {
			return FALSE;
		}
		
		return array_diff( array_keys(Task::TYPE_GROUP_TABLE()), $arrNonTypeGroup);
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:タスク割当て書込権限があるか？
	 *      管理者 or インストラクター or プロジェクトマネージャー
	 *      t_workflow.t_workflow_task_typeに値が入っている
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function write_isAllocate($id, $projectId){
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		if ( $this->write($id, $projectId) ){
			$objDAOWorkflow = DAO::instance("workflow");
			$resDAOWorkflow = $objDAOWorkflow->get(NULL, $projectId);
			if ( $resDAOWorkflow["result"] === RES_OK ){
				$dataWorkflowTasKType = $resDAOWorkflow["data"][0]["t_workflow_task_type"];
				return ( strlen($dataWorkflowTasKType) > 0 );
			}
		}

		return false;
	}
	
	/***************************************************************
	 * 名前:write_isState
	 * 説明:タスクの状態更新の権限があるか？
	 *      タスク編集権限があるユーザー or タスク担当者
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function write_isState($id, $projectId){
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		if ( $this->write($id, $projectId) ){
			return TRUE;
		} else {
			$objDAOTaskActor = DAO::instance("task_actor");
			$resDAOTaskActor = $objDAOTaskActor->get($id, $this->userId, NULL, DAO::T_TASK_ACTOR_TYPE_ACTOR);
			return ( $resDAOTaskActor["result"] === RES_OK );
		}
	}
	
	/***************************************************************
	 * 名前:write_isEditState
	 * 説明:タスクの状態更新の権限があるか？（編集画面）
	 *      管理者 or プロジェクトの役割でプロジェクトマネージャー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function write_isEditState($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isRoleRPMG($projectId) );
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:タスク削除権限があるか？
	 *      管理者 or インストラクター or プロジェクトの役割でプロジェクトマネージャー
	 * 引数: $id = 文書ID
	 * 返値:
	****************************************************************/
	public function remove($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByTaskId($id, $projectId);
		return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isRoleRPMG($projectId) );
	}
	
}
