<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_term_root extends Auth_common {
	var $kind = NULL; // Term::KIND_TABLE()
		
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $kind ){
		if ( ! Validate::inKeyArray($kind, Term::KIND_TABLE()) ){
			throw new Exception('ERROR: Auth_term_root.construct');
		}
		
		$this->kind = $kind;
		parent::__construct("");
	}
	
	/***************************************************************
	 * 名前:checkKind
	 * 説明:Kindの確認
	 * 引数:
	 * 返値:
	****************************************************************/
	private function checkKind(){
		return Validate::inKeyArray($this->kind, Term::KIND_TABLE());
	}
	
	/***************************************************************
	 * 名前:getProjectIdByTermRootId
	 * 説明:用語集IDからプロジェクトIDを取得する
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getProjectIdByTermRootId($id, $projectId){
		if ( is_numeric($projectId) && ($projectId > 0) ){
			return $projectId;
		} else {
			$objDAOTermRoot = DAO::instance("term_root", $this->kind);
			$resDAOTermRoot = $objDAOTermRoot->get($id);
			if ( $resDAOTermRoot["result"] === RES_OK ){
				return $resDAOTermRoot["data"][0]["t_project_id"];
			} else {
				return NULL;
			}
		}
	}
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:用語集一覧のSQL文
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array(
			"t_{$this->kind}_root_state" => DAO::T_TERM_ROOT_STATE_NORMAL,
		);
		if ( $this->isManager() ){
			// 管理者はOK
		} else {
			$arrSql = array();
			// 作成者
			$arrSql[] = "(get_user_id_by_project_id(t_project_id) = {$this->userId})";
			// プロジェクトメンバー
			$arrSql[] = "is_project_member(t_project_id, {$this->userId})";
			// 公開範囲：全体
			$arrSql[] = "(t_{$this->kind}_root_public = ".DAO::T_TERM_ROOT_PUBLIC_OPN.")";
			// 公開範囲：組織内
			$arrSql[] = "((t_{$this->kind}_root_public = ".DAO::T_TERM_ROOT_PUBLIC_ORG.") AND (get_project_organization_id(t_project_id) = {$this->organizationId}))";
			// インストラクター
			if ( $this->isInstructor() ){
				// 作成者と同じ組織
				$arrSql[] = "(get_organization_id_by_user_id(get_user_id_by_project_id(t_project_id)) = {$this->organizationId})";
			}
			
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => $this->getSqlOr($arrSql),
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:用語集一覧閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList($projectId){
		if ( ! $this->checkKind() ){
			return FALSE;
		}
		return ( $this->isManager() || $this->isCreateInstructor("project", $projectId) || $this->isCreator("project", $projectId) || $this->isProjectMember($projectId) );
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:用語集詳細閲覧権限があるか？
	 *      プロジェクトを作成したユーザー or 管理者 or インストラクター or プロジェクトメンバー
	 *        なおかつ、用語集の公開状態
	 * 引数: $id = 用語集ID
	 * 返値:
	****************************************************************/
	public function read($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( ! $this->checkKind() ){
			return FALSE;
		}
		if ( $this->isManager() ){
			return TRUE;
		}
		$refProjectId = $this->getProjectIdByTermRootId($id, NULL);
		if ( $this->isCreateInstructor("project", $refProjectId) || $this->isCreator("project", $refProjectId) || $this->isProjectMember($refProjectId) ){
			return TRUE;
		} else {
			// 用語集の公開状態の確認
			$objDAOTermRoot = DAO::instance("term_root", $this->kind);
			$resDAOTermRoot = $objDAOTermRoot->get($id, DAO::T_TERM_ROOT_STATE_NORMAL);
			
			if ( $resDAOTermRoot["result"] === RES_OK ){
				$dataTermRoot = $resDAOTermRoot["data"][0];
				switch ( $dataTermRoot["t_term_root_public"] ){
					// 一般公開
					case DAO::T_TERM_ROOT_PUBLIC_OPN:
						return TRUE;
					// 組織内公開
					case DAO::T_TERM_ROOT_PUBLIC_ORG:
						return ($dataTermRoot["t_organization_id"] === $this->organizationId);
					// プロジェクト内公開(公開しない)
					case DAO::T_TERM_ROOT_PUBLIC_PRJ:
						return FALSE;
				}
			}
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:用語集書込権限があるか？
	 *      [Create] 管理者 or インストラクター or プロジェクト有権者
	 *      [Update] 管理者 or インストラクター or プロジェクト有権者
	 * 引数: $id = 用語集ID
	 * 返値:
	****************************************************************/
	public function write($id, $projectId){
		if ( ! $this->checkKind() ){
			return FALSE;
		}
		if ( $this->writeIsCreate($id) ){
			// 追加
			return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) );
		} else {
			// 編集
			$refProjectId = $this->getProjectIdByTermRootId($id, NULL);
			if ( $projectId !== $refProjectId ){
				return FALSE;
			}
			return ( $this->isCreateManager("project", $refProjectId) || $this->isCreateInstructor("project", $refProjectId) || $this->isProjectVoter($refProjectId) );
		}
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:用語集削除権限があるか？
	 *      管理者 or インストラクター or プロジェクト有権者
	 * 引数: $id = 用語集ID
	 * 返値:
	****************************************************************/
	public function remove($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( ! $this->checkKind() ){
			return FALSE;
		}
		$refProjectId = $this->getProjectIdByTermRootId($id, NULL);
		if ( $projectId !== $refProjectId ){
			return FALSE;
		}
		return ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) );
	}
	
	/***************************************************************
	 * 名前:refrence
	 * 説明:用語集参照追加(削除)権限があるか？
	 *      - 追加するプロジェクトの 管理者 or インストラクター or プロジェクト有権者
	 *      - 参照元用語集の閲覧権限
	 * 引数: $id = 用語集ID
	 * 返値:
	****************************************************************/
	public function refrence($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( ! $this->checkKind() ){
			return FALSE;
		}
		$projectId = $this->getProjectIdByTermRootId($id, $projectId);
		if ( $this->isCreateManager("project", $projectId) || $this->isCreateInstructor("project", $projectId) || $this->isProjectVoter($projectId) ){
			$refProjectId = $this->getProjectIdByTermRootId($id, NULL);
			return $this->read($id, $refProjectId);
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:processImport
	 * 説明:インポート権限があるか？
	 * 引数: $id = 用語集ID
	 * 返値:
	 ****************************************************************/
	public function processImport($id, $projectId){
		return $this->write($id, $projectId);
	}
	
	/***************************************************************
	 * 名前:processImportDelete
	 * 説明:インポートキャンセル権限があるか？
	 * 引数: $id = 用語集ID
	 * 返値:
	 ****************************************************************/
	public function processImportDelete($id, $projectId){
		return $this->processImport($id, $projectId);
	}
	
	/***************************************************************
	 * 名前:processExport
	 * 説明:エクスポート権限があるか？
	 * 引数: $id = 用語集ID
	 * 返値:
	 ****************************************************************/
	public function processExport($id, $projectId){
		return $this->read($id, $projectId);
	}
	
	/***************************************************************
	 * 名前:processCopy
	 * 説明:コピー権限があるか？
	 * 引数: $id = 用語集ID
	 * 返値:
	 ****************************************************************/
	public function processCopy($id, $projectId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( ! $this->checkKind() ){
			return FALSE;
		}
		return $this->read($id, $projectId) && $this->write(INIT_ID, $projectId);
	}
	
}
