<?php require_once("class/Auth/Auth_common.php"); ?>
<?php
class Auth_user extends Auth_common {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function __construct( $__abstract ){
	}
	 */
	
	/***************************************************************
	 * 名前:getSqlCondition
	 * 説明:ユーザー一覧のSQL文
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getSqlCondition(){
		$arrCondition = array();
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			// システム管理者はOK
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) ){
			// 組織管理者はOK
		} else if ( Auth::isInsttr($this->userType, $this->userId) ){
			// インストラクターはOK
		} else if ( Auth::isNormal($this->userType, $this->userId) ){
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => "t_organization_id = {$this->organizationId}",
			);
		} else {
			// NG
			$arrCondition["auth"] = array(
				"type"  => DAO::TYPE_SQL,
				"value" => "FALSE",
			);
		}
		
		return $arrCondition;
	}
	
	/***************************************************************
	 * 名前:readList
	 * 説明:ユーザー一覧閲覧権限があるか？
	 *      管理者 or インストラクター
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList($organizationId){
		if ( $this->validID($organizationId) ){
			// "所属"
			if ( $this->isManager() || $this->isInstructor() ){
				return TRUE;
			} else {
				return ($this->organizationId === $organizationId) && Auth::isLogin($this->userType, $this->userId);
			}
		} else {
			// "すべて"
			return ( $this->isManager() || $this->isInstructor() );
		}
	}
	
	/***************************************************************
	 * 名前:readList_isInfo
	 * 説明:ユーザー一覧したときに登録日や最終ログイン日時が表示されるか
	 *      管理者 or インストラクター or インストラクター
	 * 引数:
	 * 返値:
	****************************************************************/
	public function readList_isInfo($organizationId){
		if ( $this->validID($organizationId) ){
			// "所属"
			if ( Auth::isSysmgr($this->userType, $this->userId) ){
				return TRUE;
			} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
				return ( $this->organizationId === $organizationId );
			} else {
				return FALSE;
			}
		} else {
			// "すべて"
			return Auth::isSysmgr($this->userType, $this->userId);
		}
	}
	
	/***************************************************************
	 * 名前:read
	 * 説明:ユーザー詳細閲覧権限があるか？
	 *      システム管理者 or 組織管理者（自分の組織のみ） or インストラクター（自分の組織のみ）
	 * 引数: $id = ユーザーID
	 * 返値:
	****************************************************************/
	public function read($id, $organizationId){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		if ( $this->userId === $id ){
			return TRUE;
		} else if ( Auth::isSysmgr($this->userType, $this->userId) ){
			return TRUE;
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
			$objDAOUser = DAO::instance("user");
			$resDAOUser = $objDAOUser->get($id);
			if ( $resDAOUser["result"] !== RES_OK ){
				return FALSE;
			}
			$dataUser = $resDAOUser["data"][0];
			if ( $this->organizationId !== $dataUser["t_organization_id"] ){
				return FALSE;
			}
			if ( ! Validate::inKeyArray($dataUser["t_user_type"], $this->write_getUserType()) ){
				return FALSE;
			}
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:write
	 * 説明:ユーザー書込権限があるか？
	 *      [Create] 管理者 or インストラクター
	 *      [Update] 管理者 or インストラクター
	 * 引数: $id = ユーザーID
	 * 返値:
	****************************************************************/
	public function write($id, $organizationId){
		if ( $this->writeIsCreate($id) ){
			// 追加
			if ( Auth::isSysmgr($this->userType, $this->userId) ){
				return TRUE;
			} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
				if ( ! is_numeric($organizationId) ){
					return TRUE;
				} else {
					return ( $this->organizationId === $organizationId);
				}
			} else {
				return FALSE;
			}
		} else if ( $this->userId === $id ){
			return FALSE;
		} else {
			return $this->read($id, $organizationId);
		}
	}
	
	/***************************************************************
	 * 名前:write_isLoginState
	 * 説明:非活発ユーザ表示書込権限があるか？
	 *      管理者 or インストラクター 且つ 自分自身以外
	 * 引数: $id = ユーザーID
	 * 返値:
	****************************************************************/
	public function write_isLoginState($id, $organizationId){
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			return TRUE;
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
			return $this->organizationId === $organizationId && $this->userId !== $id;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:write_isSetting
	 * 説明:ユーザー設定書込権限があるか？
	 *      ログイン and 自分の設定
	 * 引数: $id = ユーザーID
	 * 返値:
	****************************************************************/
	public function write_isSetting($id){
		return Auth::isLogin($this->userType, $this->userId) && ($id = $this->userId);
	}
	
	/***************************************************************
	 * 名前:write_isOrganization
	 * 説明:ユーザー書込時の組織変更権限があるか？
	 * 引数:
	 * 返値:
	****************************************************************/
	public function write_isOrganization(){
		return Auth::isSysmgr($this->userType, $this->userId);
	}
	
	/***************************************************************
	 * 名前:remove
	 * 説明:ユーザー削除権限があるか？
	 * 引数: $id = ユーザーID
	 * 返値:
	****************************************************************/
	public function remove($id){
		if ( ! $this->validID($id) ){
			return FALSE;
		}
		return FALSE;
	}
	
	/***************************************************************
	 * 名前:getValue_UserType
	 * 説明:ユーザーが追加可能なユーザータイプ（基本自分のタイプより下位のタイプのみ可能）
	 * 引数:
	 * 返値:
	****************************************************************/
	public function write_getUserType(){
		$arrUserType = array();
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			$arrUserType = array( DAO::T_USER_TYPE_ORGMGR, DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_NORMAL );
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) ){
			$arrUserType = array( DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_NORMAL );
		} else if ( Auth::isInsttr($this->userType, $this->userId) ){
			$arrUserType = array( DAO::T_USER_TYPE_NORMAL );
		}
		
		$result = array();
		foreach ( $arrUserType as $userType ){
			$result[$userType] = DAO::T_USER_TYPE_TABLE($userType);
		}
		
		if ( count($result) > 0 ){
			return $result;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:getValue_Organization
	 * 説明:ユーザーが追加可能な組織
	 * 引数:
	 * 返値:
	****************************************************************/
	public function write_getOrganization(){
		$arrOrganization = array();
		if ( Auth::isSysmgr($this->userType, $this->userId) ){
			// 一覧取得
			$objDAOOrganization = DAO::instance("organization");
			
			$resDAOOrganization = $objDAOOrganization->get(NULL, NULL, FALSE);
			if ( $resDAOOrganization["result"] === RES_OK ){
				foreach ( $resDAOOrganization["data"] as $dataOrganization ){
					$organizationID   = $dataOrganization["t_organization_id"];
					$organizationName = $dataOrganization["t_organization_name"];
					
					$arrOrganization[$organizationID] = $organizationName;
				}
			} else {
				return FALSE;
			}
		} else if ( Auth::isOrgmgr($this->userType, $this->userId) || Auth::isInsttr($this->userType, $this->userId) ){
			// 所属組織のみ
			$arrOrganization[User::get("organization_id")] = User::get("organization_name");
		} else {
			return FALSE;
		}
		
		return $arrOrganization;
	}
}
