<?php
Class DAO_data extends DAO {
	const ERROR_TITLE_DUPLICATE = 101;
	
	const ORDER_KEY_PROP     = "prop";
	const ORDER_KEY_PUBLIC   = "public";
	public static function ORDER_KEY_SQL_TABLE( $key = NULL ){
		$arrTable = array(
			self::ORDER_KEY_PROP   => "t_data_prop",
			self::ORDER_KEY_PUBLIC => "t_data_public",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "t_data" ){
		$this->tableName = $_tableName;
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_data_id"      => INIT_ID,
			"t_data_state"   => DAO::T_DATA_STATE_NORMAL,
			"t_data_public"  => DAO::T_DATA_PUBLIC_PRJ,
			"t_data_type"    => "",
			"t_data_title"   => "",
			"t_data_props"   => DAO::T_DATA_PROP_OTHERS,
			"t_data_comment" => "",
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function get( $id = NULL, $state = NULL, $public = NULL, $projectID = NULL, $isProject = TRUE, $prop = NULL, $organizationID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		// *フィールド
		$arrField = array(
			"t_data_id",
			"t_data_state",
			"t_data_public",
			"t_data_type",
			"t_data_title",
			"t_data_props",
			"t_data_comment",
			"t_data_register",
			"t_data_update",
			"t_user_id",
			"t_project_id",
			"get_project_name(t_project_id) AS t_project_name",
			"get_project_organization_id(t_project_id) AS t_organization_id",
			"get_organization_name(get_project_organization_id(t_project_id)) AS t_organization_name",
		);
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_data_id"     => $id,
			"t_data_state"  => $state,
			"t_data_public" => $public,
			"t_data_props"  => $prop,
		));
		if ( $public === DAO::T_DATA_PUBLIC_PRJ ){
			$arrCondition["t_project_id"] = $projectID;
		} else if ( $public === DAO::T_DATA_PUBLIC_ORG ){
			$arrCondition["get_project_organization_id(t_project_id)"] = $organizationID;
			if ( ! is_null($projectID) ){
				if ( self::storedFunction("get_project_organization_id", $projectID) !== $organizationID ){
					// 絞り込む組織IDがプロジェクトの組織でない場合はNO_DATAを返す
					$arrCondition["false"] = array("type" => self::TYPE_SQL, "value" => "FALSE");
				}
			}
		} else if ( Validate::isArray($public) ){
			$arrOrSql = array();
			$arrSettedPublic = array();
			if ( Validate::inArray(DAO::T_DATA_PUBLIC_PRJ, $public) ){
				$escProjectID = Util::esc($projectID);
				$arrOrSql[] = "(t_data_public = ".DAO::T_DATA_PUBLIC_PRJ." AND t_project_id = {$escProjectID})";
				$arrSettedPublic[] = DAO::T_DATA_PUBLIC_PRJ;
			}
			if ( Validate::inArray(DAO::T_DATA_PUBLIC_ORG, $public) ){
				if ( ! is_null($projectID) && self::storedFunction("get_project_organization_id", $projectID) !== $organizationID ){
					// 絞り込む組織IDがプロジェクトの組織でない場合はNO_DATAを返す
					$arrCondition["false"] = array("type" => self::TYPE_SQL, "value" => "FALSE");
				} else {
					$escOrgID = Util::esc($organizationID);
					$arrOrSql[] = "(t_data_public = ".DAO::T_DATA_PUBLIC_ORG." AND get_project_organization_id(t_project_id) = {$escOrgID})";
				}
				$arrSettedPublic[] = DAO::T_DATA_PUBLIC_ORG;
			}
			$arrPublic = array_diff($public, $arrSettedPublic);
			if ( count($arrPublic) > 0 ){
				$arrOrSql[] = "(t_data_public IN (".implode(",", $arrPublic)."))";
			}
			// 上書きする
			$arrCondition["t_data_public"] = array("type"  => DAO::TYPE_SQL, "value" => implode(" OR ", $arrOrSql));
		}
		if ( ! is_null($projectID) ){
			$escProjectID = $this->esc($projectID);
			$arrCondition["t_project_data"] = array("type" => "SQL", "value" => "is_project_data({$escProjectID}, t_data_id) = " . ($isProject ? "TRUE" : "FALSE"));
		}
		// *並び順
		$arrOrder = $this->getOrderBy($arrOrder, array(
			"t_data_props" => array_keys(DAO::T_DATA_PROP_TABLE()),
			"t_data_public" => array_keys(DAO::T_DATA_PUBLIC_TABLE()),
			"t_project_id" => DAO::DESC,
			"t_data_id"    => DAO::DESC,
		));
		// *実行
		$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:exists
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function exists( $title, $excludeID = NULL ){
		$escTitle = Util::esc($title);
		
		$arrCondition = array();
		$arrCondition["t_data_title"] = array(
			"type"  => DAO::TYPE_SQL,
			"value" => "t_data_title LIKE '{$escTitle}'",
		);
		$arrCondition["t_data_state"] = DAO::T_DATA_STATE_NORMAL;
		if ( Validate::isNumeric($excludeID) && $excludeID !== INIT_ID ){
			$arrCondition["t_data_id"] = array(
				"type"  => DAO::TYPE_NOT,
				"value" => $excludeID,
			);
		}
		
		$arrField = array("t_data_id");
		$resCheck = $this->_select( $arrField, $arrCondition );
		if ( $resCheck["result"] === RES_OK ){
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $id, $state = NULL, $public = NULL, $type = NULL, $title = NULL, $prop = NULL, $comment = NULL, $projectID = NULL, $userID = NULL ){
		$res    = NULL;
		
		// 登録できるかチェックする
		// （名前重複チェック）
		if ( ! is_null($title) ){
			if ( $this->exists($title, $id) ){
				$res["result"] = RES_NG;
				$res["error"]  = self::ERROR_TITLE_DUPLICATE;
				
				return $res;
			}
		}
		
		// 新規登録
		if ( $id === INIT_ID ){
			// *フィールド
			$arrValue = array(
				"t_data_state"    => DAO::T_DATA_STATE_NORMAL,
				"t_data_public"   => $public,
				"t_data_type"     => $type,
				"t_data_title"    => $title,
				"t_data_props"    => $prop,
				"t_data_comment"  => $comment,
				"t_data_register" => DAO::NOW,
				"t_data_update"   => DAO::NOW,
				"t_user_id"       => $userID,
				"t_project_id"    => $projectID,
			);
			// *実行
			$res = $this->_insert($arrValue, TRUE);
		}
		// 更新
		else {
			// *フィールド
			$arrValue = $this->getValueUpdate(array(
				"t_data_update"  => DAO::NOW,
				"t_data_state"   => $state,
				"t_data_public"  => $public,
				"t_data_type"    => $type,
				"t_data_title"   => $title,
				"t_data_props"   => $prop,
				"t_data_comment" => $comment,
				"t_user_id"      => $userID,
			));
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_data_id" => $id,
			));
			// *実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $id;
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:disabled
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function disabled( $id ){
		$res = $this->register($id, DAO::T_DATA_STATE_DELETE);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:delete
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function delete( $id, $projectID = NULL, $organizationID = NULL ){
		$res    = NULL;
		
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_data_id"    => $id,
			"t_project_id" => $projectID,
		));
		if ( ! is_null($organizationID) ){
			$escOrganizationID = $this->esc($organizationID);
			$arrCondition["t_organization_id"] = array("type" => "SQL", "value" => "get_project_organization_id(t_project_id) != {$escOrganizationID}");
		}
		
		// *削除予定IDを取得
		if ( ($arrDataID = $this->getPrimaryKey("t_data_id", $arrCondition)) === FALSE ){
			$res["result"] = RES_NG;
			return $res;
		} else if ( count($arrDataID) === 0 ){
			$res["result"] = RES_OK;
			return $res;
		}
		
		// *実行
		$res = $this->_delete($arrCondition);
		
		if ( $res["result"] === RES_OK ){
			//ProjectDataの削除
			$objDAOProjectData = DAO::instance("project_data");
			$res = $objDAOProjectData->delete(NULL, $arrDataID);
		}
		if ( $res["result"] === RES_OK ){
			foreach ( $arrDataID as $deleteID ){
				$file = Setting::get("PATH_DATA_DATA") . "/{$deleteID}";
				if ( file_exists($file) ){
					unlink($file);
				}
			}
		}
		
		return $res;
	}
}
