<?php
Class DAO_document extends DAO {
	const ERROR_TITLE_DUPLICATE = 101;
	
	const ORDER_KEY_TITLE = "title";
	const ORDER_KEY_LANG  = "lang";
	public static function ORDER_KEY_SQL_TABLE( $key = NULL ){
		$arrTable = array(
			self::ORDER_KEY_TITLE => "t_document_title",
			self::ORDER_KEY_LANG  => "t_document_lang_s",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "t_user" ){
		$this->tableName = $_tableName;
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_document_id"      => DAO::ID_INIT,
			"t_document_state"   => DAO::T_DOCUMENT_STATE_NORMAL,
			"t_document_lang_s"  => LANG_S_INIT,
			"t_document_lang_t"  => LANG_T_INIT,
			"t_document_title"   => "",
			"t_document_text"    => "",
			"t_document_url"     => "",
			"t_document_comment" => "",
			"t_user_id"          => DAO::ID_INIT,
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function get( $id = NULL, $state = NULL, $title = NULL, $projectID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		// *フィールド
		$arrField = array(
			"t_document_id",
			"t_document_state",
			"t_document_lang_s",
			"t_document_lang_t",
			"t_document_title",
			"t_document_text",
			"t_document_url",
			"t_document_comment",
			"t_document_register",
			"t_document_update",
			"t_project_id",
			"t_user_id",
		);
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_document_id"    => $id,
			"t_document_state" => $state,
			"t_document_title" => array("type" => DAO::TYPE_LIKE, "value"=>Util::esc($title)),
			"t_project_id"     => $projectID,
		));
		// *並び順
		$arrOrder = $this->getOrderBy($arrOrder, array(
			"t_document_id" => DAO::DESC,
		));
		
		// *実行
		$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:exists
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function exists( $title, $projectID, $excludeID = NULL ){
		$escTitle = Util::esc($title);
		
		$arrCondition = array();
		$arrCondition["t_document_title"] = array(
			"type"  => DAO::TYPE_SQL,
			"value" => "t_document_title LIKE '{$escTitle}'",
		);
		$arrCondition["t_document_state"] = array(
			"type"  => DAO::TYPE_IN,
			"value" => array_keys(DAO::T_DOCUMENT_STATE_VALID_TABLE()),
		);
		if ( Validate::isNumeric($projectID) ){
			$arrCondition["t_project_id"] = $projectID;
		}
		if ( Validate::isNumeric($excludeID) && $excludeID !== INIT_ID ){
			$arrCondition["t_document_id"] = array(
				"type"  => DAO::TYPE_NOT,
				"value" => $excludeID,
			);
		}
		
		$arrField = array("t_document_id");
		$resCheck = $this->_select( $arrField, $arrCondition );
		if ( $resCheck["result"] === RES_OK ){
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $id, $state = NULL, $langS = NULL, $langT = NULL, $title = NULL, $text = NULL, $comment = NULL, $projectID = NULL, $userID = NULL ){
		$res    = NULL;
		
		// 登録できるかチェックする
		// （タイトル重複チェック）
		if ( ! is_null($title) ){
			if ( $this->exists($title, $projectID, $id) ){
				$res["result"] = RES_NG;
				$res["error"]  = self::ERROR_TITLE_DUPLICATE;
				
				return $res;
			}
		}
		
		// 新規登録
		if ( $id === INIT_ID ){
			// *フィールド
			$arrValue = array(
				"t_document_state"    => DAO::T_DOCUMENT_STATE_NORMAL,
				"t_document_lang_s"   => $langS,
				"t_document_lang_t"   => $langT,
				"t_document_title"    => $title,
				"t_document_text"     => $text,
				"t_document_url"      => "",
				"t_document_comment"  => $comment,
				"t_document_register" => DAO::NOW,
				"t_document_update"   => DAO::NOW,
				"t_project_id"        => $projectID,
				"t_user_id"           => $userID,
			);
			// *実行
			$res = $this->_insert($arrValue, TRUE);
		}
		// 更新
		else {
			// *フィールド
			$arrValue = $this->getValueUpdate(array(
				"t_document_update"  => DAO::NOW,
				"t_document_state"   => $state,
				"t_document_lang_s"  => $langS,
				"t_document_lang_t"  => $langT,
				"t_document_title"   => $title,
				"t_document_text"    => $text,
				"t_document_comment" => $comment,
			));
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_document_id" => $id,
			));
			// *実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $id;
		}
		
		if ( $res["result"] === RES_OK ){
			if ( ! is_null($text) ){
				$this->registerDocumentText($res["id"], $text);
			}
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:registerText
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function registerDocumentText($id, $text){
		$res = NULL;
		$result = RES_OK;
		
		// 新規登録
		if ( $id === INIT_ID ){
			$result = RES_NG;
			$res["result"] = $result;
		} else {
			$objDAODocumentText = DAO::instance("document_text");
			
			// t_document_text の削除
			$objDAODocumentText->delete( NULL, $id );
			
			$objDocTmx = new DocTmx();
			if ( $objDocTmx->load($text) ){
				$arr = array();
				if ( ($arrSentence = $objDocTmx->getSentence()) !== FALSE ){
					foreach ( $arrSentence as $sentence ){
						$arr[] = array(
							0 => $sentence["source_text"],
							1 => $sentence["target_text"],
						);
					}
				}
				if ( count($arr) > 0 ){
					$resDAODocumentText = $objDAODocumentText->registerMulti($arr, $id);
					if ( $resDAODocumentText["result"] === RES_OK ){
						
					} else if ( $resDAODocumentText["result"] === RES_NG ){
						$result = RES_NG;
					}
				}
			} else {
				$result = RES_NG;
			}
			$res["result"] = $result;
			$res["id"] = $id;
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:disabled
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	function disabled( $documentID ){
		$res = $this->register($documentID, DAO::T_DOCUMENT_STATE_DELETE);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:delete
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function delete( $id, $projectID = NULL ){
		$res    = NULL;
		
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_document_id" => $id,
			"t_project_id"  => $projectID,
		));
		
		// *削除予定IDを取得
		if ( ($arrDocumentID = $this->getPrimaryKey("t_document_id", $arrCondition)) === FALSE ){
			$res["result"] = RES_NG;
			return $res;
		} else if ( count($arrDocumentID) === 0 ){
			$res["result"] = RES_OK;
			return $res;
		}
		
		// *実行
		$res = $this->_delete($arrCondition);
		
		if ( $res["result"] === RES_OK ){
			// DocumentTextの削除
			$objDAODocumentText = DAO::instance("document_text");
			$res = $objDAODocumentText->delete(NULL, $arrDocumentID);
		}
		if ( $res["result"] === RES_OK ){
			//TaskDocumentの削除
			$objDAOTaskDocument = DAO::instance("task_document");
			$res = $objDAOTaskDocument->delete(NULL, NULL, $arrDocumentID);
		}
		if ( $res["result"] === RES_OK ){
			//Document-Lookupの削除（ログは残したまま、documentIDを0にする）
			$objDAOLogLookup = DAO::instance("log_lookup");
			$res = $objDAOLogLookup->deleteDocumentLink($arrDocumentID);
		}
		if ( $res["result"] === RES_OK ){
			//Document-LookupSelectの削除（ログは残したまま、documentIDを0にする）
			$objDAOLogLookupSelect = DAO::instance("log_lookup_select");
			$res = $objDAOLogLookupSelect->deleteDocumentLink($arrDocumentID);
		}
		
		return $res;
	}
}
