<?php
Class DAO_project extends DAO {
	var $daoLang = NULL;
	
	const ORDER_KEY_NAME     = "name";
	const ORDER_KEY_STATE    = "state";
	const ORDER_KEY_REGISTER = "register";
	public static function ORDER_KEY_SQL_TABLE( $key = NULL ){
		$arrTable = array(
			self::ORDER_KEY_NAME     => "t_project_name",
			self::ORDER_KEY_STATE    => "t_project_state",
			self::ORDER_KEY_REGISTER => "t_project_register",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "t_project" ){
		$this->tableName = $_tableName;
		
		$this->daoLang = DAO::instance("project_lang");
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_project_id"     => DAO::ID_INIT,
			"t_project_state"  => DAO::T_PROJECT_STATE_NORMAL,
			"t_project_lang_s" => $this->daoLang->getInit("t_project_lang_s"),
			"t_project_lang_t" => $this->daoLang->getInit("t_project_lang_t"),
			"t_project_name"   => "",
			"t_project_text"   => "",
			"t_project_start"  => NULL,
			"t_project_end"    => NULL,
			"t_project_qredit_dictionary" => str_repeat("-", 63),
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function get( $projectID = NULL, $projectName = NULL, $userID = NULL, $orgID = NULL, $state = NULL, $memberUserID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		// *フィールド
		$arrField = array(
			"t_project_id",
			"t_project_state",
			"t_project_name",
			"t_project_start",
			"t_project_end",
			"t_project_text",
			"t_project_qredit_dictionary",
			"t_project_register",
			"t_project_update",
			"t_user_id",
			"get_user_name(t_user_id) AS t_user_name",
			"t_organization_id",
			"get_organization_name(t_organization_id) AS t_organization_name",
		);
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_project_id"      => $projectID,
			"t_project_name"    => $projectName,
			"t_user_id"         => $userID,
			"t_organization_id" => $orgID,
			"t_project_state"   => $state,
		));
		if ( ! is_null($memberUserID) && is_numeric($memberUserID) ){
			$arrCondition["is_project_member"] = array(
				"type"  =>  DAO::TYPE_SQL,
				"value" => "is_project_member(t_project_id, {$memberUserID})",
			);
		}
		// *並び順
		$arrOrder = $this->getOrderBy($arrOrder, array(
			"t_organization_name" => DAO::ASC,
			"t_project_start"     => DAO::DESC,
			"t_project_id"        => DAO::DESC,
		));
		// *実行
		$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		if ( $res["result"] === RES_OK ){
			foreach ( $res["data"] as &$data ){
				$projectID = $data["t_project_id"];
				
				$resDAOLang = $this->daoLang->get($projectID);
				if ( $resDAOLang["result"] === RES_OK ){
					foreach ( $resDAOLang["data"] as $dataLang ){
						$data["t_project_lang_s"] = $dataLang["t_project_lang_s"];
						$data["t_project_lang_t"][] = $dataLang["t_project_lang_t"];
					}
				} else {
					$data["t_project_lang_s"] = "";
					$data["t_project_lang_t"] = array();
				}
			}
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $projectID, $state = NULL, $langS = NULL, $arrLangT = NULL, $name = NULL, $start = NULL, $end = NULL, $text = NULL, $dict = NULL, $userID = NULL, $organizationID = NULL ){
		$res    = NULL;
		
		// 新規登録
		if ( $projectID === INIT_ID ){
			// *フィールド
			$arrValue = array(
				"t_project_state"             => DAO::T_PROJECT_STATE_NORMAL,
				"t_project_name"              => $name,
				"t_project_start"             => $start,
				"t_project_end"               => $end,
				"t_project_text"              => $text,
				"t_project_qredit_dictionary" => $dict,
				"t_project_register"          => DAO::NOW,
				"t_project_update"            => DAO::NOW,
				"t_user_id"                   => $userID,
				"t_organization_id"           => $organizationID,
			);
			// *実行
			$res = $this->_insert($arrValue, TRUE);
		}
		// 更新
		else {
			// *フィールド
			$arrValue = $this->getValueUpdate(array(
				"t_project_update" => DAO::NOW,
				"t_project_state"  => $state,
				"t_project_name"   => $name,
				"t_project_start"  => $start,
				"t_project_end"    => $end,
				"t_project_text"   => $text,
				"t_project_qredit_dictionary" => $dict,
			));
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_project_id" => $projectID,
			));
			// *実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $projectID;
		}
		
		if ( $res["result"] === RES_OK ){
			if ( ! is_null($langS) && ! is_null($arrLangT) ){
				$projectID = $res["id"];
				
				// 一度削除
				$resDAOLang = $this->daoLang->delete($projectID);
				
				if ( $resDAOLang["result"] === RES_OK ){
					foreach ( $arrLangT as $langT ){
						$resDAOLang = $this->daoLang->register($projectID, $langS, $langT);
						
						if ( $resDAOLang["result"] !== RES_OK ){
							$res["result"] = RES_NG;
							break;
						}
					}
				}
			}
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:delete
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function delete( $projectID ){
		$res    = NULL;
		
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_project_id" => $projectID,
		));
			
		// *実行
		$res = $this->_delete($arrCondition);
		
		// 連携（結合）テーブルを削除
		if ( $res["result"] === RES_OK ){
			$res = $this->daoLang->delete($projectID);
		}
		// 連携（結合）テーブルを削除
		if ( $res["result"] === RES_OK ){
			//参加者(ProjectUser)の削除
			$objDAOProjectUser = DAO::instance("project_user");
			$res = $objDAOProjectUser->delete($projectID);
		}
		// 連携（結合）テーブルを削除
		if ( $res["result"] === RES_OK ){
			//文書(Document)の削除
			$objDAODocument = DAO::instance("document");
			$res = $objDAODocument->delete(NULL, $projectID);
		}
		// 連携（結合）テーブルを削除
		if ( $res["result"] === RES_OK ){
			//ワークフロー(Workflow)の削除
			$objDAOWorkflow = DAO::instance("workflow");
			$res = $objDAOWorkflow->delete(NULL, $projectID);
		}
		// 連携（結合）テーブルを削除
		if ( $res["result"] === RES_OK ){
			//用語集(TermRoot)の削除
			$objDAOTermRoot = DAO::instance("term_root", Term::KIND_TERM);
			$res = $objDAOTermRoot->delete(NULL, $projectID);
		}
		if ( $res["result"] === RES_OK ){
			//翻訳メモリ集(BilingualRoot)の削除
			$objDAOBilingualRoot = DAO::instance("term_root", Term::KIND_BILINGUAL);
			$res = $objDAOBilingualRoot->delete(NULL, $projectID);
		}
		if ( $res["result"] === RES_OK ){
			//データ(Data)の削除
			$objDAOData = DAO::instance("data");
			$res = $objDAOData->delete(NULL, $projectID);
		}
		if ( $res["result"] === RES_OK ){
			//掲示板(Bbs)の削除
			$objDAOBbs = DAO::instance("bbs");
			$res = $objDAOBbs->delete(NULL, $projectID);
		}
		if ( $res["result"] === RES_OK ){
			//Project-Lookupの削除（ログは残したまま、projectIDを0にする）
			$objDAOLogLookup = DAO::instance("log_lookup");
			$res = $objDAOLogLookup->deleteProjectLink($projectID);
		}
		if ( $res["result"] === RES_OK ){
			//Project-LookupSelectの削除（ログは残したまま、projectIDを0にする）
			$objDAOLogLookupSelect = DAO::instance("log_lookup_select");
			$res = $objDAOLogLookupSelect->deleteProjectLink($projectID);
		}
		
		return $res;
	}

}
