<?php
Class DAO_statistics extends DAO {
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "" ){
		$this->tableName = $_tableName;
	}
	
/************************************************************************************************************************************
 * タスク
 ************************************************************************************************************************************/
	/***************************************************************
	 * 名前:getTaskCustomStatistic
	 * 説明:集計をする
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getTaskCustomStatistic( $hasUser, $hasProject, $hasDocument,
	                                         $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$arrTable = array(
			"t_task",
			"t_task_actor"    => array("type"=>DAO::TABLE_TYPE_LEFT, "table"=>"t_task_actor",    "column"=>"t_task.t_task_id=t_task_actor.t_task_id"),
		);
		$arrField = array(
			"t_task.t_task_id",
			"t_task.t_task_state",
			"t_task.t_task_type",
			"t_task.t_task_start",
			"t_task.t_task_end",
			"t_task_actor.t_task_role",
		);
		
		if ( $hasUser ){
			$arrField = array_merge($arrField, array(
				"(t_task_actor.t_user_id) as t_user_id",
				"get_user_name(t_task_actor.t_user_id) as t_user_name",
				"get_user_type(t_task_actor.t_user_id) as t_user_type",
			));
		}
		if ( $hasProject ){
			$arrField = array_merge($arrField, array(
				"get_project_id_by_workflow_id(t_task.t_workflow_id) as t_project_id",
				"get_project_name_by_workflow_id(t_task.t_workflow_id) as t_project_name"
			));
		}
		if ( $hasDocument ){
			$arrTable += array(
				"t_task_document" => array("type"=>DAO::TABLE_TYPE_LEFT, "table"=>"t_task_document", "column"=>"t_task.t_task_id=t_task_document.t_task_id"),
			);
			$arrField = array_merge($arrField, array(
				"t_task_document.t_document_id",
				"get_document_title(t_task_document.t_document_id) as t_document_title"
			));
		}
		
		$arrCondition = $this->getCondition(array(
			"t_task.t_task_type"       => $taskType,
			"t_task_actor.t_task_role" => $taskRole,
			"t_task_actor.t_user_id"   => $userID,
			"get_project_id_by_workflow_id(t_task.t_workflow_id)" => $projectID,
			"get_organization_id_by_user_id(get_user_id_by_project_id(get_project_id_by_workflow_id(t_task.t_workflow_id)))" => $orgID,
		));
		if ( ! is_null($documentID) ){
			$arrTable += array(
				"t_task_document" => array("type"=>DAO::TABLE_TYPE_LEFT, "table"=>"t_task_document", "column"=>"t_task.t_task_id=t_task_document.t_task_id"),
			);
			$arrCondition["t_task_document.t_document_id"] = $documentID;
		}
		
		$objDAOTask = new DAO( $arrTable );
		$res = $objDAOTask->_select($arrField, $arrCondition, NULL, NULL, NULL, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:getTaskStatisticByProject
	 * 説明:プロジェクトごとの集計をする
	 * 引数:[必須] $projectID
	 * 返値:
	****************************************************************/
	function getTaskStatisticByProject( $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $limit = NULL, $offset = NULL ){
		return $this->getTaskCustomStatistic( TRUE, FALSE, TRUE, $taskType, $taskRole, $userID, $projectID, $documentID, NULL, $limit, $offset );
	}
	
	/***************************************************************
	 * 名前:getTaskStatisticByUser
	 * 説明:ユーザーごとの集計をする
	 * 引数:[必須] $userID
	 * 返値:
	****************************************************************/
	function getTaskStatisticByUser( $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $limit = NULL, $offset = NULL ){
		return $this->getTaskCustomStatistic( FALSE, TRUE, TRUE, $taskType, $taskRole, $userID, $projectID, $documentID, NULL, $limit, $offset );
	}
	
	/***************************************************************
	 * 名前:getTaskStatisticByOrganization
	 * 説明:組織ごとの集計をする
	 * 引数:
	 * 返値:
	****************************************************************/
	function getTaskStatisticByOrganization( $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		return $this->getTaskCustomStatistic( TRUE, TRUE, TRUE, $taskType, $taskRole, $userID, $projectID, $documentID, $orgID, $limit, $offset );
	}
	
	
/************************************************************************************************************************************
 * 辞書引き
 ************************************************************************************************************************************/
	/***************************************************************
	 * 名前:getLookupCustomStatistic
	 * 説明:集計をする
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getLookupCustomStatistic( $hasUser, $hasProject, $hasDocument,
	                                           $text = NULL, $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$arrTable = array(
			"t_log_lookup",
		);
		$arrField = array(
			"t_log_lookup_date",
			"t_log_lookup_no",
			"t_log_lookup_lang_s",
			"t_log_lookup_lang_t",
			"t_log_lookup_index",
			"t_log_lookup_text",
			"t_log_lookup_pos",
			"t_log_lookup_dict_id",
			"t_log_lookup_dict_ids",
			"t_log_lookup_awareness",
			"t_task_id",
			"t_task_type",
			"t_task_role",
		);
		if ( $hasUser ){
			$arrField = array_merge($arrField, array(
				"t_log_lookup.t_user_id as t_user_id",
				"get_user_name(t_log_lookup.t_user_id) as t_user_name",
				"get_user_type(t_log_lookup.t_user_id) as t_user_type",
			));
		}
		if ( $hasProject ){
			$arrField = array_merge($arrField, array(
				"t_log_lookup.t_project_id as t_project_id",
				"get_project_name(t_log_lookup.t_project_id) as t_project_name",
			));
		}
		if ( $hasDocument ){
			$arrField = array_merge($arrField, array(
				"t_log_lookup.t_document_id as t_document_id",
				"get_document_title(t_log_lookup.t_document_id) as t_document_title",
			));
		}
		$arrCondition = $this->getCondition(array(
			"t_task_type" => $taskType,
			"t_task_role" => $taskRole,
			"t_log_lookup.t_user_id"     => $userID,
			"t_log_lookup.t_project_id"  => $projectID,
			"t_log_lookup.t_document_id" => $documentID,
			"get_organization_id_by_user_id(t_log_lookup.t_user_id)" => $orgID,
		));
		if ( ! is_null($text) ){
			$escText = $this->esc($text);
			$arrCondition["t_log_lookup_text"] = array("type"=>DAO::TYPE_SQL, "value"=>"t_log_lookup_text LIKE '{$escText}'");
		}
		
		$objDAOTask = new DAO( $arrTable );
		$res = $objDAOTask->_select($arrField, $arrCondition, NULL, NULL, NULL, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:getLookupStatisticByUser
	 * 説明:ユーザーごとの集計をする
	 * 引数:[必須] $userID
	 * 返値:
	****************************************************************/
	function getLookupStatisticByUser( $text = NULL, $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $limit = NULL, $offset = NULL ){
		return $this->getLookupCustomStatistic( FALSE, TRUE, TRUE, $text, $taskType, $taskRole, $userID, $projectID, $documentID, NULL, $limit, $offset );
	}
	
	/***************************************************************
	 * 名前:getLookupStatisticByOrganization
	 * 説明:組織ごとの集計をする
	 * 引数:
	 * 返値:
	****************************************************************/
	function getLookupStatisticByOrganization( $text = NULL, $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		return $this->getLookupCustomStatistic( TRUE, TRUE, TRUE, $text, $taskType, $taskRole, $userID, $projectID, $documentID, $orgID, $limit, $offset );
	}
	
	
/************************************************************************************************************************************
 * 選択辞書引き
 ************************************************************************************************************************************/
	private function getSelLookupCustomStatistic( $hasUser, $hasProject, $hasDocument,
	                                              $text = NULL, $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$arrTable = array(
			"t_log_lookup_select"
		);
		$arrField = array(
			"t_log_lookup_select_date",
			"t_log_lookup_select_mode",
			"t_log_lookup_select_no",
			"t_log_lookup_select_lang_s",
			"t_log_lookup_select_lang_t",
			"t_log_lookup_select_text_s",
			"t_log_lookup_select_text_t",
			"t_log_lookup_select_heading",
			"t_log_lookup_select_kind",
			"t_log_lookup_select_search_lang_s",
			"t_log_lookup_select_search_lang_t",
			"t_task_id",
			"t_task_type",
			"t_task_role",
		);
		if ( $hasUser ){
			$arrField = array_merge($arrField, array(
				"t_user_id",
				"get_user_name(t_user_id) AS t_user_name",
				"get_user_type(t_user_id) as t_user_type",
			));
		}
		if ( $hasProject ){
			$arrField = array_merge($arrField, array(
				"t_project_id",
				"get_project_name(t_project_id) AS t_project_name"
			));
		}
		if ( $hasDocument ){
			$arrField = array_merge($arrField, array(
				"t_document_id",
				"get_document_title(t_document_id) AS t_document_title"
			));
		}
		$arrCondition = $this->getCondition(array(
			"t_task_type" => $taskType,
			"t_task_role" => $taskRole,
			"t_log_lookup_select.t_user_id"     => $userID,
			"t_log_lookup_select.t_project_id"  => $projectID,
			"t_log_lookup_select.t_document_id" => $documentID,
			"get_organization_id_by_user_id(t_log_lookup_select.t_user_id)" => $orgID,
		));
		if ( ! is_null($text) ){
			$escText = $this->esc($text);
			$arrCondition["t_log_lookup_text"] = array("type"=>DAO::TYPE_SQL, "value"=>"t_log_lookup_select_text_s LIKE '{$escText}'");
		}
		
		$objDAOTask = new DAO( $arrTable );
		$res = $objDAOTask->_select($arrField, $arrCondition, NULL, NULL, NULL, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:getSelLookupStatisticByUser
	 * 説明:ユーザーごとの集計をする
	 * 引数:[必須] $userID
	 * 返値:
	****************************************************************/
	function getSelLookupStatisticByUser( $text = NULL, $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $limit = NULL , $offset = NULL ){
		return $this->getSelLookupCustomStatistic( FALSE, TRUE, TRUE, $text, $taskType, $taskRole, $userID, $projectID, $documentID, NULL, $limit, $offset );
	}
	
	/***************************************************************
	 * 名前:getSelLookupStatisticByOrganization
	 * 説明:組織ごとの集計をする
	 * 引数:
	 * 返値:
	****************************************************************/
	function getSelLookupStatisticByOrganization( $text = NULL, $taskType = NULL, $taskRole = NULL, $userID = NULL, $projectID = NULL, $documentID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		return $this->getSelLookupCustomStatistic( TRUE, TRUE, TRUE, $text, $taskType, $taskRole, $userID, $projectID, $documentID, $orgID, $limit, $offset );
	}
	
	
/************************************************************************************************************************************
 * 掲示板
 ************************************************************************************************************************************/
	private function getBbsPostCustomStatistic( $hasUser, $hasProject, $hasTarget, $projectID = NULL, $bbsPostStage = NULL, $bbsPostType = NULL, $bbsPostRole = NULL, $targetUserID = NULL, $targetUserRole = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$arrTable = array(
			"t_bbs_post"
		);
		$arrField = array(
			"t_bbs_post.t_bbs_post_no",
			"t_bbs_post.t_bbs_post_parent_no",
			"t_bbs_post.t_bbs_post_state",
			"t_bbs_post.t_bbs_post_stage",
			"t_bbs_post.t_bbs_post_type",
			"t_bbs_post.t_bbs_post_role",
			"t_bbs_post.t_bbs_post_object",
			"t_bbs_post.t_bbs_post_title",
			"t_bbs_post.t_bbs_post_text",
			"t_bbs_post.t_bbs_post_register",
			"t_bbs_post.t_bbs_id",
		);
		if ( $hasUser ){
			$arrField = array_merge($arrField, array(
				"t_bbs_post.t_user_id",
				"get_user_name(t_bbs_post.t_user_id) AS t_user_name",
				"get_user_type(t_bbs_post.t_user_id) as t_user_type",
			));
		}
		if ( $hasProject ){
			$arrField = array_merge($arrField, array(
				"get_project_id_by_bbs_id(t_bbs_post.t_bbs_id) AS t_bbs_id",
				"get_project_name_by_bbs_id(t_bbs_post.t_bbs_id) AS t_project_name",
			));
		}
		if ( $hasTarget ){
			$arrTable += array(
				"t_bbs_post_target" => array("type"=>DAO::TABLE_TYPE_LEFT, "table"=>"t_bbs_post_target", "column"=>"t_bbs_post.t_bbs_id=t_bbs_post_target.t_bbs_id AND t_bbs_post.t_bbs_post_no=t_bbs_post_target.t_bbs_post_no"),
			);
			$arrField = array_merge($arrField, array(
				"t_bbs_post_target.t_bbs_post_target_role",
				"t_bbs_post_target.t_user_id AS t_bbs_post_target_user_id",
				"get_user_name(t_bbs_post_target.t_user_id) AS t_bbs_post_target_user_name",
				"get_user_type(t_bbs_post_target.t_user_id) as t_bbs_post_target_user_type",
			));
		}
		$arrCondition = $this->getCondition(array(
			"t_bbs_post.t_bbs_post_state" => DAO::T_BBS_POST_STATE_NORMAL,
			"t_bbs_post.t_bbs_post_kind"  => DAO::T_BBS_POST_KIND_NORMAL,
			"t_bbs_post.t_bbs_post_stage" => $bbsPostStage,
			"t_bbs_post.t_bbs_post_type"  => $bbsPostType,
			"t_bbs_post.t_bbs_post_role"  => $bbsPostRole,
			"get_project_id_by_bbs_id(t_bbs_post.t_bbs_id)" => $projectID,
		));
		
		$objDAOBbsPost = new DAO( $arrTable );
		$res = $objDAOBbsPost->_select($arrField, $arrCondition, NULL, NULL, NULL, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:getBbsPostStatisticByProject
	 * 説明:プロジェクトごとの集計をする
	 * 引数:[必須] $projectID
	 * 返値:
	****************************************************************/
	function getBbsPostStatisticByProject( $projectID, $stage = NULL, $postType = NULL, $postRole = NULL, $limit = NULL, $offset = NULL ){
		return $this->getBbsPostCustomStatistic( TRUE, FALSE, FALSE, $projectID, $stage, $postType, $postRole, NULL, NULL, $limit, $offset);
	}
	
	/***************************************************************
	 * 名前:getBbsPostTargetStatisticByProject
	 * 説明:プロジェクトごとの集計をする
	 * 引数:[必須] $projectID
	 * 返値:
	****************************************************************/
	function getBbsPostTargetStatisticByProject( $projectID, $stage = NULL, $postType = NULL, $postRole = NULL, $limit = NULL, $offset = NULL ){
		return $this->getBbsPostCustomStatistic( TRUE, FALSE, TRUE, $projectID, $stage, $postType, $postRole, NULL, NULL, $limit, $offset);
	}
	
	
/************************************************************************************************************************************
 * その他
 ************************************************************************************************************************************/
	/***************************************************************
	 * 名前:getOrganizationInfo
	 * 説明:全組織の調査
	 * 引数:
	 * 返値:
	****************************************************************/
	function getOrganizationInfo( $orgID = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$arrTable = array(
			"t_organization",
			array("type"=>DAO::TABLE_TYPE_LEFT, "table"=>"t_user",    "column"=>"t_organization.t_organization_id=t_user.t_organization_id"),
			array("type"=>DAO::TABLE_TYPE_LEFT, "table"=>"t_project", "column"=>"t_organization.t_organization_id=get_organization_id_by_user_id(t_project.t_user_id)"),
		);
		$arrField = array(
			"t_organization.t_organization_id",
			"t_organization.t_organization_name",
			"COUNT(distinct t_user.t_user_id) AS t_user_count",
			"COUNT(distinct t_project.t_project_id) AS t_project_count",
		);
		$arrCondition = $this->getCondition(array(
			"t_organization.t_organization_id" => $orgID,
		));
		$arrGroupBy = array(
			"t_organization.t_organization_id",
		);
		
		$objDAOOrg = new DAO($arrTable);
		$res = $objDAOOrg->_select($arrField, $arrCondition, $arrGroupBy, NULL, NULL, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:getLoginInfo
	 * 説明:ログイン情報の調査
	 * 引数:
	 * 返値:
	****************************************************************/
	function getLoginInfo( $userID = NULL, $orgID = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$objDAOLogin = new DAO("t_login");
		$arrField = array(
			"t_login_state",
			"t_login_date",
			"t_user_id",
			"get_user_name(t_user_id) as t_user_name",
			"get_user_type(t_user_id) as t_user_type",
			"get_organization_id_by_user_id(t_user_id) as t_organization_id",
		);
		$arrCondition = $this->getCondition(array(
			"t_user_id"         => $userID,
			"t_organization_id" => $orgID,
			"t_login_state"     => DAO::T_LOGIN_STATE_NONE,
		));
		$arrOrder = array(
			"t_login_date" => DAO::DESC,
		);
		
		$res = $objDAOLogin->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		return $res;
	}
	
}
