<?php
Class DAO_task extends DAO {
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "t_task" ){
		$this->tableName = $_tableName;
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_task_id"     => DAO::ID_INIT,
			"t_task_state"  => DAO::T_TASK_STATE_NORMAL,
			"t_task_type"   => DAO::ID_INIT,
			"t_task_data"   => "",
			"t_task_start"  => "",
			"t_task_end"    => "",
			"t_workflow_id" => DAO::ID_INIT,
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function get( $taskID = NULL, $state = NULL, $type = NULL, $start = NULL, $end = NULL, $workflowID = NULL, $userID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		$result = RES_OK;
		
		if ( $result === RES_OK ){
			// *フィールド
			$arrField = array(
				"t_task_id",
				"t_task_state",
				"t_task_type",
				"t_task_data",
				"t_task_start",
				"t_task_end",
				"t_workflow_id",
				"get_project_id_by_workflow_id(t_workflow_id) AS t_project_id",
				"get_project_name_by_workflow_id(t_workflow_id) AS t_project_name",
			);
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_task_id"     => $taskID,
				"t_task_state"  => $state,
				"t_task_type"   => $type,
				"t_workflow_id" => $workflowID,
			));
			if ( ! is_null($start) ){
				$escStart = $this->esc($start);
				$arrCondition["t_task_start"] = array("type" => DAO::TYPE_SQL, "value" => "t_task_start <= '{$escStart}'");
			}
			if ( ! is_null($end) ){
				$escEnd = $this->esc($end);
				$arrCondition["t_task_end"] = array("type" => DAO::TYPE_SQL, "value" => "t_task_end >= '{$escEnd}'");
			}
			if ( ! is_null($userID) ){
				$escUserID = $this->esc($userID);
				$arrCondition["t_user_id"] = array("type" => DAO::TYPE_SQL, "value" => "is_task_user(t_task_id, {$escUserID})");
			}
			// *並び順
			if ( ! is_array($arrOrder) ){
				$arrOrder = array();
			}
			if ( is_array($type) ){
				$arrOrder[ DAO::SQL_ORDER_BY("t_task_type", $type) ] = DAO::ASC;
			}
			$arrOrder = $this->getOrderBy($arrOrder, array(
				"t_task_start" => DAO::ASC,
				"t_task_id"    => DAO::ASC,
			));
			// *実行
			$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $id, $state = NULL, $type = NULL, $data = NULL, $start = NULL, $end = NULL, $workflowID = NULL, $userID = NULL, $organizationID = NULL ){
		$res    = NULL;
		
		// 新規登録
		if ( $id === INIT_ID ){
			// *フィールド
			$arrValue = array(
				"t_task_state"  => DAO::T_TASK_STATE_NORMAL,
				"t_task_type"   => $type,
				"t_task_data"   => $data,
				"t_task_start"  => $start,
				"t_task_end"    => $end,
				"t_workflow_id" => $workflowID,
			);
			// *実行
			$res = $this->_insert($arrValue, TRUE);
		}
		// 更新
		else {
			// 更新前の情報を $dataTask に覚えておく
			$resDAOTask = $this->get($id);
			if ( $resDAOTask["result"] === RES_OK ){
				$dataTask = $resDAOTask["data"][0];
			} else {
				$res = array("result" => RES_NG);
				return $res;
			}
			
			// *フィールド
			$arrValue = $this->getValueUpdate(array(
				"t_task_state" => $state,
				"t_task_data"  => $data,
				"t_task_start" => $start,
				"t_task_end"   => $end,
			));
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_task_id" => $id,
			));
			// *実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $id;
			
			// stateが異なる場合のみ ログ保存
			if ( $res["result"] === RES_OK ){
				if ( ! is_null($state) && Validate::inKeyArray($state, DAO::T_TASK_STATE_TABLE()) && $state !== $dataTask["t_task_state"] ){
					$objDAOLogTask = DAO::instance("log_task");
					$resLogTask = $objDAOLogTask->register($state, $userID, $organizationID, $dataTask["t_project_id"], $id);
					if ( $resLogTask["result"] !== RES_OK ){
						$res["result"] = $resLogTask["result"];
					}
				}
			}
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:delete
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function delete( $id, $workflowID = NULL ){
		$res    = NULL;
		
		// *条件
		$arrCondition = $this->getCondition( array(
			"t_task_id"     => $id,
			"t_workflow_id" => $workflowID,
		));
		
		// *削除予定IDを取得
		if ( ($arrTaskID = $this->getPrimaryKey("t_task_id", $arrCondition)) === FALSE ){
			$res["result"] = RES_NG;
			return $res;
		} else if ( count($arrTaskID) === 0 ){
			$res["result"] = RES_OK;
			return $res;
		}
		
		// *実行
		$res = $this->_delete($arrCondition);
		
		// 連携（結合）テーブルを削除
		if ( $res["result"] === RES_OK ){
			//TaskActorの削除
			$objDAOTaskActor = DAO::instance("task_actor");
			$res = $objDAOTaskActor->delete($arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			// TaskDocumentの削除
			$objDAOTaskDocument = DAO::instance("task_document");
			$res = $objDAOTaskDocument->delete($arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			// TaskTermRootの削除
			$objDAOTaskTermRoot = DAO::instance("task_term_root", Term::KIND_TERM);
			$res = $objDAOTaskTermRoot->delete($arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			// TaskBilingualRootの削除
			$objDAOTaskBilingualRoot = DAO::instance("task_term_root", Term::KIND_BILINGUAL);
			$res = $objDAOTaskBilingualRoot->delete($arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			// TaskDataの削除
			$objDAOTaskData = DAO::instance("task_data");
			$res = $objDAOTaskData->delete($arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			// TaskLogの削除
			$objDAOLogTask = DAO::instance("log_task");
			$res = $objDAOLogTask->delete(NULL, NULL, $arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			//Task-Lookupの削除（ログは残したまま、taskIDを0にする）
			$objDAOLogLookup = DAO::instance("log_lookup");
			$res = $objDAOLogLookup->deleteTaskLink($arrTaskID);
		}
		if ( $res["result"] === RES_OK ){
			//Task-LookupSelectの削除（ログは残したまま、taskIDを0にする）
			$objDAOLogLookupSelect = DAO::instance("log_lookup_select");
			$res = $objDAOLogLookupSelect->deleteTaskLink($arrTaskID);
		}
		
		return $res;
	}

}
