<?php
Class DAO_task_actor extends DAO {
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "t_task_actor" ){
		$this->tableName = $_tableName;
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_task_id"         => DAO::ID_INIT,
			"t_task_role"       => DAO::ID_INIT,
			"t_task_actor_type" => DAO::T_TASK_ACTOR_TYPE_ACTOR,
			"t_user_id"         => DAO::ID_INIT,
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function get( $taskID, $userID = NULL, $role = NULL, $type = NULL, $workflowID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		// *フィールド
		$arrField = array(
			"t_task_id",
			"t_task_role",
			"get_task_state(t_task_id) AS t_task_state",
			"get_task_type(t_task_id) AS t_task_type",
			"t_task_actor_type",
			"t_user_id",
			"get_user_type(t_user_id) AS t_user_type",
			"get_user_name(t_user_id) AS t_user_name",
		);
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_task_id"         => $taskID,
			"t_user_id"         => $userID,
			"t_task_role"       => $role,
			"t_task_actor_type" => $type,
			"get_workflow_id_by_task_id(t_task_id)" => $workflowID,
		));
		// *並び順
		$arrOrder = $this->getOrderBy($arrOrder, array(
			"t_task_id" => DAO::ASC,
			"t_user_id" => DAO::ASC,
			"t_task_actor_type" => DAO::ASC,
		));
		// *実行
		$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:getSimpleTaskGroup
	 * 説明: t_task_idでグループ化したものを取得する
	 * 引数:
	 * 返値:
	****************************************************************/
	function getSimpleTaskGroup( $userID = NULL, $role = NULL, $type = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		// *フィールド
		$arrField = array(
			"t_task_id",
		);
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_user_id"         => $userID,
			"t_task_role"       => $role,
			"t_task_actor_type" => $type,
		));
		// *グループ
		$arrGroup = array(
			"t_task_id"
		);
		// *グループ条件
		$arrHaving = NULL;
		if ( is_array($userID) ){
			$arrHaving = array(
				"count(DISTINCT t_user_id) >= ".count($userID),
			);
		}
		if ( is_array($role) ){
			$arrHaving = array(
				"count(DISTINCT t_task_role) >= ".count($role),
			);
		}
		if ( is_array($type) ){
			$arrHaving = array(
				"count(DISTINCT t_task_actor_type) => ".count($type),
			);
		}
		// *実行
		$res = $this->_select($arrField, $arrCondition, $arrGroup, $arrHaving, NULL, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $taskID, $taskType, $arrActor ){
		$res    = NULL;
		
		// 一度削除する
		$res = $this->delete($taskID);
		
		$setting = Task::SETTING_TABLE($taskType);
		if ( isset($setting["actor"]) ){
			foreach ( $setting["actor"] as $actorSetting ){
				$arrActorUserType[$actorSetting["r"]] = DAO::T_TASK_ACTOR_TYPE_ACTOR;
			}
		}
		if ( isset($setting["contact"]) ){
			foreach ( $setting["contact"] as $actorSetting ){
				$arrActorUserType[$actorSetting["r"]] = DAO::T_TASK_ACTOR_TYPE_CONTACT;
			}
		}
		
		// 新規登録
		if ( $res["result"] === RES_OK ){
			foreach ( $arrActor as $taskRole => $arrUserID ){
				if ( isset($arrActorUserType[$taskRole]) ){
					$taskActorType = $arrActorUserType[$taskRole];
				} else {
					$taskActorType = DAO::T_TASK_ACTOR_TYPE_ACTOR;
				}
				
				foreach ( $arrUserID as $userID ){
					// *フィールド
					$arrValue = array(
						"t_task_id"         => $taskID,
						"t_task_role"       => $taskRole,
						"t_user_id"         => $userID,
						"t_task_actor_type" => $taskActorType,
					);
					// *実行
					$res = $this->_insert($arrValue, TRUE);
					if ( $res["result"] !== RES_OK ){
						break;
					}
				}
			}
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:delete
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function delete( $taskID, $actor = NULL, $userID = NULL, $taskActorType = NULL ){
		$res    = NULL;
		
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_task_id"         => $taskID,
			"t_task_role"       => $actor,
			"t_user_id"         => $userID,
			"t_task_actor_type" => $taskActorType,
		));
		
		// *実行
		$res = $this->_delete($arrCondition);
		
		return $res;
	}

}
