<?php
Class DAO_term_root extends DAO {
	const ERROR_NAME_DUPLICATE = 101;
	
	var $kind = NULL;
	
	const ORDER_KEY_PUBLIC   = "public";
	public static function ORDER_KEY_SQL_TABLE( $key = NULL ){
		$arrTable = array(
			self::ORDER_KEY_PUBLIC => "t_term_root_public",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_kind ){
		parent::__construct( "t_{$_kind}_root" );
		
		$this->kind = $_kind;
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_term_root_id"     => INIT_ID,
			"t_term_root_state"  => DAO::T_TERM_ROOT_STATE_NORMAL,
			"t_term_root_public" => DAO::T_TERM_ROOT_PUBLIC_PRJ,
			"t_term_root_name"   => "",
			"t_term_root_lang_s" => LANG_S_INIT,
			"t_term_root_lang_t" => LANG_T_INIT,
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:getDefaultName
	 * 説明: デフォルトの名前を取得。プロジェクト用の時だけ、プロジェクト名＋"_Glossary"が変える。
	 * 引数:
	 * 返値:
	****************************************************************/
	function getDefaultName( $projectID, $excludeID = NULL ){
		$projectName = self::storedFunction("get_project_name", $projectID);
		if ( $this->kind === Term::KIND_TERM ){
			$defaultName = "{$projectName}_Glossary";
		} else if ( $this->kind === Term::KIND_BILINGUAL ){
			$defaultName = "{$projectName}_TM";
		} else {
			return "";
		}
		$index = 1;
		$name = $defaultName;
		while ( $this->exists($name, $excludeID) ){
			$index++;
			$name = "{$defaultName} ({$index})";
		}
		return $name;
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数: $type = TRUE                       : 指定のプロジェクトで利用登録している。
	 *               FALSE(NULL)                : 利用登録していない。
	 *               DAO::T_TERM_ROOT_TYPE_***  : 指定のタイプ
	 * 返値:
	****************************************************************/
	function get( $termRootID = NULL, $state = NULL, $public = NULL, $update = NULL, $projectID = NULL, $type = NULL, $organizationID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL ){
		$res    = NULL;
		
		$arrCustomOrder = NULL;
		
		// *フィールド
		$arrField = array(
			"t_{$this->kind}_root_id AS t_term_root_id",
			"t_{$this->kind}_root_state AS t_term_root_state",
			"t_{$this->kind}_root_public AS t_term_root_public",
			"t_{$this->kind}_root_name AS t_term_root_name",
			"t_{$this->kind}_root_lang_s AS t_term_root_lang_s",
			"t_{$this->kind}_root_lang_t AS t_term_root_lang_t",
			"t_{$this->kind}_root_update AS t_term_root_update",
			"get_project_organization_id(t_project_id) AS t_organization_id",
			"get_organization_name(get_project_organization_id(t_project_id)) AS t_organization_name",
			"t_project_id",
			"get_project_name(t_project_id) AS t_project_name",
		);
		if ( $this->kind === Term::KIND_TERM ){
			$arrField[] = "get_exterm_state_by_term_root_id(t_{$this->kind}_root_id) AS t_exterm_state";
		} else {
			$arrField[] = "null AS t_exterm_state";
		}
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_{$this->kind}_root_id"     => $termRootID,
			"t_{$this->kind}_root_state"  => $state,
			"t_{$this->kind}_root_public" => $public,
		));
			
		if ( $public === DAO::T_TERM_ROOT_PUBLIC_PRJ ){
			$arrCondition["t_project_id"] = $projectID;
		} else if ( $public === DAO::T_TERM_ROOT_PUBLIC_ORG ){
			$arrCondition["get_project_organization_id(t_project_id)"] = $organizationID;
			if ( ! is_null($projectID ) && self::storedFunction("get_project_organization_id", $projectID) !== $organizationID ){
				// 絞り込む組織IDがプロジェクトの組織でない場合はNO_DATAを返す
				$arrCondition["false"] = array("type" => self::TYPE_SQL, "value" => "FALSE");
			}
		} else if ( Validate::isArray($public) ){
			$arrOrSql = array();
			$arrSettedPublic = array();
			if ( Validate::inArray(DAO::T_TERM_ROOT_PUBLIC_PRJ, $public) ){
				$escProjectID = Util::esc($projectID);
				$arrOrSql[] = "(t_{$this->kind}_root_public = ".DAO::T_TERM_ROOT_PUBLIC_PRJ." AND t_project_id = {$escProjectID})";
				$arrSettedPublic[] = DAO::T_TERM_ROOT_PUBLIC_PRJ;
			}
			if ( Validate::inArray(DAO::T_TERM_ROOT_PUBLIC_ORG, $public) ){
				if ( ! is_null($projectID) && self::storedFunction("get_project_organization_id", $projectID) !== $organizationID ){
					// 絞り込む組織IDがプロジェクトの組織でない場合はNO_DATAを返す
					$arrCondition["false"] = array("type" => self::TYPE_SQL, "value" => "FALSE");
				} else {
					$escOrgID = Util::esc($organizationID);
					$arrOrSql[] = "(t_{$this->kind}_root_public = ".DAO::T_TERM_ROOT_PUBLIC_ORG." AND get_project_organization_id(t_project_id) = {$escOrgID})";
				}
				$arrSettedPublic[] = DAO::T_TERM_ROOT_PUBLIC_ORG;
			}
			$arrPublic = array_diff($public, $arrSettedPublic);
			if ( count($arrPublic) > 0 ){
				$arrOrSql[] = "(t_{$this->kind}_root_public IN (".implode(",", $arrPublic)."))";
			}
			// 上書きする
			$arrCondition["t_{$this->kind}_root_public"] = array("type"  => DAO::TYPE_SQL, "value" => implode(" OR ", $arrOrSql));
		}
		if ( ! is_null($update) ){
			$escUpdate = $this->esc($update);
			$arrCondition["t_{$this->kind}_root_update"] = array("type" => "SQL", "value" => "t_{$this->kind}_root_update > DATE_SUB(NOW(), INTERVAL {$escUpdate} MINUTE)");
		}
		if ( ! is_null($projectID) && ! is_null($type) ){
			$escProjectID = $this->esc($projectID);
			//$arrField[] = "get_project_{$this->kind}_root_type({$escProjectID}, t_{$this->kind}_root_id) as t_project_term_root_type";
			//
			if ( $type === TRUE ){
				$arrCondition["t_project_{$this->kind}_root"] = array("type" => "SQL", "value" => "get_project_{$this->kind}_root_type({$escProjectID}, t_{$this->kind}_root_id) IS NOT NULL");
				$arrCustomOrder = array(
					"get_project_{$this->kind}_root_type({$escProjectID}, t_{$this->kind}_root_id)" => DAO::ASC,
				);
			} else if ( $type === FALSE ){
				$arrCondition["t_project_{$this->kind}_root"] = array("type" => "SQL", "value" => "get_project_{$this->kind}_root_type({$escProjectID}, t_{$this->kind}_root_id) IS NULL");
			} else if ( is_array($type) ){
				$arrEsc = array();
				foreach ( $type as $valType ){
					$arrEsc[] = "'" . $this->esc($valType) . "'";
				}
				$arrCondition["t_project_{$this->kind}_root"] = array("type" => "SQL", "value" => "get_project_{$this->kind}_root_type({$escProjectID}, t_{$this->kind}_root_id) IN (" . implode(",", $arrEsc) . ")");
			} else {
				$escType = $this->esc($type);
				$arrCondition["t_project_{$this->kind}_root"] = array("type" => "SQL", "value" => "get_project_{$this->kind}_root_type({$escProjectID}, t_{$this->kind}_root_id) = {$escType}");
			}
		}
		// *並び順
		$arrOrder = $this->getOrderBy($arrOrder, $arrCustomOrder, array(
			"t_{$this->kind}_root_public" => array_keys(DAO::T_TERM_ROOT_PUBLIC_TABLE()),
			"t_project_id"   => DAO::DESC,
			"t_term_root_id" => DAO::DESC,
		));
		// *実行
		$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:exists
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function exists( $name, $excludeID = NULL ){
		$escName = Util::esc($name);
		
		$arrCondition = array();
		$arrCondition["t_{$this->kind}_root_state"] = DAO::T_TERM_ROOT_STATE_NORMAL;
		$arrCondition["t_{$this->kind}_root_name"] = array(
			"type"  => DAO::TYPE_SQL,
			"value" => "t_{$this->kind}_root_name LIKE '{$escName}'",
		);
		if ( Validate::isNumeric($excludeID) && $excludeID !== INIT_ID ){
			$arrCondition["t_{$this->kind}_root_id"] = array(
				"type"  => DAO::TYPE_NOT,
				"value" => $excludeID,
			);
		}
		
		$arrField = array("t_{$this->kind}_root_id");
		$resCheck = $this->_select( $arrField, $arrCondition );
		if ( $resCheck["result"] === RES_OK ){
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $termRootID, $state = NULL, $public = NULL, $name = NULL, $langS = NULL, $langT = NULL, $projectID = NULL ){
		$res    = NULL;
		
		// 登録できるかチェックする
		// （名前重複チェック）
		if ( ! is_null($name) ){
			if ( $this->exists($name, $termRootID) ){
				$res["result"] = RES_NG;
				$res["error"]  = self::ERROR_NAME_DUPLICATE;
				
				return $res;
			}
		}
		
		// 新規登録
		if ( $termRootID === INIT_ID ){
			// *フィールド
			$arrValue = array(
				"t_{$this->kind}_root_state"  => DAO::T_TERM_ROOT_STATE_NORMAL,
				"t_{$this->kind}_root_public" => $public,
				"t_{$this->kind}_root_name"   => $name,
				"t_{$this->kind}_root_lang_s" => $langS,
				"t_{$this->kind}_root_lang_t" => $langT,
				"t_project_id"                => $projectID,
			);
			// *実行
			$res = $this->_insert($arrValue, TRUE);
		}
		// 更新
		else {
			// *フィールド
			$arrValue = $this->getValueUpdate(array(
				"t_{$this->kind}_root_state"  => $state,
				"t_{$this->kind}_root_public" => $public,
				"t_{$this->kind}_root_name"   => $name,
				"t_{$this->kind}_root_lang_s" => $langS,
				"t_{$this->kind}_root_lang_t" => $langT,
			));
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_{$this->kind}_root_id" => $termRootID,
				"t_project_id"            => $projectID,
				// t_term_root_updateは用語の更新フラグとして使用するため更新しない。
			));
			// *実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $termRootID;
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:disabled
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function disabled( $termRootID ){
		$res = $this->register($termRootID, DAO::T_TERM_ROOT_STATE_DELETE);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:delete
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function delete( $termRootID, $projectID = NULL, $organizationID = NULL ){
		$res    = NULL;
		
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_{$this->kind}_root_id" => $termRootID,
			"t_project_id"            => $projectID,
		));
		if ( ! is_null($organizationID) ){
			$escOrganizationID = $this->esc($organizationID);
			$arrCondition["t_organization_id"] = array("type" => "SQL", "value" => "get_project_organization_id(t_project_id) != {$escOrganizationID}");
		}
		
		// *削除予定IDを取得
		if ( ($arrTermRootID = $this->getPrimaryKey("t_{$this->kind}_root_id", $arrCondition)) === FALSE ){
			$res["result"] = RES_NG;
			return $res;
		} else if ( count($arrTermRootID) === 0 ){
			$res["result"] = RES_OK;
			return $res;
		}
		
		// *実行
		$res = $this->_delete($arrCondition);
		
		if ( $res["result"] === RES_OK ){
			//Termの削除
			$objDAOTerm = DAO::instance("term", $this->kind);
			$res = $objDAOTerm->delete(NULL, $arrTermRootID);
		}
		if ( $res["result"] === RES_OK ){
			//ProjectTermRootの削除
			$objDAOProjectTermRoot = DAO::instance("project_term_root", $this->kind);
			$res = $objDAOProjectTermRoot->delete(NULL, $arrTermRootID);
		}
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:outputTsv
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function outputTsv( $outputDir, $arrCondition, $filenameInLang = FALSE ){
		$result = RES_OK;
		
		$objDAOTerm = DAO::instance("term", $this->kind);
		
		// *フィールド
		$arrField = array(
			"t_{$this->kind}_root_id",
			"t_{$this->kind}_root_state",
			"t_{$this->kind}_root_lang_s",
			"t_{$this->kind}_root_lang_t",
			"t_{$this->kind}_root_update",
		);
		// *実行
		$offset = 0;
		$limit = 1000;
		do {
			$res = $this->select($arrField, $arrCondition, NULL, NULL, NULL, $limit, $offset);
			if ( $res["result"] === RES_OK ){
				foreach ( $res["data"] as $dataTermRoot ){
					$rootID = $dataTermRoot["t_{$this->kind}_root_id"];
					$state  = $dataTermRoot["t_{$this->kind}_root_state"];
					$langS  = $dataTermRoot["t_{$this->kind}_root_lang_s"];
					$langT  = $dataTermRoot["t_{$this->kind}_root_lang_t"];
					
					if ( $filenameInLang ){
						if ( strcmp($langS, LANG_CN) === 0 ){
							$langS = "zh";
						}
						if ( strcmp($langT, LANG_CN) === 0 ){
							$langT = "zh";
						}
						$tsvFile = "{$outputDir}/{$rootID}.{$langS}-{$langT}.tsv";
					} else {
						$tsvFile = "{$outputDir}/{$rootID}.tsv";
					}
					
					if ( $state === DAO::T_TERM_ROOT_STATE_DELETE ){
						// 削除された場合 TSVファイルを削除する
						if ( file_exists($tsvFile) ){
							unlink($tsvFile);
						}
					} else {
						if ( $objDAOTerm->outputTsv( $tsvFile, $rootID ) === RES_NG ){
							return $result;
						}
					}
				}
			} else if ( $res["result"] === RES_NO_DATA ){
				if ( $offset === 0 ){
					$result = RES_NO_DATA;
				}
				break;
			} else {
				$result = RES_NG;
				break;
			}
			$offset += $limit;
		} while ( $res["count"] > $offset );
		
		return $result;
	}
}

Class DAO_term_root_term extends DAO_term_root {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_dummy ){
		parent::__construct(Term::KIND_TERM);
	}
	
}

Class DAO_term_root_bilingual extends DAO_term_root {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_dummy ){
		parent::__construct(Term::KIND_BILINGUAL);
	}
	
}

Class DAO_term_root_fix extends DAO_term_root {
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_dummy ){
		parent::__construct(Term::KIND_FIX);
	}
	
}
