<?php
Class DAO_user extends DAO {
	
	const ORDER_KEY_NAME = "name";
	const ORDER_KEY_TYPE = "type";
	const ORDER_KEY_STATE = "state";
	const ORDER_KEY_REGISTER = "register";
	public static function ORDER_KEY_SQL_TABLE( $key = NULL ){
		$arrTable = array(
			self::ORDER_KEY_NAME  => "t_user_name",
			self::ORDER_KEY_TYPE  => "t_user_type",
			self::ORDER_KEY_STATE => "t_user_state",
			self::ORDER_KEY_REGISTER => "t_user_register",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_tableName = "t_user" ){
		$this->tableName = $_tableName;
	}
	
	/***************************************************************
	 * 名前:getInit
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getInit( $field ){
		$arrValue = array(
			"t_user_id"           => DAO::ID_INIT,
			"t_user_name"         => "",
			"t_user_state"        => DAO::T_USER_STATE_ACTIVE,
			"t_user_type"         => DAO::T_USER_TYPE_NORMAL,
			"t_user_login"        => DAO::T_USER_LOGIN_OK,
			"t_user_twitter"      => "",
			"t_organization_id"   => DAO::ID_INIT,
			"t_organization_name" => "",
		);
		
		return $arrValue[$field];
	}
	
	/***************************************************************
	 * 名前:get
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function get( $userID = NULL, $userName = NULL, $userState = NULL, $userType = NULL, $organizationID = NULL, $arrOrder = NULL, $limit = NULL, $offset = NULL, $isUserNameExactMatch = FALSE ){
		$res    = NULL;
		
		// *フィールド
		$arrField = array(
			"t_user_id",
			"t_user_name",
			"t_user_state",
			"t_user_type",
			"t_user_password",
			"t_user_question",
			"t_user_answer",
			"t_user_disp_lang",
			"t_user_login",
			"t_user_warn_non_active",
			"t_user_twitter",
			"t_user_qredit_dictionary",
			"t_user_qredit_display",
			"t_user_qredit_width",
			"t_user_qredit_scroll",
			"t_user_qredit_lookup",
			"t_user_qredit_action",
			"t_user_qredit_tag",
			"t_user_register",
			"t_organization_id",
			"get_organization_name(t_organization_id) AS t_organization_name",
		);
		// *条件
		$arrCondition = $this->getCondition(array(
			"t_user_id"         => $userID,
			"t_user_state"      => $userState,
			"t_user_type"       => $userType,
			"t_organization_id" => $organizationID,
		));
		if ( ! is_null($userName) ){
			if ( $isUserNameExactMatch ){
				// 大文字小文字完全一致
				$arrCondition["t_user_name"] = array("type"=>DAO::TYPE_SQL, "value"=>"t_user_name = '".$this->esc($userName)."' COLLATE utf8_bin");
			} else {
				$arrCondition["t_user_name"] = $userName;
			}
		}
		// *並び順
		if ( isset($arrOrder["t_user_type"]) ){
			if ( $arrOrder["t_user_type"] === DAO::ASC ){
				$arrOrder["t_user_type"] = array_keys(DAO::T_USER_TYPE_TABLE());
			} else if ( $arrOrder["t_user_type"] === DAO::DESC ){
				$arrOrder["t_user_type"] = array_reverse(array_keys(DAO::T_USER_TYPE_TABLE()));
			}
		} 
		$arrOrder = $this->getOrderBy($arrOrder, array(
			"t_organization_id" => DAO::ASC,
			"t_user_type"       => array_keys(DAO::T_USER_TYPE_TABLE()),
			"t_user_state"      => DAO::ASC,
			"t_user_login"      => DAO::ASC,
			"t_user_name"       => DAO::ASC,
		));
		
		// *実行
		$res = $this->_select($arrField, $arrCondition, NULL, NULL, $arrOrder, $limit, $offset);
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:exists
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function exists( $name, $excludeID = NULL ){
		$escName = Util::esc($name);
		
		$arrCondition = array();
		$arrCondition["t_user_state"] = DAO::T_USER_STATE_ACTIVE;
		$arrCondition["t_user_name"] = array(
			"type"  => DAO::TYPE_SQL,
			"value" => "t_user_name LIKE '{$escName}'",
		);
		if ( Validate::isNumeric($excludeID) && $excludeID !== INIT_ID ){
			$arrCondition["t_user_id"] = array(
				"type"  => DAO::TYPE_NOT,
				"value" => $excludeID,
			);
		}
		
		$arrField = array("t_user_id");
		$resCheck = $this->_select( $arrField, $arrCondition );
		if ( $resCheck["result"] === RES_OK ){
			return TRUE;
		} else {
			return FALSE;
		}
	}
	
	/***************************************************************
	 * 名前:register
	 * 説明:更新はパスワードとログイン許可のみ(所属組織、タイプ、ユーザー名は変更できない)
	 * 引数:
	 * 返値:
	****************************************************************/
	function register( $userID, $name = NULL, $state = NULL, $type = NULL, $password = NULL, $question = NULL, $answer = NULL, $dispLang = NULL, $login = NULL, $twitter = NULL, $qreditDictionary = NULL, $qreditDisplay = NULL, $qreditWidth = NULL, $qreditScroll = NULL, $qreditLookup = NULL, $qreditAction = NULL, $qreditTag = NULL, $sessionID = NULL, $organizationID = NULL ){
		$res    = NULL;
		
		// 新規登録
		if ( $userID === INIT_ID ){
			// *フィールド
			$arrValue = array(
				"t_user_name"       => $name,
				"t_user_state"      => $state,
				"t_user_type"       => $type,
				"t_user_password"   => Auth::encPass($password),
				"t_user_disp_lang"  => $dispLang,
				"t_user_login"      => $login,
				"t_user_warn_non_active" => DAO::T_USER_WARN_NON_ACTIVE_NONE,
				"t_organization_id" => $organizationID,
				"t_user_qredit_dictionary" => QReditDict::FixIDs(0),
				"t_user_register"   => DAO::NOW,
			);
			// *実行
			$res = $this->_insert($arrValue, TRUE);
		}
		// 更新
		else {
			// *フィールド
			$arrValue = $this->getValueUpdate(array(
				"t_user_state"          => $state,
				"t_user_question"       => $question,
				"t_user_answer"         => $answer,
				"t_user_disp_lang"      => $dispLang,
				"t_user_login"          => $login,
				"t_user_twitter"        => $twitter,
				"t_user_qredit_display" => $qreditDisplay,
				"t_user_qredit_width"   => $qreditWidth,
				"t_user_qredit_scroll"  => $qreditScroll,
				"t_user_qredit_lookup"  => $qreditLookup,
				"t_user_qredit_action"  => $qreditAction,
				"t_user_qredit_tag"     => $qreditTag,
				"t_user_session_id"     => $sessionID,
			));
			if ( ! is_null($password) ){
				$arrValue["t_user_password"] = Auth::encPass($password);
			}
			if ( ! is_null($qreditDictionary) ){
				$arrValue["t_user_qredit_dictionary"] = QReditDict::FixIDs($qreditDictionary);
			}
			// *条件
			$arrCondition = $this->getCondition(array(
				"t_user_id" => $userID,
			));
			// *実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $userID;
		}
		
		return $res;
	}
	
	function registerWarnNonActive( $userID, $warnNonActive ){
		$res    = NULL;
		
		if ( $userID === INIT_ID ){
			$res = array("result" => RES_NG);
		} else {
			$arrValue = $this->getValueUpdate(array(
				"t_user_warn_non_active" => $warnNonActive,
			));
			$arrCondition = $this->getCondition(array(
				"t_user_id" => $userID,
			));
			// * 実行
			$res = $this->_update($arrValue, $arrCondition);
			$res["id"] = $userID;
		}
		
		return $res;
	}
}
