<?php
abstract Class Doc {
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct(){
	}
	
	/***************************************************************
	 * 名前:load
	 * 説明:ロード
	 * 引数:XMLデータ
	 * 返値:
	****************************************************************/
	function load( $xml ){
		if ( ! @$this->dom->loadXML($xml) ){
			return FALSE;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:validate
	 * 説明:tmxのvalidate
	 * 引数: -
	 * 返値:TRUE or FALSE
	****************************************************************/
	function validate( &$arrError = NULL ){
		// 内部エラー表示を行う
		libxml_use_internal_errors(TRUE);
		
		if ( ! @$this->dom->schemaValidate(Setting::get("PATH_DATA") . "/xsd/" . $this->XSD) ){
			// エラー情報を取得する
			$errors = libxml_get_errors();
			
			foreach ($errors as $error) {    // $error の型 : libXMLError
				$code    = $error->code;     // エラーコード
				$line    = $error->line;     // エラー箇所（行数）
				$column  = $error->column;   // エラーが発生した列
				$message = $error->message;  // エラーメッセージ
				$file    = $error->file;     // エラーファイル
				
				$str = "{$message}({$file}:{$line})";
				$str = preg_replace("/(\r\n|\r|\n)/", "", $str);
				
				$arrError[] = $str;
			}
			
			// エラー情報をクリアする
			libxml_clear_errors();
			
			return FALSE;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:validateHeaderProp
	 * 説明:tmxのvalidate
	 * 引数: -
	 * 返値:TRUE or FALSE
	****************************************************************/
	function validateHeaderProp( $typeValue, $value ){
		$val = $this->getHeaderPropValue($typeValue);
		
		return ($value === $val);
	}
	
	/***************************************************************
	 * 名前:save
	 * 説明:セーブ
	 * 引数:
	 * 返値:
	****************************************************************/
	function saveXML(){
		return $this->dom->saveXML();
	}
	
	/***************************************************************
	 * 名前:makeXML
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function makeXML( $arr ){
		$dom = new domDocument('1.0', 'UTF-8');
		$dom->formatOutput = true;
		
		$this->makeNode($dom, NULL, $arr);
		
		return $dom->saveXML();
	}
	
	/***************************************************************
	 * 名前:makeNode
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function makeNode( $dom, $parent, $arr ){
		if ( is_array($arr) ){
			foreach ( $arr as $key => $val ){
				// 添字が数値の場合
				if ( is_numeric($key) ){
					$parent = $this->makeNode($dom, $parent, $val);
				} else {
					// 属性
					if ( strcmp($key, "attribute") == 0 ){
						if ( is_array($val) ){
							foreach ( $val as $attrKey => $attrVal ){
								$parent->setAttribute($attrKey, $attrVal);
							}
						}
					} else if ( strcmp($key, "nodetext") == 0 ){
						$parent->appendChild($dom->createTextNode($val));
					} else if ( strcmp($key, "cdata") == 0 ){
						$parent->appendChild($dom->createCDATASection($val));
					} else {
						$node = $dom->createElement($key);
						if ( is_array($val) ){
							$node = $this->makeNode($dom, $node, $val);
						} else {
							$node->appendChild($dom->createTextNode($val));
						}
						if ( is_null($parent) ){
							$dom->appendChild($node);
						} else {
							$parent->appendChild($node);
						}
					}
				}
			}
		}
		return $parent;
	}

	/***************************************************************
	 * 名前:xml2text
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function xml2text( $xml ){
		$res     = array();
		$result  = RES_OK;
		$sm = array();
		// 原文
		$sm["source"] = "";
		// 修正文
		$sm["target"] = "";
		
		$res['data'][] = $sm;
		$res['result'] = RES_OK;
		
		return $res;
	}
	*/
	abstract protected function xml2text($xml);
	
	/***************************************************************
	 * 名前:xml2table
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	/*
	function xml2table( $xml ){
		$res     = array();
		$result  = RES_OK;
		$html    = "";
		
		$sentences = $this->xml2text($xml);
		$result = $sentences["result"];
		
		if ( $result == RES_OK ){
			foreach ( $sentences["data"] as $sentence ){
				$hSource = "";
				$hTarget = "";
				if ( isset($sentence["source"]) ){
					$hSource = Util::esc($sentence["source"]);
				}
				if ( isset($sentence["target"]) ){
					$hTarget = Util::esc($sentence["target"]);
				}

$html .= <<< HTML
	<tr>
		<td>{$hSource}</td>
		<td>{$hTarget}</td>
	</tr>

HTML;

			}
			
$html = <<< HTML
<table class="table_translated">
	{$html}
</table>

HTML;

		}
		
		$res["data"] = $html;
		$res["result"] = $result;
		
		return $res;
	}
	*/
	abstract protected function xml2table($xml);


}
