<?php require_once("class/Doc.php"); ?>
<?php
Class DocProject extends Doc {
	const VERSION = "1.0";
	
	var $XSD = "project.xsd";
	
	var $dom;
	var $importMessage = array();
	
	public static function esc( $str ){
		return htmlspecialchars( $str, ENT_QUOTES );
	}
	
	public static function dec( $str ){
		return htmlspecialchars_decode( $str, ENT_QUOTES );
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct(){
		$this->dom = new domDocument("1.0", "UTF-8");
		
		$this->dom->formatOutput = TRUE;
		$this->dom->preserveWhiteSpace = FALSE;
	}
	
	/***************************************************************
	 * 名前:createTextElement
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	private function createTextElement( $name, $value = "" ){
		$node = $this->dom->createElement($name);
		$node->nodeValue = $value;
		return $node;
	}
	
	/***************************************************************
	 * 名前:createCDATAElement
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	private function createCDATAElement( $name, $value = "" ){
		$node = $this->dom->createElement($name);
		$node->appendChild($this->dom->createCDATASection($value));
		return $node;
	}
	
	/***************************************************************
	 * 名前:getNodeValue
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getNodeValue( $parentNode, $name, $default = "" ){
		$els = $parentNode->getElementsByTagName($name);
		if ( $els->length === 1 ){
			return $els->item(0)->nodeValue;
		} else {
			return $default;
		}
	}
	
	/***************************************************************
	 * 名前:create
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	function create( $projectID ){
		// node : edu(root)
		$nodeRoot = $this->dom->createElement("edu");
		// edu@version
		$nodeRoot->setAttribute("version", self::VERSION);
		// add
		$this->dom->appendChild($nodeRoot);
		
		//--------------------
		// Project
		//--------------------
		$nodeProject = $this->dom->createElement("project");
		$nodeRoot->appendChild($nodeProject);

		$nodeProject->appendChild( $this->createDetail($projectID) );
		$nodeProject->appendChild( $this->createWorkflow($projectID) );
		$nodeProject->appendChild( $this->createMembers($projectID) );
		$nodeProject->appendChild( $this->createDocuments($projectID) );
		$nodeProject->appendChild( $this->createTermRoots($projectID) );
		$nodeProject->appendChild( $this->createBilingualRoots($projectID) );
		$nodeProject->appendChild( $this->createData($projectID) );
		$nodeProject->appendChild( $this->createTasks($projectID) );
	}
	
	/***************************************************************
	 * 名前:createDetail
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createDetail( $projectID ){
		$nodeProject = $this->dom->createElement("detail");
		
		$objDAOProject = DAO::instance("project");
		$resDAOProject = $objDAOProject->get( $projectID );
		if ( $resDAOProject["result"] !== RES_OK ){
			return $nodeProject;
		}
		$data = $resDAOProject["data"][0];
		
		$nodeProject->setAttribute("id", $data["t_project_id"]);
		$nodeProject->setAttribute("state", $data["t_project_state"]);
		
		$node = $this->createCDATAElement("name", $data["t_project_name"]);
		$nodeProject->appendChild($node);
		
		$node = $this->createTextElement("start_date", $data["t_project_start"]);
		$nodeProject->appendChild($node);
		
		$node = $this->createTextElement("end_date", $data["t_project_end"]);
		$nodeProject->appendChild($node);
		
		$node = $this->createCDATAElement("text", $data["t_project_text"]);
		$nodeProject->appendChild($node);
		
		$node = $this->createTextElement("dictionary", $data["t_project_qredit_dictionary"]);
		$nodeProject->appendChild($node);
		
		return $nodeProject;
	}
	
	/***************************************************************
	 * 名前:createWorkflow
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createWorkflow( $projectID ){
		$nodeWorkflow = $this->dom->createElement("workflow");
		
		$objDAOWorkflow = DAO::instance("workflow");
		$resDAOWorkflow = $objDAOWorkflow->get( NULL, $projectID );
		if ( $resDAOWorkflow["result"] !== RES_OK ){
			return $nodeWorkflow;
		}
		$data = $resDAOWorkflow["data"][0];
		
		$node = $this->createTextElement("task_type", $data["t_workflow_task_type"]);
		$nodeWorkflow->appendChild($node);
		
		return $nodeWorkflow;
	}
	
	/***************************************************************
	 * 名前:createMembers
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createMembers( $projectID ){
		$nodeMembers = $this->dom->createElement("members");
		
		$objDAOProjectUser = DAO::instance("project_user");
		$resDAOProjectUser = $objDAOProjectUser->get( $projectID );
		if ( $resDAOProjectUser["result"] !== RES_OK ){
			return $nodeMembers;
		}
		
		$arrUserRole = array();
		foreach ( $resDAOProjectUser["data"] as $data ){
			$role   = $data["t_project_user_role"];
			$userID = $data["t_user_id"];
			$arrUserRole[$userID][] = $role;
		}
		
		//member要素を作成
		foreach ( $arrUserRole as $userID => $arrRole ){
			$nodeMember = $this->dom->createElement("member");
			$nodeMember->setAttribute("user_id", $userID);
			$nodeMembers->appendChild( $nodeMember );
			
			foreach ($arrRole as $role ){
				$nodeRole = $this->createTextElement("role", Task::ROLE_TABLE($role));
				$nodeRole->setAttribute("type", $role);
				$nodeMember->appendChild( $nodeRole );
			}
		}
		
		return $nodeMembers;
	}
	
	/***************************************************************
	 * 名前:createDocuments
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createDocuments( $projectID ){
		$nodeDocuments = $this->dom->createElement("documents");
		
		$objDAODocument = DAO::instance("document");
		$resDAODocument = $objDAODocument->get( NULL, array_keys(DAO::T_DOCUMENT_STATE_VALID_TABLE()), NULL, $projectID );
		if ( $resDAODocument["result"] !== RES_OK ){
			return $nodeDocuments;
		}
		
		//document要素を作成
		foreach ( array_reverse($resDAODocument["data"]) as $data ){
			$nodeDocument = $this->dom->createElement("document");
			$nodeDocument->setAttribute("id", $data["t_document_id"]);
			$nodeDocument->setAttribute("source_lang", $data["t_document_lang_s"]);
			$nodeDocument->setAttribute("target_lang", $data["t_document_lang_t"]);
			$nodeDocuments->appendChild($nodeDocument);
			
			$node = $this->createCDATAElement("title", $data["t_document_title"]);
			$nodeDocument->appendChild($node);
			
			$node = $this->createCDATAElement("text", self::esc($data["t_document_text"]));
			$nodeDocument->appendChild($node);
			
			$node = $this->createCDATAElement("comment", $data["t_document_comment"]);
			$nodeDocument->appendChild($node);
			
			$node = $this->createCDATAElement("url", $data["t_document_url"]);
			$nodeDocument->appendChild($node);
		}
		
		return $nodeDocuments;
	}
	
	/***************************************************************
	 * 名前:createTermRoots
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createTermRoots( $projectID ){
		$nodeTermRoots = $this->dom->createElement("term_roots");
		
		$objDAOTermRoot = DAO::instance("term_root", Term::KIND_TERM);
		$resDAOTermRoot = $objDAOTermRoot->get( NULL, DAO::T_TERM_ROOT_STATE_NORMAL, NULL, NULL, $projectID, TRUE );
		if ( $resDAOTermRoot["result"] !== RES_OK ){
			return $nodeTermRoots;
		}
		
		$objDAOProjectTermRoot = DAO::instance("project_term_root", Term::KIND_TERM);
		$resDAOProjectTermRoot = $objDAOProjectTermRoot->get($projectID);
		if ( $resDAOProjectTermRoot["result"] !== RES_OK ){
			return $nodeTermRoots;
		}
		$arrType = array();
		foreach ( $resDAOProjectTermRoot["data"] as $data ){
			$id = $data["t_term_root_id"];
			$arrType[ $data["t_term_root_id"] ] = $data["t_project_term_root_type"];
		}
		
		//term_root要素を作成
		foreach ( $resDAOTermRoot["data"] as $data ){
			$id = $data["t_term_root_id"];
			
			$nodeTermRoot = $this->createTextElement("term_root");
			$nodeTermRoot->setAttribute("id", $id);
			$nodeTermRoot->setAttribute("type", $arrType[$id]);
			$nodeTermRoot->setAttribute("source_lang", $data["t_term_root_lang_s"]);
			$nodeTermRoot->setAttribute("target_lang", $data["t_term_root_lang_t"]);
			$nodeTermRoot->setAttribute("public", $data["t_term_root_public"]);
			$nodeTermRoots->appendChild($nodeTermRoot);
			
			$node = $this->createCDATAElement("name", $data["t_term_root_name"]);
			$nodeTermRoot->appendChild($node);
		}
		
		return $nodeTermRoots;
	}
	
	/***************************************************************
	 * 名前:createBilingualRoots
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createBilingualRoots( $projectID ){
		$nodeBilingualRoots = $this->dom->createElement("bilingual_roots");
		
		$objDAOBilingualRoot = DAO::instance("term_root", Term::KIND_BILINGUAL);
		$resDAOBilingualRoot = $objDAOBilingualRoot->get( NULL, DAO::T_TERM_ROOT_STATE_NORMAL, NULL, NULL, $projectID, TRUE );
		if ( $resDAOBilingualRoot["result"] !== RES_OK ){
			return $nodeBilingualRoots;
		}
		
		$objDAOProjectBilingualRoot = DAO::instance("project_term_root", Term::KIND_BILINGUAL);
		$resDAOProjectBilingualRoot = $objDAOProjectBilingualRoot->get($projectID);
		if ( $resDAOProjectBilingualRoot["result"] !== RES_OK ){
			return $nodeBilingualRoots;
		}
		$arrType = array();
		foreach ( $resDAOProjectBilingualRoot["data"] as $data ){
			$id = $data["t_term_root_id"];
			$arrType[ $data["t_term_root_id"] ] = $data["t_project_term_root_type"];
		}
		
		//bilingual_root要素を作成
		foreach ( $resDAOBilingualRoot["data"] as $data ){
			$id = $data["t_term_root_id"];
			
			$nodeBilingualRoot = $this->createTextElement("bilingual_root");
			$nodeBilingualRoot->setAttribute("id", $id);;
			$nodeBilingualRoot->setAttribute("type", $arrType[$id]);
			$nodeBilingualRoot->setAttribute("source_lang", $data["t_term_root_lang_s"]);
			$nodeBilingualRoot->setAttribute("target_lang", $data["t_term_root_lang_t"]);
			$nodeBilingualRoot->setAttribute("public", $data["t_term_root_public"]);
			$nodeBilingualRoots->appendChild($nodeBilingualRoot);
			
			$node = $this->createCDATAElement("name", $data["t_term_root_name"]);
			$nodeBilingualRoot->appendChild($node);
		}
		
		return $nodeBilingualRoots;
	}
	
	/***************************************************************
	 * 名前:createData
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createData( $projectID ){
		$nodeData = $this->dom->createElement("data");
		
		$objDAOData = DAO::instance("data");
		$resDAOData = $objDAOData->get( NULL, DAO::T_DATA_STATE_NORMAL, NULL, $projectID, TRUE );
		if ( $resDAOData["result"] !== RES_OK ){
			return $nodeData;
		}
		
		//datum要素を作成
		foreach ( $resDAOData["data"] as $data ){
			$nodeDatum = $this->dom->createElement("datum");
			$nodeDatum->setAttribute("id", $data["t_data_id"]);
			$nodeDatum->setAttribute("public", $data["t_data_public"]);
			$nodeDatum->setAttribute("props", $data["t_data_props"]);
			$nodeData->appendChild($nodeDatum);
			
			$node = $this->createTextElement("type", $data["t_data_type"]);
			$nodeDatum->appendChild($node);
			
			$node = $this->createCDATAElement("title", $data["t_data_title"]);
			$nodeDatum->appendChild($node);
			
			$node = $this->createCDATAElement("comment", $data["t_data_comment"]);
			$nodeDatum->appendChild($node);
		}
		
		return $nodeData;
	}
	
	/***************************************************************
	 * 名前:createTasks
	 * 説明:
	 * 引数:
	 * 返値:
	 ****************************************************************/
	private function createTasks( $projectID ){
		$nodeTasks = $this->dom->createElement("tasks");

		// workflowID の取得
		$objDAOWorkflow = DAO::instance("workflow");
		$resDAOWorkflow = $objDAOWorkflow->get( NULL, $projectID, NULL, 1 );
		if ( $resDAOWorkflow["result"] !== RES_OK ){
			return $nodeTasks;
		}
		$workflowID = $resDAOWorkflow["data"][0]["t_workflow_id"];
		
		$objDAOTask = DAO::instance("task");
		$resDAOTask = $objDAOTask->get( NULL, NULL, NULL, NULL, NULL, $workflowID );
		if ( $resDAOTask["result"] !== RES_OK ){
			return $nodeTasks;
		}
		
		$objDAOTaskActor         = DAO::instance("task_actor");
		$objDAOTaskDocument      = DAO::instance("task_document");
		$objDAOTaskTermRoot      = DAO::instance("task_term_root", Term::KIND_TERM);
		$objDAOTaskBilingualRoot = DAO::instance("task_term_root", Term::KIND_BILINGUAL);
		$objDAOTaskData          = DAO::instance("task_data");
		
		//task要素を作成
		foreach ( $resDAOTask["data"] as $data ){
			$taskID = $data["t_task_id"];
			
			$nodeTask = $this->dom->createElement("task");
			$nodeTask->setAttribute("id", $data["t_task_id"]);
			$nodeTask->setAttribute("type", $data["t_task_type"]);
			$nodeTask->setAttribute("state", $data["t_task_state"]);
			$nodeTasks->appendChild($nodeTask);
			
			$node = $this->createTextElement("data", self::esc($data["t_task_data"]));
			$nodeTask->appendChild($node);
			
			$node = $this->createTextElement("start_date", $data["t_task_start"]);
			$nodeTask->appendChild($node);
			
			$node = $this->createTextElement("end_date", $data["t_task_end"]);
			$nodeTask->appendChild($node);
			
			$resDAOTaskDocument = $objDAOTaskDocument->get($taskID);
			if ( $resDAOTaskDocument["result"] === RES_OK ){
				foreach( $resDAOTaskDocument["data"] as $dataDocument ){
					$nodeDocument = $this->createTextElement("document", $dataDocument["t_document_title"]);
					$nodeDocument->setAttribute("id",    $dataDocument["t_document_id"]);
					$nodeDocument->setAttribute("props", $dataDocument["t_task_props"]);
					$nodeTask->appendChild($nodeDocument);
				}
			}
			
			$resDAOTaskTerm = $objDAOTaskTermRoot->get($taskID);
			if ( $resDAOTaskTerm["result"] === RES_OK ){
				foreach ( $resDAOTaskTerm["data"] as $dataTaskTerm ){
					$nodeTerm = $this->createTextElement("term", $dataTaskTerm["t_term_root_name"]);
					$nodeTerm->setAttribute("id",    $dataTaskTerm["t_term_root_id"]);
					$nodeTerm->setAttribute("props", $dataTaskTerm["t_task_props"]);
					$nodeTask->appendChild($nodeTerm);
				}
			}
			
			$resDAOTaskBilingual = $objDAOTaskBilingualRoot->get($taskID);
			if ( $resDAOTaskBilingual["result"] === RES_OK ){
				foreach ( $resDAOTaskBilingual["data"] as $dataTaskBilingual ){
					$nodeBilingual = $this->createTextElement("bilingual", $dataTaskBilingual["t_bilingual_root_name"]);
					$nodeBilingual->setAttribute("id",    $dataTaskBilingual["t_bilingual_root_id"]);
					$nodeBilingual->setAttribute("props", $dataTaskBilingual["t_task_props"]);
					$nodeTask->appendChild($nodeBilingual);
				}
			}
			
			$resDAOTaskData = $objDAOTaskData->get($taskID);
			if ( $resDAOTaskData["result"] === RES_OK ){
				foreach ( $resDAOTaskData["data"] as $dataTaskData ){
					$nodeData = $this->createTextElement("data", $dataTaskData["t_data_title"]);
					$nodeData->setAttribute("id",    $dataTaskData["t_data_id"]);
					$nodeData->setAttribute("props", $dataTaskData["t_task_props"]);
					$nodeTask->appendChild($nodeData);
				}
			}
			
			$resDAOTaskActor = $objDAOTaskActor->get($taskID);
			if ( $resDAOTaskActor["result"] === RES_OK ){
				foreach( $resDAOTaskActor["data"] as $dataActor ){
					$nodeActor = $this->createTextElement("actor", Task::ROLE_TABLE($dataActor["t_task_role"]));
					$nodeActor->setAttribute("role", $dataActor["t_task_role"]);
					$nodeActor->setAttribute("type", $dataActor["t_task_actor_type"]);
					$nodeActor->setAttribute("user_id", $dataActor["t_user_id"]);
					$nodeTask->appendChild($nodeActor);
				}
			}
		}
		
		return $nodeTasks;
	}
	
	/***************************************************************
	 * 名前:getDetail
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getDetail(){
		$arrRes = array();
		
		$nodeDetail = $this->dom->getElementsByTagName("detail")->item(0);
		if ( ! $nodeDetail ){
			return $arrRes;
		}
		
		$id         = $nodeDetail->getAttribute("id");
		$state      = $nodeDetail->getAttribute("state");
		$name       = $this->getNodeValue($nodeDetail, "name");
		$start      = $this->getNodeValue($nodeDetail, "start_date");
		$end        = $this->getNodeValue($nodeDetail, "end_date");
		$text       = $this->getNodeValue($nodeDetail, "text");
		$dictionary = $this->getNodeValue($nodeDetail, "dictionary");
		
		$arrRes = array(
			"id"         => $id,
			"state"      => $state,
			"name"       => $name,
			"start"      => $start,
			"end"        => $end,
			"text"       => $text,
			"dictionary" => $dictionary,
		);
		
		$nodeWorkflow = $this->dom->getElementsByTagName("workflow")->item(0);
		if ( ! $nodeWorkflow ){
			return $arrRes;
		}
		
		$taskType = $this->getNodeValue($nodeWorkflow, "task_type");
		$arrRes["task_type"] = $taskType;
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getMembers
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getMembers(){
		$arrRes = array();
		
		$arrNodeRoot = $this->dom->getElementsByTagName("members")->item(0);
		if ( ! $arrNodeRoot ){
			return $arrRes;
		}
		
		$arrUserRole = array();
		$arrNodeMember = $arrNodeRoot->getElementsByTagName("member");
		foreach( $arrNodeMember as $nodeMember ){
			$userID      = $nodeMember->getAttribute("user_id");
			$arrNodeRole = $nodeMember->getElementsByTagName("role");
			$arrRole = array();
			foreach ( $arrNodeRole as $nodeRole ){
				$arrRole[] = $nodeRole->getAttribute("type");
			}
			$arrRes[] = array(
				"user_id" => $userID,
				"role"    => $arrRole,
			);
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getDocuments
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getDocuments(){
		$arrRes = array();
		
		$arrNodeRoot = $this->dom->getElementsByTagName("documents")->item(0);
		if ( ! $arrNodeRoot ){
			return $arrRes;
		}
		
		$arrNodeDocument = $arrNodeRoot->getElementsByTagName("document");
		foreach ( $arrNodeDocument as $nodeDocument ){
			$id = $nodeDocument->getAttribute("id");
			$slang = $nodeDocument->getAttribute("source_lang");
			$tlang = $nodeDocument->getAttribute("target_lang");
			
			$title = $this->getNodeValue($nodeDocument, "title");
			$text    = self::dec( $this->getNodeValue($nodeDocument, "text"));
			$comment = $this->getNodeValue($nodeDocument, "comment");
			$url     = $this->getNodeValue($nodeDocument, "url");
			
			$arrRes[] = array(
				"id"          => $id,
				"source_lang" => $slang,
				"target_lang" => $tlang,
				"title"       => $title,
				"text"        => $text,
				"comment"     => $comment,
				"url"         => $url,
			);
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getTermRoots
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getTermRoots(){
		$arrRes = array();
		
		$arrNodeRoot = $this->dom->getElementsByTagName("term_roots")->item(0);
		if ( ! $arrNodeRoot ){
			return $arrRes;
		}
		
		$arrNodeTermRoot = $arrNodeRoot->getElementsByTagName("term_root");
		foreach ( $arrNodeTermRoot as $nodeTermRoot ){
			$id     = $nodeTermRoot->getAttribute("id");
			$type   = $nodeTermRoot->getAttribute("type");
			$slang  = $nodeTermRoot->getAttribute("source_lang");
			$tlang  = $nodeTermRoot->getAttribute("target_lang");
			$public = $nodeTermRoot->getAttribute("public");
			$title  = $this->getNodeValue($nodeTermRoot, "title");
			if ( strlen($title) === 0 ){
				$title = $nodeTermRoot->nodeValue;
			}
			$arrRes[] = array(
				"id"          => $id,
				"type"        => $type,
				"source_lang" => $slang,
				"target_lang" => $tlang,
				"public"      => $public,
				"title"       => $title,
			);
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getBilingualRoots
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getBilingualRoots(){
		$arrRes = array();
		
		$arrNodeRoot = $this->dom->getElementsByTagName("bilingual_roots")->item(0);
		if ( ! $arrNodeRoot ){
			return $arrRes;
		}
		
		$arrNodeBilingualRoot = $arrNodeRoot->getElementsByTagName("bilingual_root");
		foreach ( $arrNodeBilingualRoot as $nodeBilingualRoot ){
			$id     = $nodeBilingualRoot->getAttribute("id");
			$type   = $nodeBilingualRoot->getAttribute("type");
			$slang  = $nodeBilingualRoot->getAttribute("source_lang");
			$tlang  = $nodeBilingualRoot->getAttribute("target_lang");
			$public = $nodeBilingualRoot->getAttribute("public");
			$title  = $this->getNodeValue($nodeBilingualRoot, "title");
			if ( strlen($title) === 0 ){
				$title = $nodeBilingualRoot->nodeValue;
			}
			$arrRes[] = array(
				"id"          => $id,
				"type"        => $type,
				"source_lang" => $slang,
				"target_lang" => $tlang,
				"public"      => $public,
				"title"       => $title,
			);
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getData
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getData(){
		$arrRes = array();
		
		$arrNodeRoot = $this->dom->getElementsByTagName("data")->item(0);
		if ( ! $arrNodeRoot ){
			return $arrRes;
		}
		
		$arrNodeDatum = $arrNodeRoot->getElementsByTagName("datum");
		foreach ( $arrNodeDatum as $nodeDatum ){
			$id      = $nodeDatum->getAttribute("id");
			$public  = $nodeDatum->getAttribute("public");
			$props   = $nodeDatum->getAttribute("props");
			$type    = $this->getNodeValue($nodeDatum, "type");
			$title   = $this->getNodeValue($nodeDatum, "title");
			$comment = $this->getNodeValue($nodeDatum, "comment");
			$arrRes[] = array(
				"id"      => $id,
				"public"  => $public,
				"props"   => $props,
				"type"    => $type,
				"title"   => $title,
				"comment" => $comment,
			);
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getTasks
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getTasks(){
		$arrRes = array();
		
		$arrNodeRoot = $this->dom->getElementsByTagName("tasks")->item(0);
		if ( ! $arrNodeRoot ){
			return $arrRes;
		}
		
		$arrNodeTask = $arrNodeRoot->getElementsByTagName("task");
		foreach ( $arrNodeTask as $nodeTask ){
			$id    = $nodeTask->getAttribute("id");
			$type  = $nodeTask->getAttribute("type");
			$state = $nodeTask->getAttribute("state");
			$data  = self::dec( $this->getNodeValue($nodeTask, "data") );
			$startDate = $this->getNodeValue($nodeTask, "start_date");
			$endDate   = $this->getNodeValue($nodeTask, "end_date");
			
			$arrDocument = array();
			foreach ( $nodeTask->getElementsByTagName("document") as $nodeDocument ){
				$arrDocument[] = array(
					"id"    => $nodeDocument->getAttribute("id"),
					"props" => $nodeDocument->getAttribute("props"),
				);
			}
			
			$arrTerm = array();
			foreach ( $nodeTask->getElementsByTagName("term") as $nodeTerm ){
				$arrTerm[] = array(
					"id"    => $nodeTerm->getAttribute("id"),
					"props" => $nodeTerm->getAttribute("props"),
					"name"  => $nodeTerm->nodeValue,
				);
			}
			
			$arrBilingual = array();
			foreach ( $nodeTask->getElementsByTagName("bilingual") as $nodeBilingual ){
				$arrBilingual[] = array(
					"id"    => $nodeBilingual->getAttribute("id"),
					"props" => $nodeBilingual->getAttribute("props"),
					"name"  => $nodeBilingual->nodeValue,
				);
			}
			
			$arrData = array();
			foreach ( $nodeTask->getElementsByTagName("data") as $nodeData ){
				$arrData[] = array(
					"id"    => $nodeData->getAttribute("id"),
					"props" => $nodeData->getAttribute("props"),
					"title" => $nodeData->nodeValue,
				);
			}
			
			$arrActor = array();
			foreach ( $nodeTask->getElementsByTagName("actor") as $nodeActor ){
				$arrActor[] = array(
					"role"    => $nodeActor->getAttribute("role"),
					"user_id" => $nodeActor->getAttribute("user_id"),
					"type"    => $nodeActor->getAttribute("type"),
				);
			}
			
			$arrRes[] = array(
				"id"         => $id,
				"type"       => $type,
				"state"      => $state,
				"data"       => $data,
				"start_date" => $startDate,
				"end_date"   => $endDate,
				"document"   => $arrDocument,
				"term"       => $arrTerm,
				"bilingual"  => $arrBilingual,
				"datum"      => $arrData,
				"actor"      => $arrActor,
			);
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:importWorkflow
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function importWorkflow( $destProjectID, $destWorkflowID = NULL ){
		global $WORD;
		$wk = "WORD_P_CLASS_DOC_PROJECT";
		
		$objDAOWorkflow = DAO::instance("workflow");
		
		if ( ! Validate::isNumeric($destWorkflowID) ){
			$resDAOWorkflow = $objDAOWorkflow->get(NULL, $destProjectID);
			if ( $resDAOWorkflow["result"] === RES_OK ){
				$destWorkflowID = $resDAOWorkflow["data"][0]["t_workflow_id"];
			} else {
				$this->importMessage[] = "{$WORD($wk, 14)}";
				return FALSE;
			}
		}
		
		// 登録
		$projectDetail = $this->getDetail();
		if ( isset($projectDetail["task_type"]) ){
			$resDAOWorkflow = $objDAOWorkflow->register($destWorkflowID, NULL, NULL, $projectDetail["task_type"]);
			if ( $resDAOWorkflow["result"] === RES_NG ){
				$this->importMessage[] = "{$WORD($wk, 2)}";
				return FALSE;
			}
			
			$result = array(
				"id"        => $destWorkflowID,
				"task_type" => $projectDetail["task_type"],
			);
		} else {
			$result = array(
				"id" => $destWorkflowID,
			);
		}
		
		return $result;
	}
	
	/***************************************************************
	 * 名前:importMembers
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function importMembers( $destProjectID, $destClear, $userID ){
		global $WORD;
		$wk = "WORD_P_CLASS_DOC_PROJECT";
		
		$objDAOProjectUser = DAO::instance("project_user");
		
		$arrUserRole = array();
		
		// 削除
		if ( $destClear ){
			$resDAOProjectUser = $objDAOProjectUser->delete($destProjectID);
			$result = $resDAOProjectUser["result"];
			if ( $resDAOProjectUser["result"] === RES_NG ){
				$this->importMessage[] = "{$WORD($wk, 1)}";
				return FALSE;
			}
		}
		// 登録
		$arrMembers = $this->getMembers();
		foreach ( $arrMembers as $dataMember ){
			$arrUserRole[$dataMember["user_id"]] = $dataMember["role"];
		}
		$resDAOProjectUser = $objDAOProjectUser->register($destProjectID, $arrUserRole);
		if ( $resDAOProjectUser["result"] === RES_NG ){
			$this->importMessage[] = "{$WORD($wk, 2)}";
			return FALSE;
		}
		
		return $arrUserRole;
	}
	
	/***************************************************************
	 * 名前:importDocuments
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function importDocuments( $destProjectID, $destClear, $userID ){
		global $WORD;
		$wk = "WORD_P_CLASS_DOC_PROJECT";
		
		$objDAODocument = DAO::instance("document");
		$objDAODocument->setAuth( Auth::instance("document") );
		
		$arrDocumentID = array();
		
		// 削除
		if ( $destClear ){
			$resDAODocument = $objDAODocument->get(NULL, NULL, NULL, $destProjectID);
			if ( $resDAODocument["result"] === RES_OK ){
				foreach ( $resDAODocument["data"] as $dataDocument ){
					$resDAODocumentDelete = $objDAODocument->disabled( $dataDocument["t_document_id"] );
					if ( $resDAODocumentDelete["result"] === RES_NG ){
						$this->importMessage[] = "{$WORD($wk, 3)} ({$dataDocument["t_document_id"]})";
						return FALSE;
					}
				}
			}
		}
		// 登録
		$arrDocuments = $this->getDocuments();
		foreach ( $arrDocuments as $dataDocument ){
			$resDAODocument = $objDAODocument->register(INIT_ID, NULL, $dataDocument["source_lang"], $dataDocument["target_lang"], $dataDocument["title"], $dataDocument["text"], $dataDocument["comment"], $destProjectID, $userID);
			if ( $resDAODocument["result"] === RES_NG ){
				$this->importMessage[] = "{$WORD($wk, 4)} ({$dataDocument["id"]})";
				return FALSE;
			}
			
			$arrDocumentID[ $dataDocument["id"] ] = $resDAODocument["id"];
		}
		
		return $arrDocumentID;
	}
	
	/***************************************************************
	 * 名前:importTermRoots
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function importTermRoots( $kind, $destProjectID, $destClear, $userID, $orgID ){
		global $WORD;
		$wk = "WORD_P_CLASS_DOC_PROJECT";
		
		$objDAOTermRoot        = DAO::instance("term_root", $kind);
		$objDAOProjectTermRoot = DAO::instance("project_term_root", $kind);
		
		$arrTermRootID = array();
		
		if ( ! Validate::inKeyArray($kind, Term::KIND_TABLE()) ){
			$this->importMessage[] = "{$WORD($wk, 5)}";
			return FALSE;
		}
		$kindText = Term::KIND_TABLE($kind);
		
		// 削除
		// 参照情報のみ削除する
		if ( $destClear ){
			$resDAOProjectTermRoot = $objDAOProjectTermRoot->get($destProjectID); // コピー先のプロジェクトに登録されている用語集一覧（参照も含む）
			if ( $resDAOProjectTermRoot["result"] === RES_OK ){
				foreach ( $resDAOProjectTermRoot["data"] as $dataProjectTermRoot ){
					$deleteTermRootID = $dataProjectTermRoot["t_term_root_id"];
					$deleteProjectID = DAO::storedFunction("get_project_id_by_{$kind}_root_id", $deleteTermRootID);
					if ( $destProjectID !== $deleteProjectID ){ //参照登録している
						$resDAOProjectTermRootDelete = $objDAOProjectTermRoot->delete( $destProjectID, $deleteTermRootID );
						if ( $resDAOProjectTermRootDelete["result"] !== RES_OK ){
							$this->importMessage[] = "{$WORD($wk, 6, $kindText)} ({$deleteTermRootID})";
							return FALSE;
						}
					}
				}
			}
		}
		
		// 登録
		switch ( $kind ){
			case Term::KIND_TERM:
				$arrTermRoots = $this->getTermRoots();
				break;
			case Term::KIND_BILINGUAL:
				$arrTermRoots = $this->getBilingualRoots();
				break;
		}
		foreach ( $arrTermRoots as $dataTermRoot ){
			// 追加するか参照するか判断する
			$isNew = FALSE;
			$resDAOTermRoot = $objDAOTermRoot->get($dataTermRoot["id"], DAO::T_TERM_ROOT_STATE_NORMAL);
			if ( $resDAOTermRoot["result"] === RES_OK ){
				$existsTermRoot = $resDAOTermRoot["data"][0];
				$existsTermRootPublic = $existsTermRoot["t_term_root_public"];
				if ( $existsTermRootPublic === DAO::T_TERM_ROOT_PUBLIC_OPN ){
					$arrTermRootID[ $dataTermRoot["id"] ] = $dataTermRoot["id"];
				} else if ( $existsTermRootPublic === DAO::T_TERM_ROOT_PUBLIC_ORG && $existsTermRoot["t_organization_id"] === $orgID ){
					$arrTermRootID[ $dataTermRoot["id"] ] = $dataTermRoot["id"];
				} else if ( $existsTermRootPublic === DAO::T_TERM_ROOT_PUBLIC_PRJ && $existsTermRoot["t_project_id"] === $destProjectID ){
					$arrTermRootID[ $dataTermRoot["id"] ] = $dataTermRoot["id"];
				} else {
					$isNew = TRUE;
				}
			} else {
				$isNew = TRUE;
			}
			
			// 新規作成
			if ( $isNew ){
				$resDAOTermRoot = $objDAOTermRoot->register(INIT_ID, DAO::T_TERM_ROOT_STATE_NORMAL, $dataTermRoot["public"], $dataTermRoot["title"], $dataTermRoot["source_lang"], $dataTermRoot["target_lang"], $destProjectID);
				if ( $resDAOTermRoot["result"] === RES_NG && $resDAOTermRoot["error"] === DAO_term_root::ERROR_NAME_DUPLICATE ){
					// 名前が重複している → スキップ
					$this->importMessage[] = "{$WORD($wk, 7, $kindText)} ({$dataTermRoot["title"]})";
				} else if ( $resDAOTermRoot["result"] !== RES_OK ){
					$this->importMessage[] = "{$WORD($wk, 8, $kindText)} ({$dataTermRoot["id"]})";
					return FALSE;
				} else {
					// 成功
					$arrTermRootID[ $dataTermRoot["id"] ] = $resDAOTermRoot["id"];
				}
			}
			
			if ( isset($arrTermRootID[$dataTermRoot["id"]]) ){
				$resDAOProjectTermRoot = $objDAOProjectTermRoot->register( $destProjectID, $arrTermRootID[$dataTermRoot["id"]], $userID, $dataTermRoot["type"]);
				if ( $resDAOProjectTermRoot["result"] !== RES_OK ){
					$this->importMessage[] = "{$WORD($wk, 9, $kindText)} ({$arrTermRootID[$dataTermRoot["id"]]})";
					return FALSE;
				}
			}
		}
		
		return $arrTermRootID;
	}
	
	/***************************************************************
	 * 名前:importData
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function importData( $destProjectID, $destClear, $userID, $orgID ){
		global $WORD;
		$wk = "WORD_P_CLASS_DOC_PROJECT";
		
		$objDAOData        = DAO::instance("data");
		$objDAOProjectData = DAO::instance("project_data");
		
		$arrDataID = array();
		
		// 削除
		// 参照情報のみ削除する
		if ( $destClear ){
			$resDAOProjectData = $objDAOProjectData->get($destProjectID);
			if ( $resDAOProjectData["result"] === RES_OK ){
				foreach ( $resDAOProjectData["data"] as $dataProjectData ){
					$deleteDataID = $dataProjectData["t_data_id"];
					$deleteProjectID = DAO::storedFunction("get_project_id_by_data_id", $deleteDataID);
					if ( $destProjectID !== $deleteProjectID ){ //参照登録している
						$resDAOProjectDataDelete = $objDAOProjectData->delete( $destProjectID, $deleteDataID );
						if ( $resDAOProjectDataDelete["result"] !== RES_OK ){
							$this->importMessage[] = "{$WORD($wk, 10)} ({$deleteDataID})";
							return FALSE;
						}
					}
				}
			}
		}
		
		// 登録
		$arrData = $this->getData();
		foreach ( $arrData as $dataDatum ){
			// 追加するか参照するか判断する
			$isNew = FALSE;
			$resDAOData = $objDAOData->get($dataDatum["id"], DAO::T_DATA_STATE_NORMAL);
			if ( $resDAOData["result"] === RES_OK ){
				$existsData = $resDAOData["data"][0];
				$existsDataPublic = $existsData["t_data_public"];
				if ( $existsDataPublic === DAO::T_DATA_PUBLIC_OPN ){
					$arrDataID[ $dataDatum["id"] ] = $dataDatum["id"];
				} else if ( $existsDataPublic === DAO::T_DATA_PUBLIC_ORG && $existsData["t_organization_id"] === $orgID ){
					$arrDataID[ $dataDatum["id"] ] = $dataDatum["id"];
				} else if ( $existsDataPublic === DAO::T_DATA_PUBLIC_PRJ && $existsData["t_project_id"] === $destProjectID ){
					$arrDataID[ $dataDatum["id"] ] = $dataDatum["id"];
				} else {
					$isNew = TRUE;
				}
			} else {
				$isNew = TRUE;
			}
			
			if ( $isNew ){
				$resDAOData = $objDAOData->register(INIT_ID, DAO::T_DATA_STATE_NORMAL, NULL, $dataDatum["type"], $dataDatum["title"], $dataDatum["props"], $dataDatum["comment"], $destProjectID, $userID);
				if ( $resDAOData["result"] === RES_NG && $resDAOData["error"] === DAO_data::ERROR_TITLE_DUPLICATE ){
					// 名前が重複している → スキップ
					$this->importMessage[] = "{$WORD($wk, 11)} ({$dataDatum["title"]})";
				} else if ( $resDAOData["result"] !== RES_OK ){
					$this->importMessage[] = "{$WORD($wk, 12)} ({$dataDatum["id"]})";
					return FALSE;
				} else {
					$arrDataID[ $dataDatum["id"] ] = $resDAOData["id"];
				}
			}
			
			if ( isset($arrDataID[$dataDatum["id"]]) ){
				$resDAOProjectData = $objDAOProjectData->register( $destProjectID, $arrDataID[$dataDatum["id"]], $userID);
				if ( $resDAOProjectData["result"] !== RES_OK ){
					$this->importMessage[] = "{$WORD($wk, 13)} ({$arrDataID[$dataDatum["id"]]})";
					return FALSE;
				}
			}
		}
		
		return $arrDataID;
	}
	
	/***************************************************************
	 * 名前:importTask
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function importTask( $destProjectID, $destClear, $userID, $destWorkflowID = NULL, $destBaseDate, $arrUserRole, $arrDocumentID, $arrTermRootID, $arrBilingualRootID, $arrDataID ){
		global $WORD;
		$wk = "WORD_P_CLASS_DOC_PROJECT";
		
		$objDAOWorkflow      = DAO::instance("workflow");
		$objDAOTask          = DAO::instance("task");
		$objDAOTask->setAuth( Auth::instance("task") );
		$objDAOTaskActor     = DAO::instance("task_actor");
		$objDAOTaskDocument      = DAO::instance("task_document");
		$objDAOTaskTermRoot      = DAO::instance("task_term_root", Term::KIND_TERM);
		$objDAOTaskBilingualRoot = DAO::instance("task_term_root", Term::KIND_BILINGUAL);
		$objDAOTaskData          = DAO::instance("task_data");
		
		$arrTaskActor         = array();
		
		if ( ! Validate::isNumeric($destWorkflowID) ){
			$resDAOWorkflow = $objDAOWorkflow->get(NULL, $destProjectID);
			if ( $resDAOWorkflow["result"] === RES_OK ){
				$destWorkflowID = $resDAOWorkflow["data"][0]["t_workflow_id"];
			} else {
				$this->importMessage[] = "{$WORD($wk, 14)}";
				return FALSE;
			}
		}
		
		// 削除
		if ( $destClear ){
			$resDAOTask = $objDAOTask->get(NULL, NULL, NULL, NULL, NULL, $destWorkflowID);
			if ( $resDAOTask["result"] === RES_OK ){
				foreach ( $resDAOTask["data"] as $dataTask ){
					$taskID = $dataTask["t_task_id"];
					//Taskを削除する前に役割の情報を取得する
					$resDAOTaskActor = $objDAOTaskActor->get($taskID);
					if ( $resDAOTaskActor["result"] === RES_OK ){
						$arrTaskActor = array_merge($arrTaskActor, $resDAOTaskActor["data"]);
					}
					//Taskの削除
					$resDAOTaskDelete = $objDAOTask->delete($taskID);
					if ( $resDAOTaskDelete["result"] !== RES_OK ){
						$this->importMessage[] = "{$WORD($wk, 15)} ({$taskID})";
						return FALSE;
					}
				}
			}
		}
		
		// 基準日を計算
		$projectDetail = $this->getDetail();
		$baseDate = $projectDetail["start"];
		if ( Validate::isDate($baseDate) ){
			$offsetDay = (strtotime($destBaseDate) - strtotime($baseDate)) / (60 * 60 * 24);
		} else {
			$offsetDay = 0;
		}
		
		// 登録
		$arrTasks = $this->getTasks();
		foreach ( $arrTasks as $dataTask ){
			$startDate = date("Y-m-d", strtotime($dataTask["start_date"]." {$offsetDay} day"));
			$endDate   = date("Y-m-d", strtotime($dataTask["end_date"]." {$offsetDay} day"));
			
			$resDAOTask = $objDAOTask->register(INIT_ID, DAO::T_TASK_STATE_NORMAL, $dataTask["type"], $dataTask["data"], $startDate, $endDate, $destWorkflowID);
			if ( $resDAOTask["result"] !== RES_OK ){
				$this->importMessage[] = "{$WORD($wk, 16)} ({$dataTask["id"]})";
				return FALSE;
			} else {
				$oldTaskID    = $dataTask["id"];
				$newTaskID    = $resDAOTask["id"];
				$newTaskType  = $dataTask["type"];
				$newTaskState = $dataTask["state"];
				$newTaskData  = $dataTask["data"];
				$newStartDate = $dataTask["start_date"];
				$newEndDate   = $dataTask["end_date"];
				$taskDocument  = $dataTask["document"];
				$taskTerm      = $dataTask["term"];
				$taskBilingual = $dataTask["bilingual"];
				$taskDatum     = $dataTask["datum"];
				$taskActor     = $dataTask["actor"];
				
				// タスクと文書
				if ( is_array($arrDocumentID) && count($arrDocumentID) > 0 ){
					foreach ( $taskDocument as $dataTaskDocument ){
						if ( ! isset($arrDocumentID[$dataTaskDocument["id"]]) ){
							continue;
						}
						$newDocumentID = $arrDocumentID[$dataTaskDocument["id"]];
						$resDAOTaskDocument = $objDAOTaskDocument->register($newTaskID, $dataTaskDocument["props"], $newDocumentID);
						if ( $resDAOTaskDocument["result"] !== RES_OK ){
							$this->importMessage[] = "{$WORD($wk, 17)} ({$oldTaskID})";
							return FALSE;
						}
					}
				}
				// タスクと用語集
				if ( is_array($arrTermRootID) && count($arrTermRootID) > 0 ){
					foreach ( $taskTerm as $dataTaskTermRoot ){
						if ( ! isset($arrTermRootID[$dataTaskTermRoot["id"]]) ){
							continue;
						}
						$newTermRootID = $arrTermRootID[$dataTaskTermRoot["id"]];
						$resDAOTaskTermRoot = $objDAOTaskTermRoot->register($newTaskID, $dataTaskTermRoot["props"], $newTermRootID);
						if ( $resDAOTaskTermRoot["result"] !== RES_OK ){
							$this->importMessage[] = "{$WORD($wk, 18)} ({$oldTaskID})";
							return FALSE;
						}
					}
				}
				// タスクと対訳集
				if ( is_array($arrBilingualRootID) && count($arrBilingualRootID) > 0 ){
					foreach ( $taskBilingual as $dataTaskBilingualRoot ){
						if ( ! isset($arrBilingualRootID[$dataTaskBilingualRoot["id"]]) ){
							continue;
						}
						$newBilingualRootID = $arrBilingualRootID[$dataTaskBilingualRoot["id"]];
						$resDAOTaskBilingualRoot = $objDAOTaskBilingualRoot->register($newTaskID, $dataTaskBilingualRoot["props"], $newBilingualRootID);
						if ( $resDAOTaskBilingualRoot["result"] !== RES_OK ){
							$this->importMessage[] = "{$WORD($wk, 19)} ({$oldTaskID})";
							return FALSE;
						}
					}
				}
				// タスクとデータ
				if ( is_array($arrDataID) && count($arrDataID) > 0 ){
					foreach ( $taskDatum as $dataTaskData ){
						if ( ! isset($arrDataID[$dataTaskData["id"]]) ){
							continue;
						}
						$newTaskDataID = $arrDataID[$dataTaskData["id"]];
						$resDAOTaskData = $objDAOTaskData->register($newTaskID, $dataTaskData["props"], $newTaskDataID);
						if ( $resDAOTaskData["result"] !== RES_OK ){
							$this->importMessage[] = "{$WORD($wk, 20)} ({$oldTaskID})";
							return FALSE;
						}
					}
				}
				// タスクと役割
				if ( is_array($arrUserRole) && count($arrUserRole) > 0 ){
					$arrActor = array();
					foreach ( $taskActor as $dataTaskActor ){
						if ( ! isset($arrUserRole[$dataTaskActor["user_id"]]) ){
							continue;
						}
						if ( ! Validate::inArray($dataTaskActor["role"], $arrUserRole[$dataTaskActor["user_id"]]) ){
							continue;
						}
						$arrActor[$dataTaskActor["role"]][] = $dataTaskActor["user_id"];
					}
					$resDAOTaskActor = $objDAOTaskActor->register($newTaskID, $newTaskType, $arrActor);
					if ( $resDAOTaskActor["result"] !== RES_OK ){
						$this->importMessage[] = "{$WORD($wk, 21)} ({$oldTaskID})";
						return FALSE;
					}
				}
			}
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:xml2text
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function xml2text( $xml ){
	}
	
	/***************************************************************
	 * 名前:xml2table
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function xml2table( $xml ){
	}
}
