<?php require_once("class/Doc.php"); ?>
<?php
Class DocTmx extends Doc {
	const VERSION             = "1.4b";
	const XMLNS               = "http://www.w3.org/1999/xhtml";
	const XMLNS_QREDIT        = "http://trans-aid.jp/";
	const XMLNS_QC            = "http://trans-aid.jp/teachedit";
	const HEADER_CREATIONTOOL = "Minna no Honyaku";
	const HEADER_DATATYPE     = "plaintext";
	const HEADER_SEGTYPE      = "sentence";
	const HEADER_ADMINLANG    = "ja";
	const HEADER_O_TMF        = "minna_no_honyaku";
	const HEADER_O_ENCODING   = "UTF-8";
	const NOTE_ALIGNMENT      = "alignment";

	var $XSD = "qc.xsd";
	
	var $dom;
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct(){
		$this->dom = new DOMDocument("1.0", "UTF-8");
		$this->dom->formatOutput = TRUE;
		$this->dom->preserveWhiteSpace = FALSE;
	}
	
	/***************************************************************
	 * 名前:getRevisionList
	 * 説明:履歴情報
	 * 引数:
	 * 返値:
	****************************************************************/
	function getRevisionList( $xml ){
		// xml
		if ( ! $this->load($xml) ){
			Util::printLog("load xml failed.");
			return FALSE;
		}
		
		// スキーマチェック
		if ( ! $this->validate($errs) ) {
			Util::printLog("validate xml schema failed.");
			foreach ($errs as $errText){
				Util::printLog($errText);
			}
			return FALSE;
		}
		
		$result = array();
		
		$arrNodeHistory = $this->dom->getElementsByTagNameNS(self::XMLNS_QC, "history");
		foreach ( $arrNodeHistory as $nodeHistory ){
			$revision = $nodeHistory->getAttribute("revision");
			$date     = $nodeHistory->getAttribute("date");
			$user     = $nodeHistory->getAttribute("corrector");
			$title    = $nodeHistory->getAttribute("task");
			
			$result[] = array(
				"revision" => $revision,
				"date"     => $date,
				"user"     => $user,
				"title"    => $title,
			);
		}
		
		return $result;
	}
	
	/***************************************************************
	 * 名前:createHeader
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function createHeader( $srcLang, $sLang, $tLang, $note, $dom = NULL ){
		if ( is_null($dom) ){
			$dom = $this->dom;
		}
		
		// node : tmx
		$nodeTmx = $dom->createElement("tmx");
		// tmx@version
		$nodeTmx->setAttribute("version", self::VERSION);
		// tmx@xmlns
		//$nodeTmx->setAttribute("xmlns", self::XMLNS);
		// tmx@xmlns:qredit
		$nodeTmx->setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:qredit", self::XMLNS_QREDIT);
		// tmx@xmlns:qc
		$nodeTmx->setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:qc", self::XMLNS_QC);
		// add
		$dom->appendChild($nodeTmx);
		
		//----------------------
		// node : header
		//----------------------
		$nodeHeader = $dom->createElement("header");
		// attribute : header
		$nodeHeader->setAttribute("creationtool",        self::HEADER_CREATIONTOOL);
		$nodeHeader->setAttribute("creationtoolversion", Setting::get("VERSION"));
		$nodeHeader->setAttribute("creationdate",        date("Y-m-d\TH:i:s"));
		$nodeHeader->setAttribute("datatype",            self::HEADER_DATATYPE);
		$nodeHeader->setAttribute("segtype",             self::HEADER_SEGTYPE);
		$nodeHeader->setAttribute("adminlang",           self::HEADER_ADMINLANG);
		$nodeHeader->setAttribute("srclang",             $srcLang);
		$nodeHeader->setAttribute("o-tmf",               self::HEADER_O_TMF);
		$nodeHeader->setAttribute("o-encoding",          self::HEADER_O_ENCODING);
		// add
		$nodeTmx->appendChild($nodeHeader);
		
		//----------------------
		// qredit:header
		//----------------------
		$nodeQReditHeader = $dom->createElementNS(self::XMLNS_QREDIT, "header");
		// qredit:header@source_language
		$nodeQReditHeader->setAttribute("source_language", $sLang);
		// qredit:header@target_language
		$nodeQReditHeader->setAttribute("target_language", $tLang);
		// qredit:header@original_format
		$nodeQReditHeader->setAttribute("original_format", "plain_text");
		// add
		$nodeTmx->appendChild($nodeQReditHeader);
		
		//----------------------
		// node : note
		//----------------------
		$nodeNote = $dom->createElement("note");
		$nodeNote->appendChild($dom->createCDATASection($note));
		// add
		$nodeHeader->appendChild($nodeNote);
		
		// node : body
		$nodeBody = $dom->createElement("body");
		// add
		$nodeTmx->appendChild($nodeBody);
	}
	
	/***************************************************************
	 * 名前:removeLineNumber
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function removeLineNumber(){
		$arrNodeTu = $this->dom->getElementsByTagName("tu");
		
		foreach ( $arrNodeTu as $nodeTu ){
			$nodeProp = $nodeTu->getElementsByTagName("prop");
			
			if ( $nodeProp->length > 0 ){
				$nodeTu->removeChild($nodeProp->item(0));
			}
		}
	}
	
	/***************************************************************
	 * 名前:getHeaderPropValue
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getHeaderPropValue( $typeValue ){
		return $this->getPropValue($this->dom->getElementsByTagName("header")->item(0), $typeValue);
	}
	
	/***************************************************************
	 * 名前:getPropValue
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getPropValue( $targetNode, $typeValue ){
		$arrNodeProp = $targetNode->getElementsByTagName("prop");
		
		foreach ( $arrNodeProp as $nodeProp ){
			$val = $nodeProp->getAttribute("type");
			
			if ( strcmp($val, "") == 0 ){
				continue;
			}
			
			if ( strcmp($val, $typeValue) == 0 ){
				return $nodeProp->nodeValue;
			}
		}
		
		return NULL;
	}
	
	/***************************************************************
	 * 名前:setHeaderPropValue
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function setHeaderPropValue( $typeValue, $value, $cdata = TRUE ){
		return $this->setPropValue($this->dom->getElementsByTagName("header")->item(0), $typeValue, $value, $cdata);
	}
	
	/***************************************************************
	 * 名前:setHeaderAttr
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function setHeaderAttr( $typeValue, $value ){
		$nodeHeader = $this->dom->getElementsByTagName("header")->item(0);
		$nodeHeader->setAttribute($typeValue,  $value);
	}
	
	/***************************************************************
	 * 名前:setQreditHeaderPropValue
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function setQreditHeaderAttr( $typeValue, $value ){
		if ( ($nodeHeader = $this->dom->getElementsByTagNameNS(self::XMLNS_QREDIT, "header")->item(0)) === NULL ){
			$nodeHeader = $this->dom->createElementNS(self::XMLNS_QREDIT, "header");
			if ( ($nodeTmx = $this->dom->getElementsByTagName("tmx")->item(0)) === NULL ){
				return FALSE;
			} else {
				$nodeTmx->appendChild($nodeHeader);
			}
		}
		$nodeHeader->setAttribute($typeValue,  $value);
	}
	
	/***************************************************************
	 * 名前:setPropValue
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function setPropValue( $targetNode, $typeValue, $value, $cdata = TRUE ){
		$isExists = FALSE;
		
		$arrNodeProp = $targetNode->getElementsByTagName("prop");
		
		foreach ( $arrNodeProp as $nodeProp ){
			$val = $nodeProp->getAttribute("type");
			
			if ( strcmp($val, "") == 0 ){
				continue;
			}
			
			if ( strcmp($val, $typeValue) == 0 ){
				$isExists = TRUE;
				
				if ( $cdata ){
					$nodeProp->nodeValue = "";
					$nodeProp->appendChild($this->dom->createCDATASection($value));
				} else {
					$nodeProp->nodeValue = $value;
				}
				
				break;
			}
		}
		
		if ( ! $isExists ){
			$nodeProp = $this->dom->createElement("prop");
			$nodeProp->setAttribute("type", $typeValue);
			
			if ( $cdata ){
				$nodeProp->appendChild($this->dom->createCDATASection($value));
			} else {
				$nodeProp->nodeValue = $value;
			}
			
			$targetNode->appendChild($nodeProp);
		}
	}
	
	/***************************************************************
	 * 名前:getNote
	 * 説明:
	 * 引数: -
	 * 返値:テキスト
	****************************************************************/
	function getNote(){
		$arrNodeNote = $this->dom->getElementsByTagName("note");
		
		if ( count($arrNodeNote) === 0 ){
			return "";
		}
		
		return $arrNodeNote->item(0)->nodeValue;
	}
	
	/***************************************************************
	 * 名前:setNote
	 * 説明:
	 * 引数: -
	 * 返値:
	****************************************************************/
	function setNote( $text ){
		$arrNodeNote = $this->dom->getElementsByTagName("note");
		
		if ( count($arrNodeNote) === 0 ){
			$nodeNote = $this->dom->createElement("note");
		} else {
			$nodeNote = $arrNodeNote->item(0);
		}
		
		$nodeNote->appendChild($this->dom->createCDATASection($text));
	}
	
	
	/***************************************************************
	 * 名前:getSentence
	 * 説明:テキストgetter
	 * 引数: -
	 * 返値:テキスト
	****************************************************************/
	function getSentence( $allAttribute = FALSE, $dom = NULL ){
		if ( is_null($dom) ){
			$dom = $this->dom;
		}
		$arrRes = array();
		
		$arrNodeBody = $dom->getElementsByTagName("body");
		if ( $arrNodeBody->length >= 1 ){
			$nodeBody = $arrNodeBody->item(0);
		} else {
			return $arrRes;
		}
		
		$arrNodeTU = $nodeBody->getElementsByTagName("tu");
		
		foreach ( $arrNodeTU as $nodeTU ){
			$arrText = array();
			
			$arrNodeProp = $nodeTU->getElementsByTagName("prop");
			// prop要素
			foreach ( $arrNodeProp as $nodeProp ){
				$type  = $nodeProp->getAttribute("type");
				$value = $nodeProp->nodeValue;
				
				$arrText["prop"][$type] = $value;
			}
			
			if ( $allAttribute ){
				foreach ( $nodeTU->attributes as $key => $value ){
					$arrText["@{$value->name}"] = $value->value;
				}
			}
			
			// テキスト取得
			$arrNodeTUV = $nodeTU->getElementsByTagName("tuv");
			
			if ( $arrNodeTUV->item(0) ){
				$arrText["source_lang"] = $arrNodeTUV->item(0)->getAttribute("xml:lang");
				$arrNodeSeg = $arrNodeTUV->item(0)->getElementsByTagName("seg");
				if ( $arrNodeSeg->item(0) ){
					if ( $arrNodeSeg->item(0)->nodeType === XML_TEXT_NODE || 
					     $arrNodeSeg->item(0)->nodeType === XML_CDATA_SECTION_NODE ){
						$arrText["source_text"] = $arrNodeSeg->item(0)->nodeValue;
					} else {
						$arrText["source_text"] = "";
						foreach ( $arrNodeSeg->item(0)->childNodes as $node ){
							if ( $node->nodeType === XML_TEXT_NODE || 
								 $node->nodeType === XML_CDATA_SECTION_NODE ){
								$arrText["source_text"] .= $node->nodeValue;
							} else {
								$arrText["source_text"] .= $dom->saveXML($node);
							}
						}
					}
				} else {
					$arrText["source_text"] = "";
				}
			} else {
				$arrText["source_text"] = "";
			}
			
			if ( $arrNodeTUV->item(1) ){
				$arrText["target_lang"] = $arrNodeTUV->item(1)->getAttribute("xml:lang");
				$arrNodeSeg = $arrNodeTUV->item(1)->getElementsByTagName("seg");
				if ( $arrNodeSeg->item(0) ){
					$arrText["target_text"] = $arrNodeSeg->item(0)->nodeValue;
				} else {
					$arrText["target_text"] = "";
				}
			} else {
				$arrText["target_text"] = "";
			}
			
			$arrRes[] = $arrText;
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:getRevisionSentence
	 * 説明:履歴TSV
	 * 引数:
	 * 返値:
	****************************************************************/
	function getRevisionSentence( $revision, $allAttribute = FALSE ){
		if ( ($xml = $this->getRevisionTsv($revision)) === FALSE ){
			return FALSE;
		}
		
		$doc = new DOMDocument("1.0", "UTF-8");
		$doc->loadXML( $xml );
		$doc->formatOutput = TRUE;
		$doc->preserveWhiteSpace = FALSE;
		$doc->encoding = "UTF-8"; //load後に要設定
		
		return $this->getSentence($allAttribute, $doc);
	}
	
	/***************************************************************
	 * 名前:setSentence
	 * 説明:テキストsetter
	 *     :TMX:body以下を入れ替える
	 * 引数:arrSentence
	 *     :  - id
	 *     :  - source
	 *     :  - target
	 * 返値:テキスト
	****************************************************************/
	function setSentence( $arrSentence ){
		if ( ($root = $this->dom->getElementsByTagName("tmx")->item(0)) === NULL ){
			return FALSE;
		}
		
		if ( strlen($root->getAttributeNS(self::XMLNS_QREDIT, "qredit")) === 0 ){
			// tmx@xmlns:qredit
			$root->setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:qredit", self::XMLNS_QREDIT);
		}
		if ( strlen($root->getAttributeNS(self::XMLNS_QC, "qc")) === 0 ){
			// tmx@xmlns:qc
			$root->setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:qc", self::XMLNS_QC);
		}
		
		$nodeBody = $this->dom->createElement("body");
		$root->replaceChild($nodeBody, $this->dom->getElementsByTagName("body")->item(0));
		//$root->appendChild($nodeBody);
		
		$repid = 0;
		
		foreach ( $arrSentence as $sentence ){
			$bID     = isset($sentence["bilingual_id"]) ? $sentence["bilingual_id"] : "";
			$sLang   = isset($sentence["source_lang"])  ? $sentence["source_lang"]  : "";
			$tLang   = isset($sentence["target_lang"])  ? $sentence["target_lang"]  : "";
			$sText   = isset($sentence["source_text"])  ? $sentence["source_text"]  : "";
			$tText   = isset($sentence["target_text"])  ? $sentence["target_text"]  : "";
			$arrProp = isset($sentence["prop"])         ? $sentence["prop"]         : array();
			
			// tu
			$nodeTU = $this->dom->createElement("tu");
			$nodeBody->appendChild($nodeTU);
			// tu@qredit:repid
			$nodeTU->setAttributeNS(self::XMLNS_QREDIT, "qredit:repid", $repid++);
			// tu@qredit:position
			$nodeTU->setAttributeNS(self::XMLNS_QREDIT, "qredit:position", "all");
			
			// tuv
			$nodeSource = $this->dom->createElement("tuv");
			$nodeTU->appendChild($nodeSource);
			// tuv@xml:lang
			$nodeSource->setAttribute("xml:lang", $sLang);
			// seg
			$nodeSeg = $this->dom->createElement("seg");
			$nodeSource->appendChild($nodeSeg);
			// seg@qredit:type
			$nodeSeg->setAttributeNS(self::XMLNS_QREDIT, "qredit:type", "element");
			// seg value
			$nodeSeg->appendChild($this->dom->createCDATASection($sText));
			
			// tuv
			$nodeTarget = $this->dom->createElement("tuv");
			$nodeTU->appendChild($nodeTarget);
			// tuv@xml:lang
			$nodeTarget->setAttribute("xml:lang", $tLang);
			// seg
			$nodeSeg = $this->dom->createElement("seg");
			$nodeTarget->appendChild($nodeSeg);
			// seg@qredit:type
			$nodeSeg->setAttributeNS(self::XMLNS_QREDIT, "qredit:type", "element");
			// seg value
			$nodeSeg->appendChild($this->dom->createCDATASection($tText));
			
			
			if ( strlen($bID) > 0 && is_numeric($bID) ){
				$nodeID = $this->dom->createElement("prop");
				$nodeID->setAttribute("type", "bilingual_id");
				$nodeID->nodeValue = $bID;
				$nodeTU->appendChild($nodeID);
			}
			
			foreach ( $arrProp as $type => $value ){
				$nodeProp = $this->dom->createElement("prop");
				$nodeProp->setAttribute("type", $type);
				if ( strcmp($type, "comment") === 0 ){
					$nodeProp->appendChild($this->dom->createCDATASection($value));
				}
				else {
					$nodeProp->nodeValue = $value;
				}
				
				$nodeTU->appendChild($nodeProp);
			}
			
		}
		
		
		//--------------------------
		// qredit:original
		//--------------------------
		if ( ($nodeOriginal = $this->dom->getElementsByTagNameNS(self::XMLNS_QREDIT, "original")->item(0)) === NULL ){
			$nodeOriginal = $this->dom->createElementNS(self::XMLNS_QREDIT, "original");
			$root->appendChild($nodeOriginal);
		}
		
		//--------------------------
		// qc:origin
		//--------------------------
		/*
		if ( ($nodeOrigin = $this->dom->getElementsByTagNameNS(self::XMLNS_QC, "origin")->item(0)) !== NULL ){
			if ( ($nodeOriginBody = $nodeOrigin->getElementsByTagName("body")->item(0)) !== NULL ){
				$nodeOrigin->replaceChild($nodeBody->cloneNode(true), $nodeOriginBody);
			}
		}
		 */
	}
	
	/***************************************************************
	 * 名前:getRevisionTsv
	 * 説明:履歴TSV
	 * 引数:
	 * 返値:
	****************************************************************/
	function getRevisionTsv( $revision ){
		//libxml_use_internal_errors(true);
		
		$xpath = new DOMXPath($this->dom);
		if ( (($nodeOriginBody = $xpath->query("/tmx/qc:origin/body")) === FALSE) || ($nodeOriginBody->length < 1) ){
			return FALSE;
		}
		$nodeOriginBody = $nodeOriginBody->item(0);
		//echo htmlspecialchars( $this->dom->saveXML($nodeOriginBody) );
		$content = $this->dom->saveXML($nodeOriginBody);
		
		$qreditNamespace = self::XMLNS_QREDIT;
		$qcNamespace     = self::XMLNS_QC;
$content = <<< HTML
		<tmx xmlns:qredit="{$qreditNamespace}" xmlns:qc="{$qcNamespace}">
			{$content}
		</tmx>
HTML;
		//$content = mb_convert_encoding($content, "HTML-ENTITIES", "UTF-8");
		
		$docRevBody = new DOMDocument("1.0", "UTF-8");
		$docRevBody->formatOutput = TRUE;
		$docRevBody->preserveWhiteSpace = FALSE;
		
		$docRevBody->loadXML( $content );
		$docRevBody->encoding = "UTF-8"; //load後に要設定
		
		$xpathRevBody = new DOMXPath($docRevBody);
		
		$arrNodeHistory = $this->dom->getElementsByTagNameNS(self::XMLNS_QC, "history");
		foreach ( $arrNodeHistory as $nodeHistory ){
			if ( $nodeHistory->getAttribute("revision") <= $revision ){
				$date  = $nodeHistory->getAttribute("date");
				$user  = $nodeHistory->getAttribute("corrector");
				$title = $nodeHistory->getAttribute("task");
				$arrNodeRevtu = $nodeHistory->getElementsByTagName("tu");
				foreach ( $arrNodeRevtu as $nodeRevtu ){
					$repid = $nodeRevtu->getAttribute("repid");
					$arrNodeRevDiff = $nodeRevtu->getElementsByTagName("diff");
					foreach ( $arrNodeRevDiff as $nodeRevDiff ){
						$lang = $nodeRevDiff->getAttribute("lang");
						$from = $nodeRevDiff->getAttribute("range_from");
						$to   = $nodeRevDiff->getAttribute("range_to");
						$replaceText = $nodeRevDiff->getElementsByTagName("modified")->item(0)->nodeValue;
						if ( (($chgNodeElement = $xpathRevBody->query("//body/tu[@qredit:repid=\"{$repid}\"]/tuv[@xml:lang=\"{$lang}\"]")) === FALSE) || $chgNodeElement->length < 1 ){
						//if ( (($chgNodeElement = $xpathRevBody->query("//body/tu[@repid=\"{$repid}\"]/tuv[@xml:lang=\"{$lang}\"]")) === FALSE) || $chgNodeElement->length < 1 ){
//							continue;
						}
						$chgNodeElement = $chgNodeElement->item(0);
						$text = $chgNodeElement->nodeValue;
						$chgNodeElement->nodeValue  = mb_substr($text, 0, $from, "UTF-8");
						$chgNodeElement->nodeValue .= $replaceText;
						$chgNodeElement->nodeValue .= mb_substr($text, $to, NULL, "UTF-8");
					}
				}
			}
		}
		
		//return $docRevBody->saveXML();
		
		if ( ($thisHeader = $this->dom->getElementsByTagName("header")->item(0)) === NULL ){
			return FALSE;
		}
		$docRev = new DOMDocument("1.0", "UTF-8");
		$docRev->formatOutput = TRUE;
		$docRev->preserveWhiteSpace = FALSE;
		$this->createHeader($thisHeader->getAttribute("srclang"), $this->getPropValue($thisHeader, "source_lang"), $this->getPropValue($thisHeader, "target_lang"), $thisHeader->getElementsByTagName("note")->item(0)->nodeValue, $docRev);
		$revHeader = $docRev->getElementsByTagName("header")->item(0);
		$newHeader = $docRev->importNode($thisHeader->cloneNode(TRUE), TRUE);
		$revHeader->parentNode->replaceChild($newHeader, $revHeader);
		
		$newBody = $docRevBody->getElementsByTagName("body")->item(0);
		$oldBody = $docRev->getElementsByTagName("body")->item(0);
		$newBody = $docRev->importNode($newBody->cloneNode(TRUE), TRUE);
		
		$parentNode = $oldBody->parentNode;
		$parentNode->replaceChild($newBody, $oldBody);
		
		$xml = $docRev->saveXML();
		$xml = str_replace("<body xmlns:qredit=\"".self::XMLNS_QREDIT."\">", "<body>", $xml); //replaceしたときに自動で名前空間が保持されているため、無理矢理削除する
		return $xml;
	}

	/***************************************************************
	 * 名前:getLastestTsv
	 * 説明:最新TSV
	 * 引数:
	 * 返値:
	****************************************************************/
	function getLatestTsv( $haveRevision ){
		if ( ($root = $this->dom->getElementsByTagName("tmx")->item(0)) === NULL ){
			return FALSE;
		}
		
		if ( $haveRevision ){
			return $this->dom->saveXML();
		}
		
		$doc = new DOMDocument("1.0", "UTF-8");
		$doc->formatOutput = TRUE;
		$doc->preserveWhiteSpace = FALSE;
		
		$doc->loadXML( $this->dom->saveXML() );
		if ( ($originNode = $doc->getElementsByTagNameNS(self::XMLNS_QC, "origin")->item(0)) !== NULL ){
			$originNode->parentNode->removeChild( $originNode );
		}
		if ( ($historiesNode = $doc->getElementsByTagNameNS(self::XMLNS_QC, "histories")->item(0)) !== NULL ){
			$historiesNode->parentNode->removeChild( $historiesNode );
		}
		$doc->getElementsByTagName("tmx")->item(0)->removeAttributeNS(self::XMLNS_QC, "qc");
		
		return $doc->saveXML();
	}
	
	/***************************************************************
	 * 名前:xml2text
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function xml2text( $xml ){
		$res     = array();
		$result  = RES_OK;
		$sm = array();
		
		// xml
		if ( ! $this->load($xml) ){
			$result = RES_NG;
			Util::printLog("load xml failed.");
		}
		
		// スキーマチェック
		if ( $result === RES_OK && ! $this->validate($errs) ) {
			$result = RES_NG;
			Util::printLog("validate xml schema failed.");
			foreach ($errs as $errText){
				Util::printLog($errText);
			}
		}
		
		if ( $result == RES_OK ){
			$arrSentence = $this->getSentence();
			
			foreach ( $arrSentence as $sentence ){
				// value : source
				$sm["source"] = $sentence["source_text"];
				// value : target
				$sm["target"] = $sentence["target_text"];
				
				$res["data"][] = $sm;
			}
		}
		
		$res["result"] = $result;
		
		return $res;
	}
	
	/***************************************************************
	 * 名前:xml2table
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function xml2table( $xml ){
		$result  = RES_OK;
		$html    = "";
		
		// xml
		if ( ! $this->load($xml) ){
			$result = RES_NG;
			Util::printLog("load xml failed.");
		}
		
		// スキーマチェック
		if ( $result === RES_OK && ! $this->validate($errs) ) {
			$result = RES_NG;
			Util::printLog("validate xml schema failed.");
			foreach ($errs as $errText){
				Util::printLog($errText);
			}
		}
		
		if ( $result == RES_OK ){
			$arrSentence = $this->getSentence();
			
			$isValidTargetText = FALSE;
			foreach ( $arrSentence as $sentence ){
				if ( isset($sentence["target_text"]) && strlen($sentence["target_text"]) > 0 ){
					$isValidTargetText = TRUE;
					break;
				}
			}
			
			foreach ( $arrSentence as $sentence ){
				if ( isset($sentence["source_text"]) ){
					$hSource = Util::esc($sentence["source_text"]);
				} else {
					$hSource = "";
				}
				$tableRow = "<td>{$hSource}</td>";
				
				if ( $isValidTargetText ){
					if ( isset($sentence["target_text"]) ){
						$hTarget = Util::esc($sentence["target_text"]);
					} else {
						$hTarget = "";
					}
					$tableRow .= "<td>{$hTarget}</td>";
				}
				
				$html .= "<tr>{$tableRow}</tr>";
			}
			
$html = <<< HTML
			<table class="table table-striped table-fixed">
				<tbody>
					{$html}
				</tbody>
			</table>
HTML;
		}
		
		return ($result === RES_OK) ? $html : FALSE;
	}
}
