<?php require_once("class/DocTmx.php"); ?>
<?php
Class Document {
	const BATCH_MODE_SOURCE_TEXT = 0;
	const BATCH_MODE_SIMILARITY  = 1;
	const BATCH_MODE_MT          = 2;
	
	const QREDIT_MODE_EDIT             = 1;
	const QREDIT_MODE_CORRECT          = 2;
	const QREDIT_MODE_CORRECT_DISABLED = 3;
	const QREDIT_MODE_COMPARE          = 10;
	
	public static function BATCH_MODE_TABLE($key = NULL){
		global $WORD;
		$wk = "WORD_P_CLASS_DOCUMENT";
		
		$arrTable = array(
			self::BATCH_MODE_SOURCE_TEXT => "{$WORD($wk, 1)}",
			self::BATCH_MODE_SIMILARITY  => "{$WORD($wk, 2)}",
			self::BATCH_MODE_MT          => "{$WORD($wk, 3)}",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	public static function QREDIT_MODE_TABLE($key = NULL){
		global $WORD;
		$wk = "WORD_P_CLASS_DOCUMENT";
		
		$arrTable = array(
			self::QREDIT_MODE_EDIT             => "{$WORD($wk, 4)}",
			self::QREDIT_MODE_CORRECT          => "{$WORD($wk, 5)}",
			self::QREDIT_MODE_CORRECT_DISABLED => "{$WORD($wk, 6)}",
			self::QREDIT_MODE_COMPARE          => "{$WORD($wk, 7)}",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	public static function QREDIT_MODE_COLLECTION_TABLE($key = NULL){
		$arrTable = array(
			self::QREDIT_MODE_EDIT             => "edit",
			self::QREDIT_MODE_CORRECT          => "correct",
			self::QREDIT_MODE_CORRECT_DISABLED => FALSE,
			self::QREDIT_MODE_COMPARE          => "compare",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	var $id = NULL;
	var $xml = NULL;
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function __construct( $_id ){
		$this->id = $_id;
		$this->xml = "";
	}
	
	/***************************************************************
	 * 名前:getRevisionList
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getRevisionList( $data = NULL ){
		$objDocTmx = new DocTmx();
		
		if ( strlen($this->xml) === 0 ){
			if ( is_null($data) ){
				$data = $this->getDocumentData();
			}
			$this->xml = $data["t_document_text"];
		}
		
		return $objDocTmx->getRevisionList($this->xml);
	}
	
	/***************************************************************
	 * 名前:getDocumentData
	 * 説明:タスク取得
	 * 引数:
	 * 返値:
	****************************************************************/
	protected function getDocumentData(){
		$objDAODocument = DAO::instance("document");
		$resDAODocument = $objDAODocument->get($this->id, array_keys(DAO::T_DOCUMENT_STATE_VALID_TABLE()));
		if ( $resDAODocument["result"] !== RES_OK ){
			return FALSE;
		}
		return $resDAODocument["data"][0];
	}
	
	/***************************************************************
	 * 名前:getTaskData
	 * 説明:タスク取得
	 * 引数:
	 * 返値:
	****************************************************************/
	function getTaskData($userID = NULL){
		$arrRes = array();
		
		// 使用しているタスク一覧
		$objDAOTaskDocument = DAO::instance("task_document");
		$resDAOTaskDocument = $objDAOTaskDocument->get(NULL, $this->id);
		if ( $resDAOTaskDocument["result"] !== RES_OK ){
			return FALSE;
		}
		$arrTaskID = array();
		foreach ( $resDAOTaskDocument["data"] as $dataTaskDocument ){
			$arrTaskID[] = $dataTaskDocument["t_task_id"];
		}
		$arrTaskID = array_unique($arrTaskID);
		
		// タスク取得
		$objDAOTask = DAO::instance("task");
		$resDAOTask = $objDAOTask->get($arrTaskID, array(DAO::T_TASK_STATE_NORMAL, DAO::T_TASK_STATE_CARIED, DAO::T_TASK_STATE_FINISH), NULL, NULL, NULL, NULL, $userID);
		if ( $resDAOTask["result"] !== RES_OK ){
			return FALSE;
		}
		foreach ( $resDAOTask["data"] as $dataDAOTask ){
			$id = $dataDAOTask["t_task_id"];
			$arrRes[$id] = $dataDAOTask;
		}
		
		return $arrRes;
	}
	
	/***************************************************************
	 * 名前:isTranslated
	 * 説明:翻訳済みか
	 * 引数:
	 * 返値:
	****************************************************************/
	function isTranslated(){
		if ( ($arrTaskData = $this->getTaskData() ) === FALSE ){
			return FALSE;
		}
		
		// 翻訳タスクが登録されていない場合は、翻訳済みと認識する
		$isTranslated = TRUE;
		foreach ( $arrTaskData as $taskID => $dataTask ){
			if ( $dataTask["t_task_type"] === TASK::TYPE_TRANSLATION_TRANSLATE ){
				if ( $dataTask["t_task_state"] === DAO::T_TASK_STATE_FINISH ){
					// 翻訳タスクのうち、１タスクでも完了状態の場合は翻訳済み状態にする
					$isTranslated = TRUE;
					break;
				} else if ( $dataTask["t_task_state"] === DAO::T_TASK_STATE_NORMAL || $dataTask["t_task_state"] === DAO::T_TASK_STATE_CARIED ){
					// 翻訳タスクが登録されていれば"未翻訳状態にする
					// このタスク以降に完了状態の翻訳タスクが有る場合も考えられるので、for文で繰り返しチェックを行う
					$isTranslated = FALSE;
				}
			}
		}
		
		return $isTranslated;
	}
	
	function canStartTask( $userID, $taskID, $arrTaskData = NULL ){
		if ( ! is_array($arrTaskData) ){
			if ( ($arrTaskData = $this->getTaskData()) === FALSE ){
				return FALSE;
			}
		}
		if ( isset($arrTaskData[$taskID]) ){
			$activeTask = $arrTaskData[$taskID];
		} else {
			return FALSE;
		}
		
		$arrQReditTaskType = array(
			Task::TYPE_TRANSLATION_TRANSLATE,
			Task::TYPE_TRANSLATION_REVISE,
			Task::TYPE_TRANSLATION_REVIEW,
			Task::TYPE_TRANSLATION_PROOF_READ,
			NULL,
		);
		
		// 先にやるべきタスクが終了しているか確認
		$requireTaskType = NULL;
		foreach ( $arrQReditTaskType as $taskType ){
			if ( $taskType === $activeTask["t_task_type"] ){
				// タスクタイプが自分のタスクタイプの時に break することで その前のタスクタイプを保持したままになる。
				break;
			}
			$requireTaskType = $taskType;
		}
		if ( ! is_null($requireTaskType) ){
			$isRequireFinish = TRUE; // $requireTaskTypeが見つからない時は許可する
			foreach ( $arrTaskData as $dataTask ){
				if ( $dataTask["t_task_type"] === $requireTaskType ){
					if ( $dataTask["t_task_state"] === DAO::T_TASK_STATE_FINISH ){
						$isRequireFinish = TRUE;
					} else {
						$isRequireFinish = FALSE;
						break;
					}
				}
			}
			if ( ! $isRequireFinish ){
				// やるべきタスクが終了していない
				return FALSE;
			}
		}
		
		// $arrQReditTaskType以外のタスクが指定された場合も TRUE
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:getQReditMode
	 * 説明:タスク取得
	 * 引数:
	 * 返値:
	****************************************************************/
	function getQReditMode( $userID, $taskID = NULL ){
		if ( ($arrTaskData = $this->getTaskData() ) === FALSE ){
			return FALSE;
		}
		if ( ($arrUserTaskData = $this->getTaskData($userID) ) === FALSE ){
			return FALSE;
		}
		
		$arrQReditTaskType = array(
			Task::TYPE_TRANSLATION_TRANSLATE,
			Task::TYPE_TRANSLATION_REVISE,
			Task::TYPE_TRANSLATION_REVIEW,
			Task::TYPE_TRANSLATION_PROOF_READ,
		);
		
		// タスクIDが指定されていない場合はアクティブなタスクを取得する
		if ( is_null($taskID) ){
			$arrFinishTaskType = array_fill_keys($arrQReditTaskType, FALSE);
			foreach ( $arrTaskData as $dataTask ){
				if ( $dataTask["t_task_state"] === DAO::T_TASK_STATE_FINISH ){
					$arrFinishTaskType[$dataTask["t_task_type"]] = TRUE;
				}
			}
			$activeTaskType = -1;
			foreach ( $arrFinishTaskType as $taskType => $finish ){
				if ( ! $finish ){
					$activeTaskType = $taskType;
					break;
				}
			}
			if ( $activeTaskType === -1 ){
				// すべてのタスクタイプが終了している
				return FALSE;
			}
			
			foreach ( $arrUserTaskData as $dataTask ){
				if ( $dataTask["t_task_type"] === $activeTaskType && $dataTask["t_task_state"] !== DAO::T_TASK_STATE_FINISH ){
					$taskID = $dataTask["t_task_id"];
					break;
				}
			}
			if ( is_null($taskID) ){
				// ユーザーが担当のタスク内に存在しない場合は、担当外のタスクからも検索する
				foreach ( $arrTaskData as $dataTask ){
					if ( $dataTask["t_task_type"] === $activeTaskType && $dataTask["t_task_state"] !== DAO::T_TASK_STATE_FINISH ){
						$taskID = $dataTask["t_task_id"];
						break;
					}
				}
			}
		}
		if ( is_null($taskID) ){
			return FALSE;
		}
		
		if ( isset($arrTaskData[$taskID]) ){
			$activeTask = $arrTaskData[$taskID];
		} else {
			return FALSE;
		}
		
		// やるべきタスクが終了して開始できる状態であるか確認
		if ( ! $this->canStartTask($userID, $taskID, $arrTaskData) ){
			$isTaskMember = FALSE;
		} else {
			$isTaskMember = isset($arrUserTaskData[$taskID]);
		}
		
		switch ( $activeTask["t_task_type"] ){
			// 翻訳タスク
			case TASK::TYPE_TRANSLATION_TRANSLATE:
				return $isTaskMember ? self::QREDIT_MODE_EDIT : FALSE;
			// 修正タスク
			case TASK::TYPE_TRANSLATION_REVISE:
				return $isTaskMember ? self::QREDIT_MODE_CORRECT : self::QREDIT_MODE_CORRECT_DISABLED;
			// レビュータスク
			case TASK:: TYPE_TRANSLATION_REVIEW:
				return $isTaskMember ? self::QREDIT_MODE_CORRECT : self::QREDIT_MODE_CORRECT_DISABLED;
				break;
			// 校正タスク
			case TASK::TYPE_TRANSLATION_PROOF_READ:
				return $isTaskMember ? self::QREDIT_MODE_CORRECT : self::QREDIT_MODE_CORRECT_DISABLED;
				break;
		}
		return FALSE;
	}
	
	/***************************************************************
	 * 名前:getQReditMode_Compare
	 * 説明:比較モードで開くことができるか
	 * 引数:
	 * 返値:
	****************************************************************/
	function getQReditMode_Compare(){
		if ( ($revisions = $this->getRevisionList()) === FALSE ){
			return FALSE;
		}
		return (count($revisions) > 0);
	}
	
	function getQReditModeInfo($qreditMode, $documentID, $taskID = NULL){
		$result = array();
		$urlParam = array(
			"q_id"      => $documentID,
			"q_task_id" => $taskID,
			"q_mode"    => $qreditMode,
		);
		if ( is_null($taskID) ){
			unset($urlParam["q_task_id"]);
		}
		
		switch ( $qreditMode ){
			case self::QREDIT_MODE_EDIT:
				$result = array(
					"text" => self::QREDIT_MODE_TABLE($qreditMode),
					"icon" => "icon-pencil",
					"url"  => "/content/document/editor.html?".http_build_query($urlParam),
					"opt"  => array("target"=>"_blank"),
				);
				break;
			case self::QREDIT_MODE_CORRECT:
				$result = array(
					"text" => self::QREDIT_MODE_TABLE($qreditMode),
					"icon" => "icon-pencil",
					"url"  => "/content/document/editor.html?".http_build_query($urlParam),
					"opt"  => array("target"=>"_blank"),
				);
				break;
			case self::QREDIT_MODE_CORRECT_DISABLED:
				$result = array(
					"text" => self::QREDIT_MODE_TABLE($qreditMode),
					"icon" => "icon-pencil",
					"url"  => "", //"/content/document/editor.html?".http_build_query($urlParam),
					"opt"  => array("target"=>"_blank", "disabled"=>"disabled"),
				);
				break;
			case self::QREDIT_MODE_COMPARE:
				$result = array(
					"text" => self::QREDIT_MODE_TABLE($qreditMode),
					"icon" => "icon-resize-horizontal",
					"url"  => "/content/document/editor.html?".http_build_query($urlParam),
					"opt"  => array("target"=>"_blank"),
				);
				break;
		}
		if ( count($result) > 0 ){
			return $result;
		} else {
			return FALSE;
		}
	}
	
}
