<?php
Class Graph {
	const KIND_INIT       = 0;
	const KIND_TRANSITION = 1;
	
	var $kind = 0;
	var $projectID = 0;
	// option
	var $stage = NULL;
	
	//--------------------------------------------------------------
	public static function KIND_TABLE( $key = NULL ){
		global $WORD;
		$wk = "WORD_P_CLASS_GRAPH";
		
		$arrTable = array(
			self::KIND_TRANSITION => "{$WORD($wk, 1)}",
		);
		
		return is_null($key) ? $arrTable : $arrTable[$key];
	}
	
	/***************************************************************
	 * 名前:__construct
	 * 説明:コンストラクタ
	 * 引数: $kind=TRANSITIONの時、optionには DAO::T_BBS_POST_STAGE_*** を渡す
	 * 返値:
	****************************************************************/
	function __construct( $kind, $projectID, $option = NULL ){
		$this->kind      = $kind;
		$this->projectID = $projectID;
		
		switch ( $this->kind ){
			case self::KIND_TRANSITION:
				$this->_setTransitionProp_Stage( $option );
				break;
			default:
				throw new Exception('Construct Error: Graph');
				break;
		}
	}
	
	/***************************************************************
	 * 名前: property
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	private function _setTransitionProp_Stage($stage){
		if ( ! array_key_exists($stage, DAO::T_BBS_POST_STAGE_TABLE()) ){
			throw new Exception('Property Error: Graph.stage');
		}
		$this->stage = $stage;
	}
	private function _getTransitionProp_Stage(){
		return $this->stage;
	}
	
	/***************************************************************
	 * 名前: getImagePath
	 * 説明: 画像ファイルパス
	 * 引数:
	 * 返値:
	****************************************************************/
	public function getImagePath(){
		switch ( $this->kind ){
			case self::KIND_TRANSITION:
				$stage = DAO::T_BBS_POST_STAGE_TABLE($this->_getTransitionProp_Stage());
				return Setting::get("PATH_DATA_GRAPH_TRANS")."/{$this->projectID}_{$stage}.png";
				break;
		}
		
		throw new Exception('Function Error: Graph.getImagePath()');
	}
	
	/***************************************************************
	 * 名前: fileExists
	 * 説明: 画像ファイルが存在するかどうか
	 * 引数:
	 * 返値:
	****************************************************************/
	public function fileExists(){
		return file_exists($this->getImagePath());
	}
	
	/***************************************************************
	 * 名前: createImage
	 * 説明: 画像ファイルを作成
	 * 引数:
	 * 返値:
	****************************************************************/
	public function createImage(){
		switch ( $this->kind ){
			case self::KIND_TRANSITION:
				return $this->_createImage_Transition();
				break;
		}
		
		throw new Exception('Function Error: Graph.createImage()');
	}
	
	private function _getTransitionRole( $role ){
		if ( array_key_exists($role, Task::ROLE_TABLE()) ){
			$text = Task::ROLE_TABLE($role);
			$text = str_replace("-", "", $text);
			return $text;
		} else {
			return FALSE;
		}
	}
	
	private function _createImage_Transition(){
		$objDAOStatistics = DAO::instance("statistics");
		$resDAOStatistics = $objDAOStatistics->getBbsPostTargetStatisticByProject( $this->projectID );
		if ( $resDAOStatistics["result"] !== RES_OK ){
			return FALSE;
		}
		
		$arrData = $resDAOStatistics["data"];
		
		$arrEdge = array();
		$arrNode = array();
		$no = 0;
		$lastUniqueNo = -1;
		foreach ( $arrData as $data ){
			$uniqueNo   = $data["t_bbs_post_no"];
			$stage      = $data["t_bbs_post_stage"];
			$type       = $data["t_bbs_post_type"];
			$sourceRole = $data["t_bbs_post_role"];
			$targetRole = $data["t_bbs_post_target_role"];
			
			$hSourceRole = $this->_getTransitionRole($sourceRole);
			$hTargetRole = $this->_getTransitionRole($targetRole);
			if ( $hSourceRole === FALSE || $hTargetRole === FALSE ){
				continue;
			}
			
			if ( ($this->_getTransitionProp_Stage() !== $stage) ){
				continue;
			}
			
			// t_bbs_post_no が前列と異なるとき（相手が複数いる場合は、前列と同じになる）
			$isNext = FALSE;
			if ( $lastUniqueNo !== $uniqueNo ){
				$isNext = TRUE;
				$lastUniqueNo = $uniqueNo;
			}
			
			//---------------------
			// node
			//---------------------
			if ( $isNext ){
				if ( ! isset($arrNode[$sourceRole]) ){
					$arrNode[$sourceRole] = 0;
				}
				$arrNode[$sourceRole]++;
			}
			
			//---------------------
			// edge
			//---------------------
			if ( $isNext ){
				$no++;
			}
			$edgeText = "{$hSourceRole} -- {$hTargetRole} [taillabel={$no}]";
			switch ( $type ){
				case DAO::T_BBS_POST_TYPE_DIRQ:
				case DAO::T_BBS_POST_TYPE_DIDF:
				case DAO::T_BBS_POST_TYPE_DISP:
				case DAO::T_BBS_POST_TYPE_DIOF:
					$arrEdge[0][] = $edgeText;
					break;
				case DAO::T_BBS_POST_TYPE_DACK:
				case DAO::T_BBS_POST_TYPE_DMED:
				case DAO::T_BBS_POST_TYPE_DCLA:
				case DAO::T_BBS_POST_TYPE_DMOV:
					$arrEdge[1][] = $edgeText;
					break;
				case DAO::T_BBS_POST_TYPE_DSCH:
				case DAO::T_BBS_POST_TYPE_DSRP:
					$arrEdge[2][] = $edgeText;
					break;
				case DAO::T_BBS_POST_TYPE_DRRQ:
				case DAO::T_BBS_POST_TYPE_DRDC:
				case DAO::T_BBS_POST_TYPE_DRAC:
				case DAO::T_BBS_POST_TYPE_DROF:
					$arrEdge[3][] = $edgeText;
					break;
			}
		}
		
		// node
		$arrNodeText = array();
		foreach ( $arrNode as $role => $roleCount ){
			$hRole = $this->_getTransitionRole($role);
	   		if ( $hRole === FALSE ){
				continue;
			}
			$size = round(sqrt($roleCount) * 2, 1);
			$arrNodeText[] = "{$hRole} [width={$size}, height={$size}]";
		}
		$nodeText = implode("\n", $arrNodeText);
		// edge
		$arrEdgeText = array("", "", "", "");
		foreach ( $arrEdge as $key => $arrEdge ){
			$arrEdgeText[$key] = implode("\n", $arrEdge);
		}
		
$gvText = <<< TEXT
graph preparation_interation {
	graph [overlap=false, sep=1];
	
	/* node */
	{ node [shape=circle, defaultdist=6, fontsize=36];
{$nodeText}
	}
	
	/* common edge */
	edge [penwidth=2, dir=forward, minlen=1.0, labeldistance=2, labelfontsize=24];
	
	/* edge [information-*] */
	{ edge [color=royalblue4];
{$arrEdgeText[0]}
	}
	
	/* edge [maintenance-*] */
	{ edge [color=violetred4];
{$arrEdgeText[1]}
	}
	
	/* edge [status-*] */
	{ edge [color=chartreuse4];
{$arrEdgeText[2]}
	}
	
	/* edge [role-*] */
	{ edge [color=goldenrod4];
{$arrEdgeText[3]}
	}
}
TEXT;
		
		// コードファイル生成
		$codeFile = Setting::get("PATH_DATA_TMP")."/{$this->projectID}.gv";
		if ( file_put_contents($codeFile, $gvText, LOCK_EX) === FALSE ){
			return FALSE;
		}
		
		// 画像ファイル生成
		$cmd = Setting::get("PATH_GRAPHVIZ_TOOL")." -Tpng {$codeFile} -o ".$this->getImagePath();
		exec($cmd, $output, $return_var);
		
		// コードファイル削除
		if ( ! Setting::get("DEBUG") && file_exists($codeFile) ){
			unlink($codeFile);
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前: deleteImage
	 * 説明: 画像ファイルを削除
	 * 引数:
	 * 返値:
	****************************************************************/
	public function deleteImage(){
		$filePath = $this->getImagePath();
		if ( file_exists($filePath) ){
			unlink($filePath);
		}
	}
}
