<?php
setlocale(LC_CTYPE, "en_US.UTF-8");

Class Tool {
	const TYPE_ALIGN  = "align";
	const TYPE_EXTERM = "exterm";
	
	protected static $SUPPORT_LANG_TABLE = array(
		self::TYPE_ALIGN => array(
			LANG_JA => array(LANG_EN),
			LANG_EN => array(LANG_JA),
		),
		self::TYPE_EXTERM => array(
			LANG_JA => array(LANG_EN, LANG_CN, LANG_KO, LANG_TW, LANG_FR, LANG_DE, LANG_RU, LANG_ES),
			LANG_EN => array(LANG_JA, LANG_CN, LANG_KO, LANG_TW, LANG_FR, LANG_DE, LANG_RU, LANG_ES),
			LANG_CN => array(LANG_JA, LANG_EN, LANG_KO, LANG_TW, LANG_FR, LANG_DE, LANG_RU, LANG_ES),
			LANG_KO => array(LANG_JA, LANG_EN, LANG_CN, LANG_TW, LANG_FR, LANG_DE, LANG_RU, LANG_ES),
			LANG_TW => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO, LANG_FR, LANG_DE, LANG_RU, LANG_ES),
			LANG_FR => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO, LANG_TW, LANG_DE, LANG_RU, LANG_ES),
			LANG_DE => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO, LANG_TW, LANG_FR, LANG_RU, LANG_ES),
			LANG_RU => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO, LANG_TW, LANG_FR, LANG_DE, LANG_ES),
			LANG_ES => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO, LANG_TW, LANG_FR, LANG_DE, LANG_RU),
		),
	);
	
	/***************************************************************
	 * 名前:isSupport
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static public function isSupport( $type, $sLang, $tLang ){
		if ( ! isset(self::$SUPPORT_LANG_TABLE[$type][$sLang]) ){
			return FALSE;
		}
		
		return in_array($tLang, self::$SUPPORT_LANG_TABLE[$type][$sLang]);
	}
	
	/***************************************************************
	 * 名前:cmdExec
	 * 説明:一時ファイルを生成して、execを実行させる
	 * 引数:
	 * 返値:
	****************************************************************/
	static protected function cmdExec( $text, $pipeText, &$output, &$return_var ){
		//一時ファイル作成
		$filePathCmd = tempnam(Setting::get("PATH_DATA_TMP"), "CMD");
		$fhandle = fopen($filePathCmd, "w");
		fwrite($fhandle, $text);
		fclose($fhandle);
		
		$escfilePathCmd = escapeshellarg($filePathCmd);
		
		if ( isset($pipeText) ){
			$cmd = "cat {$escfilePathCmd} | {$pipeText}";
		} else {
			$cmd = "cat {$escfilePathCmd}";
		}
		exec($cmd, $output, $return_var);
		
		// 一時ファイル削除
		if ( file_exists($filePathCmd) ) unlink($filePathCmd);
	}
	
	/***************************************************************
	 * 名前:makeDir
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static public function makeDir( $path ){
		if ( ! file_exists($path) ){
			if ( ! mkdir($path) ){
				$result = FALSE;
			}
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:makeFile
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static public function makeFile( $path, $name, $text ){
		if ( ($fp = fopen("{$path}/{$name}", "w")) ){
			fwrite($fp, $text);
			fclose($fp);
		} else {
			$result = FALSE;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:removeDir
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static public function removeDir( $path ){
		if ( $handle = opendir($path) ){
			while ( ($item = readdir($handle)) ){
				if ( $item != "." && $item != ".." ){
					if ( is_dir("{$path}/{$item}") ){
						self::removeDir("{$path}/{$item}");
					} else {
						unlink("{$path}/{$item}");
					}
				}
			}
			
			closedir($handle);
			rmdir($path);
		} else {
			return FALSE;
		}
		
		return TRUE;
	}
}
