<?php require_once("class/Tool.php"); ?>
<?php
Class ToolExTerm extends Tool {
	const DELIMITER = "\t";
	
	var $tempPath = NULL;
	
	private static $CONVERT_LANG = array(
		LANG_CN => "zh",
		LANG_TW => "zh",
	);
		
	/***************************************************************
	 * name    :convertLang
	 * explain :言語コード変換
	 * input   :lang 変換元言語コード
	 * output  :変換先言語コード
	****************************************************************/
	static function convertLang( $lang ){
		return isset(self::$CONVERT_LANG[$lang]) ? self::$CONVERT_LANG[$lang] : $lang;
	}
	
	/***************************************************************
	 * name    :run
	 * explain :
	 * input   :
	 * output  :
	****************************************************************/
	function run( $path, $id, $langS, $langT = NULL ){
		$res    = array();
		$arrRes = array();
		$arrRet = array();
		
		$langS = self::convertLang($langS);
		$langT = self::convertLang($langT);
		
		$escPath  = escapeshellcmd($path);
		$escID    = escapeshellcmd($id);
		$escLangS = escapeshellcmd($langS);
		$escLangT = escapeshellcmd($langT);
		
		$input  = "{$escPath}/{$escID}/{$escID}.tsv";
		$output = "{$escPath}/{$escID}/result";
		if ( strlen($this->tempPath) > 0 ){
			$output = "{$this->tempPath}/{$escID}.result";
		}
		
		// 単言語方向の場合
		if ( is_null($langT) || $langT === LANG_NN ){
			$escLang = "{$escLangS}";
		} else {
			$escLang = "{$escLangS}-{$escLangT}";
		}
		
		//実行
		$cmd  = Setting::get("PATH_EXTERM_HOME") . "/extract.sh";
		$cmd .= " {$escLang}";
		$cmd .= " {$input}";
		$cmd .= " {$output}";
		exec($cmd, $arrRes, $arrRet);
		
		$res["res"] = $arrRes;
		$res["ret"] = $arrRet;
		
		// 単言語方向の場合
		if ( is_null($langT) || $langT === LANG_NN ){
			if ( ! file_exists($output) ){
				return $res;
			}
			
			rename("{$output}", "{$output}.tsv");
			$output .= ".tsv";
		} else {
			if ( ! file_exists("{$output}.{$langS}-{$langT}.txt") ){
				return $res;
			}
			
			rename("{$output}.{$langS}-{$langT}.txt", "{$output}.tsv");
			$output .= ".tsv";
		}
		
		$content = file_get_contents($output);
		$content = preg_replace("/(\r|\n|\r\n)/", "\r\n", $content);
		file_put_contents($output, $content);
		
		if ( strlen($this->tempPath) > 0 ){
			rename($output, "{$escPath}/{$escID}/result.tsv");
		}
		
		return $res;
	}
	
	/***************************************************************
	 * name    :exclude
	 * explain :
	 * input   :
	 * output  :
	****************************************************************/
	function exclude( $path, $id, $arrPathDict, $dictDelimiter = "\t" ){
		$arrRes = array();
		
		if ( count($arrPathDict) === 0 ){
			return TRUE;
		}
		
		// 結果ファイル
		$filename = "{$path}/{$id}/result.tsv";
		
		if ( ($fpR = fopen($filename, "r")) === FALSE ){
			return FALSE;
		}
		
		// 一時ファイル
		$tmpname = tempnam("/tmp", "exterm");
		
		if ( ($fpW = fopen($tmpname, "w")) === FALSE ){
			return FALSE;
		}
		
		while ( ($line = fgets($fpR)) !== FALSE ){
			$line = preg_replace("/(\r|\n)/", "", $line);
			$arr = explode(self::DELIMITER, $line);
			
			$count = array_shift($arr);
			$textS = array_shift($arr);
			$arrTextT = $arr;
			
			$isHit = FALSE;
			
			foreach ( $arrPathDict as $pathDict ){
				if ( ($fpDict = fopen($pathDict, "r")) === FALSE ){
					return FALSE;
				}
				
				while ( ($lineDict = fgets($fpDict)) !== FALSE ){
					$lineDict = preg_replace("/(\r|\n)/", "", $lineDict);
					$arr = explode($dictDelimiter, $lineDict);
					
					$txtS = array_shift($arr);
					
					$arrPattern = array();
					
					// 正規表現
					$p = preg_replace("/\//", "\/", $txtS);
					// 構文チェック
					if ( @preg_match("/{$p}/", "") !== FALSE ){
						$arrPattern[] = $p;
					}
					
					$pattern = implode("|", $arrPattern);
					
					if ( preg_match("/{$pattern}/u", $textS) > 0 ){
						$isHit = TRUE;
						break;
					}
				}
				
				fclose($fpDict);
				
				if ( $isHit ){
					break;
				}
			}
			
			if ( ! $isHit ){
				fwrite($fpW, $line . "\r\n");
			}
		}
		
		fclose($fpR);
		fclose($fpW);
		
		return copy($tmpname, $filename);
	}
	
	/***************************************************************
	 * name    :suggest
	 * explain :
	 * input   :
	 * output  :
	****************************************************************/
	function suggest( $path, $id, $arrPathDict, $dictDelimiter = "\t" ){
		$arrRes = array();
		
		if ( count($arrPathDict) === 0 ){
			return TRUE;
		}
		
		// 結果ファイル
		$filename = "{$path}/{$id}/result.tsv";
		
		if ( ($fpR = fopen($filename, "r")) === FALSE ){
			return FALSE;
		}
		
		// 一時ファイル
		$tmpname = tempnam("/tmp", "exterm");
		
		if ( ($fpW = fopen($tmpname, "w")) === FALSE ){
			return FALSE;
		}
		
		while ( ($line = fgets($fpR)) !== FALSE ){
			$line = preg_replace("/(\r|\n)/", "", $line);
			$arr = explode(self::DELIMITER, $line);
			
			$count = array_shift($arr);
			$textS = array_shift($arr);
			$arrTextT = $arr;
			
			foreach ( $arrPathDict as $pathDict ){
				if ( ($fpDict = fopen($pathDict, "r")) === FALSE ){
					return FALSE;
				}
				
				while ( ($line = fgets($fpDict)) !== FALSE ){
					$line = preg_replace("/(\r|\n)/", "", $line);
					$arr = explode($dictDelimiter, $line);
					
					$txtS = array_shift($arr);
					$txtT = array_shift($arr);
					
					if ( strcmp($textS, $txtS) === 0 ){
						$arrTextT[] = $txtT;
					}
					if ( strcmp($textS, $txtT) === 0 ){
						$arrTextT[] = $txtS;
					}
				}
				
				fclose($fpDict);
			}
			
			$arrTextT = array_unique($arrTextT);
			$arr = array_merge(array($count, $textS), $arrTextT);
			$arr = array_filter($arr, "strlen");
			
			fwrite($fpW, implode(self::DELIMITER, $arr) . "\r\n");
		}
		
		fclose($fpR);
		fclose($fpW);
		
		return copy($tmpname, $filename);
	}
	
	/***************************************************************
	 * name    :mt
	 * explain :
	 * input   :
	 * output  :
	****************************************************************/
	function mt( $path, $id, $arrMTID ){
		$arrRes = array();
		
		if ( count($arrMTID) === 0 ){
			return TRUE;
		}
		
		// 結果ファイル
		$filename = "{$path}/{$id}/result.tsv";
		
		if ( ($fpR = fopen($filename, "r")) === FALSE ){
			return FALSE;
		}
		
		// 一時ファイル
		$tmpname = tempnam("/tmp", "exterm");
		
		if ( ($fpW = fopen($tmpname, "w")) === FALSE ){
			return FALSE;
		}
		
		while ( ($line = fgets($fpR)) !== FALSE ){
			$line = preg_replace("/(\r|\n)/", "", $line);
			$arr = explode(self::DELIMITER, $line);
			
			$count = array_shift($arr);
			$textS = array_shift($arr);
			$arrTextT = $arr;
			
			foreach ( $arrMTID as $mtID ){
				$objMT = new MT($mtID);
				
				$arrTextT[] = $objMT->run($textS);
			}
			
			$arrTextT = array_unique($arrTextT);
			$arr = array_merge(array($count, $textS), $arrTextT);
			$arr = array_filter($arr, "strlen");
			
			fwrite($fpW, implode(self::DELIMITER, $arr) . "\r\n");
		}
		
		fclose($fpR);
		fclose($fpW);
		
		return copy($tmpname, $filename);
	}
}
