<?php require_once("class/Tool.php"); ?>
<?php
Class ToolSimilarity extends Tool {

	/***************************************************************
	 * name    :make
	 * explain : インデックス生成プロシージャ
	 *       TSVファイルを指定の場所にインデックスを作成
	 * input   :
	 * output  : TRUE : 成功
	 *       -1   : インデックス作成に失敗
	 *       -2   : インデックスファイルの移動に失敗
	****************************************************************/
	function make( $tsvFile, $outputDir ){
		$makeIndexDir = "{$outputDir}/sim.index.pre";
		
		//---------------------------------
		// インデックス作成
		//   sim.index.preディレクトリ を作成し、処理成功後、sim.indexディレクトリ にリネームする
		//   ( ※作成処理失敗を考慮して、別名で処理をする）
		//---------------------------------
		$escMakeIndexDir = escapeshellcmd($makeIndexDir);
		$escTsvFile      = escapeshellcmd($tsvFile);
		$cmd  = Setting::get("PATH_SIMILARITY_HOME") . "/make_map";
		$cmd .= " --index-dir {$escMakeIndexDir}";
		$cmd .= " {$escTsvFile}";
		exec($cmd);

		// 移動先（確定したインデックス）
		$moveIndexDir = "{$outputDir}/sim.index";
		
		// 処理が成功
		if ( file_exists($makeIndexDir) ){
			if ( file_exists($moveIndexDir) ){
				// sim.index削除
				$this->removeDir($moveIndexDir);
			}
			
			// 作成したインデックスディレクトリを移動
			if ( rename($makeIndexDir, $moveIndexDir) === FALSE ){
				return -2;
			}
		} else {
			return -1;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * name    :search
	 * explain : 検索プロシージャ
	 * input   :
	 * output  : array() : 成功
	 *       FALSE   : 失敗
	****************************************************************/
	private function search( $text, $score, $arrIndexDir ){
		$res    = array();
		$arrRes = array();
		$arrRet = array();
		
		// スコア
		$escScore = escapeshellcmd($score);
		// インデックスディレクトリ
		foreach ( $arrIndexDir as &$indexDir ){
			$indexDir = escapeshellcmd($indexDir);
		}
		$escIndexDir = implode(",", $arrIndexDir);
		
		//実行
		$cmd  = Setting::get("PATH_SIMILARITY_HOME") . "/search_map";
		$cmd .= " --mode client";
		$cmd .= " --index-dir {$escIndexDir}";
		$cmd .= " --limit-sim {$escScore}";
		$cmd .= " --info 1";
		$this->cmdExec($text, $cmd, $arrRes, $arrRet);
		
		$res["res"] = $arrRes;
		$res["ret"] = $arrRet;
		
		return $res;
	}
	
	/***************************************************************
	 * name    : run
	 * explain : 
	 * input   :
	 * output  : 
	****************************************************************/
	function run( $text, $score, $arrBilingualRootID, $isMerge = TRUE ){
		$result = array();
		
		$arrIndexDir = array();
		foreach ( $arrBilingualRootID as $brID ){
			$simPath = Setting::get("PATH_DATA_SIMILARITY") . "/{$brID}/sim.index";
			if ( file_exists($simPath) ){
				$arrIndexDir[] = $simPath;
			}
		}
		
		$cmdResult = $this->search($text, $score, $arrIndexDir);
		
		if ( ! $isMerge ){
			return $cmdResult;
		}
		
		foreach ( $cmdResult["res"] as $line ){
			// ファイル終端
			if ( strcmp($line, "EOR") == 0 ) {
				break;
			}
			
			// データ処理
			if ( preg_match_all("/^#\ssim=(.*)/", $line, $matches) ) {
				// simの行
				$score = $matches[1][0];
			} else if ( preg_match_all("/^#\smatch=(.*)/", $line, $matches ) ) {
				// matchの行
				$match = $matches[1][0];
			} else {
				// 原文 - 訳文の行
				$arrLineText = explode("\t", $line);
				if ( count($arrLineText) !== 2 ){
					continue;
				}
				list($sText, $tText) = $arrLineText;
				
				$arrMatch = explode(",", $match);
				$arrMatchRange = array();
				foreach ( $arrMatch as $matchPos ){
					$arrMatchPos = explode("-", $matchPos);
					if ( count($arrMatchPos) !== 2 ){
						continue;
					}
					list($fpos, $epos) = $arrMatchPos;
					$len = $epos - $fpos;
					// 範囲の文字列を取得
					$matchStr = mb_substr($sText, $fpos, $len);
					// 空白のみの文字は削除
					if ( mb_strlen(preg_replace("/(\s)/", "", $matchStr)) === 0 ){
						continue;
					}
					// 先頭が空白からはじまる
					if ( preg_match_all("/^(\s+?)/", $matchStr, $fMatche) ) {
						foreach ( $fMatche[0] as $space ) {
							$spaceLength = mb_strlen($space[0]);
							$fpos += $spaceLength;
							$len  -= $spaceLength;
						}
					}
					// 後ろが空白で終わる
					if ( preg_match_all("/\s+?$/", $matchStr, $eMatche ) ) {
						foreach ( $eMatche[0] as $space ) {
							$len -= mb_strlen($space[0]);
						}
					}
					// 空白の要素を除去
					$arrMatchRange[] = $fpos . "-" . $len;
				}
				
				$result[] = array(
					"source" => $sText,
					"target" => $tText,
					"score"  => $score,
					"position" => join(",", $arrMatchRange),
				);
			}
		}
		
		return $result;
	}
}
