<?php
Class Util {
	
	const SET_TYPE_STRING = "string";
	const SET_TYPE_INT    = "int";
	const SET_TYPE_FLOAT  = "float";
	
	/***********************************
	 * 名前:token
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function token( $name = "q_token" ){
		mt_srand((double)microtime()*1000000);
		$token = sha1((string)mt_rand());
		$_SESSION[$name] = $token;
		
		return $token;
	}
	
	/***********************************
	 * 名前:esc
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function esc( $str ){
		return htmlspecialchars($str, ENT_QUOTES, "UTF-8");
	}
	
	/***********************************
	 * 名前:url
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function url( $str ){
		$url = htmlspecialchars($str, ENT_QUOTES, "UTF-8");
		
		if ( preg_match("/^https?:\//", $url) ){
			return $url;
		}
		if ( preg_match("/^\./", $url) ){
			return $url;
		}
		if ( preg_match("/^#/", $url) ){
			return $url;
		}
		
		return Setting::get("URL_HTTP_ROOT") . $url;
	}
	
	/***********************************
	 * 名前:escRichText
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function escRichText( $str ){
		require_once("lib/kses/kses.php");
		
		global $URL;
		
		$arrEscTag = array(
			"p"      => array(),
			"b"      => array(),
			"i"      => array(),
			"u"      => array(),
			"strike" => array(),
			"font"   => array("color" => 1), 
			"a"      => array("href" => array("minlen" => 3, "maxlen" => 100), "title" => array("valueless" => "n")), 
			"hr"     => array(),
			"br"     => array(),
			"div"    => array(),
			"ul"     => array(),
			"li"     => array(),
			"img"    => array("src" => 1),
		);
		
		// 絵文字
		$str = preg_replace("/{@minhon:emoji(\d+)}/", "<img src=\"" . URL_HTTP_EMOJI_ICON . "/$1.png\" alt=\"\" width=\"20\" height=\"20\" />", $str);
		
		// 閲覧ページリンク
		$url = "{$URL('/content/task/viewer/?task_id=')}";
		$str = preg_replace("/\[id:(\d+)\]/", "<a href=\"{$url}$1\">[id:$1]</a>", $str);
		
		return kses($str, $arrEscTag, array("http", "https"));
	}
	
	/***********************************
	 * 名前:dec
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function dec( $str ){
		return htmlspecialchars_decode($str, ENT_QUOTES);
	}
	
	/***********************************
	 * 名前:explode
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function explode( $delimiter, $string ){
		$arrRes = array();
		
		$arr = explode($delimiter, $string);
		
		foreach ( $arr as $a ){
			if ( strcmp(trim($a), "") === 0 ){
				continue;
			}
			
			$arrRes[] = $a;
		}
		
		return $arrRes;
	}
	
	/***********************************
	 * 名前:array_shift_data
	 * 説明:配列から指定した要素を削除する
	 * 引数:
	 * 返値:
	************************************/
	public static function array_shift_data( $array, $data ){
		$res = array();
		foreach ( $array as $a ){
			if ( $a === $data ) continue;
			$res[] = $a;
		}
		return $res;
	}
	
	/***********************************
	 * 名前:array_push_array
	 * 説明:配列の後ろに配列を追加する
	 * 引数:
	 * 返値:
	************************************/
	public static function array_push_array( $arrTarget, $arrPush ){
		foreach ( $arrPush as $val ){
			$arrTarget[] = $val;
		}
		
		return $arrTarget;
	}
	
	/***********************************
	 * 名前:key_array_push
	 * 説明:連想配列を結合する
	 * 引数:
	 * 返値:
	************************************/
	public static function key_array_push( $arrBefore, $arrAfter ){
		$arrRes = array();
		
		foreach ( $arrBefore as $key => $val ){
			$arrRes[$key] = $val;
		}
		foreach ( $arrAfter as $key => $val ){
			$arrRes[$key] = $val;
		}
		
		return $arrRes;
	}
	
	/***********************************
	 * 名前:setType
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function setType( &$value, $type ){
		if ( $value === REQ_NON ){
			// 何もしない
		} else {
			if ( is_array($value) ){
				foreach ( $value as &$v ){
					if ( strcmp(gettype($v), $type) === 0 ){
						// 何もしない
					} else if ( is_array($v) ){
						Util::setType($v, $type);
					} else {
						settype($v, $type);
					}
				}
			} else {
				if ( strcmp(gettype($value), $type) === 0 ){
					// 何もしない
				} else {
					settype($value, $type);
				}
			}
		}
	}
	
	/***********************************
	 * 名前:addSerialNumber
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function addSerialNumber( $arrList, $char = "a" ){
		$arrRes = array();
		
		foreach ( $arrList as $key => $val ){
			$arrRes[$key] = $char . ".   " . $val;
			
			$char = chr(ord($char) + 1);
		}
		
		return $arrRes;
	}
	
	/***********************************
	 * 名前:divideLangs
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function divideLangs( $langs, &$sLang, &$tLang ){
		$arrLang = explode("_", $langs);
		
		if ( count($arrLang) === 2 ){
			$sLang = $arrLang[0];
			$tLang = $arrLang[1];
		} else {
			return FALSE;
		}
		
		return TRUE;
	}
	
	/***********************************
	 * 名前:devideWords
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function devideWords( $words, $div = " " ){
		$res = array();
		
		if ( strcmp($div, " ") === 0 ){
			$words = preg_replace("/　/", " ", $words);
		}
		
		$tmp = explode($div, $words);
		
		foreach ( $tmp as $val ){
			$val = trim($val);
			
			if ( strcmp($val, "") !== 0 ){
				$res[] = $val;
			}
		}
		
		return $res;
	}
	
	/***********************************
	 * 名前:getJSAddress
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function getJSAddress($tableName, $arrAddressID ){
		$arr = array();
		
		$objDAO = new DAO($tableName);
		
		foreach ( $arrAddressID as $addressID ){
			$arrFields = array(
				"{$tableName}_name"
			);
			$arrConditions = array(
				"{$tableName}_id" => $addressID
			);
			$resDAO = $objDAO->_select($arrFields, $arrConditions);
			
			if ( $resDAO["result"] === RES_OK ){
				$addressName  = $resDAO["data"][0]["{$tableName}_name"];
			
				$hAddressID    = Util::esc($addressID);
				$hAddressName  = Util::esc($addressName);
				if ( strcmp($tableName, "t_user") === 0 ){
					$hAddressImage = Image::getUserIcon($addressID);
				} else {
					$hAddressImage = "/img/icon/group.png";
				}
			
				$arr[] = "{'id' : '{$hAddressID}', 'name' : '{$hAddressName}', 'image' : '{$hAddressImage}'}";
			}
		}
		
		return "[".implode(",", $arr)."]";
	}
	
	/***********************************
	 * 名前:makeFile
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	function makeFile( $fileDir, $fileName, $mode = "w", $arrData ){
		$result = RES_OK;
		
		//-------------------------
		// ディレクトリ作成
		//-------------------------
		if ( $result === RES_OK ){
			// ディレクトリが存在しない場合
			if ( ! is_dir($fileDir) ){
				// ディレクトリ作成
				if ( ! mkdir($fileDir) ){
					$result = RES_NG;
					Util::printLog("mkdir {$fileDir} failed.");
				}
			}
		}
		
		//-------------------------
		// ファイル書き込み
		//-------------------------
		if ( $result === RES_OK ){
			if ( $fp = fopen($fileDir . $fileName, $mode) ){
				echo "write file {$fileDir} {$fileName} \n";
				
				foreach ( $arrData as $data ){
					fwrite($fp, $data . "\n");
				}
				
				fclose($fp);
			} else {
				$result = RES_NG;
				Util::printLog("fopen {$fileDir} {$fileName} failed.");
			}
		}
		
		return $result;
	}
	
	/***********************************
	 * 名前:file_get_contents
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function file_get_contents( $filename, $use_include_path = FALSE, $context = NULL, &$header = NULL ){
		$host = Setting::get("PROXY_HOST");
		$port = Setting::get("PROXY_PORT");
		
		if ( strcmp($host, "") !== 0 && strcmp($port, "") !== 0 ){
			$context["http"]["proxy"]           = "{$host}:{$port}";
			$context["http"]["request_fulluri"] = TRUE;
			$context["http"]["timeout"]         = 5;
		}
		
		if ( is_null($context) ){
			if ( ($content = @file_get_contents($filename)) ){
				if ( isset($http_response_header) ){
					$header = $http_response_header;
				}
			}
		} else {
			if ( ($content = @file_get_contents($filename, $use_include_path, stream_context_create($context))) ){
				if ( isset($http_response_header) ){
					$header = $http_response_header;
				}
			}
		}
		
		return $content;
	}
	
	/***********************************
	 * 名前:mimetype
	 * 説明:
	 * 引数:
	 * 返値:
	 ************************************/
	public static function mimetype($filename){
		$mime = "";
		
		$cmd = "file -bi ".escapeshellarg($filename);
		exec($cmd, $output);
		if ( is_array($output) ){
			$mime = $output[0];
		} else {
			$mime = $output;
		}
		
		if ( preg_match("/^application\/(?:x-|)zip(?:;.+|)$/", $mime) ){
			// office2007以上の可能性がある
			// http://colo-ri.jp/develop/2011/09/office2007mime-type.html
			$tmp = Setting::get("PATH_DATA_TMP")."/OOXML_".time();
			exec("unzip {$filename} '\[Content_Types\].xml' -d {$tmp}", $output);
			
			$contentTypesFile = $tmp."/[Content_Types].xml";
			$xml = FALSE;
			if ( file_exists($contentTypesFile) ){
				$xml = file_get_contents($contentTypesFile);
			}
			if ( $xml !== FALSE ){
				if (preg_match("/.*?<Override PartName=\"\/([^\/]+)\/.*/m", $xml, $match)) {
					$t = $match[1];
					if ( $t === "word" ){
						// docx
						$mime = "application/vnd.openxmlformats-officedocument.wordprocessingml.document; charset=binary";
					} else if ( $t === "xl" ){
						// xlsx
						$mime = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet; charset=binary";
					} else if ( $t === "ppt" ){
						// pptx
						$mime = "application/vnd.openxmlformats-officedocument.presentationml.presentation; charset=binary";
					}
				}
			}
			
			if ( file_exists($contentTypesFile) ){
				unlink($contentTypesFile);
			}
			rmdir($tmp);
		}
		
		return $mime;
	}
	
	
	/***********************************
	 * 名前:printLog
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function notFound(){
		global $URL;
		
		header("Location: {$URL('/content/etc/404.html')}");
		exit();
	}
	
	/***********************************
	 * 名前:printLog
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function printLog( $str, $path = NULL ){
		$path = is_null($path) ? Setting::get("PATH_ERROR_LOG") : $path;
		$backtrace = debug_backtrace();
		$str       = "{$backtrace[0]['file']}({$backtrace[0]['line']}){$backtrace[0]['function']} ::: {$str}";
		
		$path           = "{$path}/log";
		$activeFilePath = "{$path}.01";
		
		if ( file_exists($activeFilePath) ){
			if ( filesize($activeFilePath) > Setting::get("SIZE_ERROR_LOG") ){
				for ( $i=9; $i>0; $i-- ){
					$fileFrom = "{$path}." . sprintf("%02d", $i);
					
					if ( file_exists($fileFrom) ){
						$fileTo = "{$path}." . sprintf("%02d", $i+1);
						copy($fileFrom, $fileTo);
					}
				}
				unlink($activeFilePath);
			}
		}
		if ( $fp = fopen($activeFilePath, "a") ){
			flock($fp, LOCK_EX);
			fwrite($fp, date("Y-m-d H:i:s") . " : {$str}\n");
			flock($fp, LOCK_UN);
			fclose($fp);
			
			chmod($activeFilePath, 0777);
		}
	}
}

	/***********************************
	 * 名前:pr
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	function pr( $array ){
		echo "<pre>";
		print_r($array);
		echo "</pre>";
	}
