<?php
Class Validate {
	/***********************************
	 * 名前:isAlphabetNumber
	 * 説明:アルファベットor数値のみ
	 * 引数:
	 * 返値:
	************************************/
	public static function isAlphabetNumber( $val ){
		return preg_match("/^[0-9a-zA-Z]+$/", $val);
	}
	
	/***********************************
	 * 名前:isLowercaseAlphabetNumber
	 * 説明:小文字のアルファベットor数値のみ
	 * 引数:
	 * 返値:
	************************************/
	public static function isLowercaseAlphabetNumber( $val ){
		return preg_match("/^[0-9a-z]+$/", $val);
	}
	
	/***********************************
	 * 名前:isPassword
	 * 説明:パスワード文字列
	 * 引数:
	 * 返値:
	************************************/
	public static function isPassword( $val ){
		return preg_match("/^[!-~]+$/", $val);
	}
	
	/***********************************
	 * 名前:isLang
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isLang( $lang ){
		global $G_LANG_TABLE;
		return array_key_exists($lang, $G_LANG_TABLE);
	}
	
	/***********************************
	 * 名前:isLangScript
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isLangScript( $lang ){
		global $G_LANG_SCRIPT_TABLE;
		return array_key_exists($lang, $G_LANG_SCRIPT_TABLE);
	}
	
	/***********************************
	 * 名前:isLangDirection
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isLangDirection( $sLang, $tLang ){
		global $G_CONFIG_LANG_DIRECTION;
		
		if ( ! isset($G_CONFIG_LANG_DIRECTION[$sLang]) ){
			return FALSE;
		}
		
		return in_array($tLang, $G_CONFIG_LANG_DIRECTION[$sLang]);
	}

	/***********************************
	 * 名前:isCountry
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isCountry( $country ){
		global $G_DB_T_USER_COUNTRY_TABLE;
		
		foreach ( $G_DB_T_USER_COUNTRY_TABLE as $arrCountry ){
			if ( isset($arrCountry[$country]) ){
				return TRUE;
			}
		}
		
		return FALSE;
	}
	
	/***********************************
	 * 名前:isLength
	 * 説明:文字列長
	 * 引数:
	 * 返値:
	************************************/
	public static function isLength( $val, $min, $max ){
		$len = mb_strlen($val);
		return ($min <= $len && $len <= $max);
	}

	/***********************************
	 * 名前:isRangeNum
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isRangeNum( $val, $min, $max = NULL ){
		return is_null($max) ? ($min <= $val) : ($min <= $val && $val <= $max);
	}

	/***********************************
	 * 名前:isNotNull
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isNotNull( $val ){
		return ! is_null($val);
	}

	/***********************************
	 * 名前:isString
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isString( $val ){
		$val = trim($val);
		return (strcmp($val, "") !== 0 && ! is_null($val));
	}

	/***********************************
	 * 名前:isNumeric
	 * 説明:数値
	 * 引数:
	 * 返値:
	************************************/
	public static function isNumeric( $val ){
		return is_numeric($val) && (strcmp($val, "") !== 0) && ! is_null($val);
	}
	
	/***********************************
	 * 名前:isFloat
	 * 説明:浮動小数
	 * 引数:
	 * 返値:
	************************************/
	public static function isFloat( $val ){
		return is_float($val) && (strcmp($val, "") !== 0) && ! is_null($val);
	}
	
	/***********************************
	 * 名前:isEMail
	 * 説明:e-mail
	 * 引数:
	 * 返値:
	************************************/
	public static function isEMail( $val ){
		return filter_var($val, FILTER_VALIDATE_EMAIL);
	}

	/***********************************
	 * 名前:isEqualStr
	 * 説明:文字列が同じか
	 * 引数:
	 * 返値:
	************************************/
	public static function isEqualStr( $val, $target ){
		return strcmp($val, $target) === 0;
	}

	/***********************************
	 * 名前:isArray
	 * 説明:配列か
	 * 引数:
	 * 返値:
	************************************/
	public static function isArray( $val ){
		return is_array($val);
	}

	/***********************************
	 * 名前:isToken
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isToken( $val, $tokenName, $isUnset = TRUE ){
		if ( empty($val) ){
			return FALSE;
		} else if ( isset($_SESSION[$tokenName]) ){
			if ( strcmp($val, $_SESSION[$tokenName]) !== 0 ){
				return FALSE;
			} else {
				if ( $isUnset ){
					unset($_SESSION[$tokenName]);
				}
			}
		} else {
			return FALSE;
		}
		
		return TRUE;
	}

	/***********************************
	 * 名前:inArray
	 * 説明:配列に含まれているか
	 * 引数:
	 * 返値:
	************************************/
	public static function inArray( $val, $array ){
		foreach ( $array as $v ){
			if ( $val === $v ){
				return TRUE;
			}
		}
		return FALSE;
	}

	/***********************************
	 * 名前:inKeyArray
	 * 説明:配列キーに含まれているか
	 * 引数:
	 * 返値:
	************************************/
	public static function inKeyArray( $val, $array ){
		return array_key_exists($val, $array);
	}

	/***********************************
	 * 名前:isDateTime
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isDateTime( $str ){
		if ( preg_match("/^(\d\d\d\d)-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)$/", $str, $match) ){
			if ( mktime($match[4], $match[5], 0, $match[2], $match[3], $match[1]) ){
				return TRUE;
		    }
		}
		
		return FALSE;
	}
	
	/***********************************
	 * 名前:isDate
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isDate( $str ){
		if ( preg_match("/^(\d\d\d\d)-(\d\d)-(\d\d)$/", $str, $match) ){
			if ( mktime(0, 0, 0, $match[2], $match[3], $match[1]) > mktime(0, 0, 0, 1, 1, 2000) ){
				return TRUE;
			}
		}
		
		return FALSE;
	}
	
	/***********************************
	 * 名前:isDateFuture
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isDateFuture( $str, $target = "now" ){
		if ( self::isDate($str) ){
			if ( ($time = strtotime($str)) ){
				if ( $time >= strtotime($target) ){
					return TRUE;
				}
			}
		}
		
		return FALSE;
	}
	
	/***********************************
	 * 名前:isOrderParam
	 * 説明:
	 * 引数:
	 * 返値:
	************************************/
	public static function isOrderParam( $param, $arrOrderNameTable, $arrOrderTypeTable){
		if ( ! Validate::isString($param) ){
			return FALSE;
		}
		$arrStr = explode("-", $param);
		if ( count($arrStr) !== 2 ){
			return FALSE;
		}
		
		$name = array_shift($arrStr);
		$type = array_shift($arrStr);
		if ( ! Validate::inKeyArray($name, $arrOrderNameTable) ){
			return FALSE;
		}
		if ( ! Validate::inKeyArray($type, $arrOrderTypeTable) ){
			return FALSE;
		}
		
		return TRUE;
	}
}
