<?php require_once("lib/OAuth/OAuthStore.php"); ?>
<?php require_once("lib/OAuth/OAuthRequester.php"); ?>
<?php
Class WebAPI {
	// 機械翻訳
	const TRANSLATOR_ENGINE_NAME_GENERAL = "general"; // NICT Translator (general)
	const EXTERM_DELIMITER = "\t";
	const TYPE_EXTERM_WEB_API    = "exterm_webapi";
	const TYPE_TRANSLATE_WEB_API = "translate_webapi";


	var $URL_WEBAPI    = NULL;
	var $USER_WEBAPI   = NULL;
	var $KEY_WEBAPI    = NULL;
	var $SECRET_WEBAPI = NULL;
	
	private static $CONVERT_LANG = array(
		LANG_CN => LANG_ZH_CN,
	);
	
	protected static $SUPPORT_WEB_API_LANG_TABLE = array(
		self::TYPE_EXTERM_WEB_API    => array(
			LANG_JA => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO),
			LANG_EN => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO),
			LANG_CN => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO),
			LANG_KO => array(LANG_JA, LANG_EN, LANG_CN, LANG_KO),
		),
		self::TYPE_TRANSLATE_WEB_API => array(
			LANG_JA => array(LANG_EN, LANG_CN, LANG_KO),
			LANG_EN => array(LANG_JA),
			LANG_CN => array(LANG_JA),
			LANG_KO => array(LANG_JA),
		),
	);
	
	/***************************************************************
	 * name    :convertLang
	 * explain :言語コード変換
	 * input   :lang 変換元言語コード
	 * output  :変換先言語コード
	****************************************************************/
	static function convertLang( $lang ){
		return isset(self::$CONVERT_LANG[$lang]) ? self::$CONVERT_LANG[$lang] : $lang;
	}
	
	/***************************************************************
	 * 名前:isSupport
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static public function isSupport( $type, $sLang, $tLang ){
		if ( ! isset(self::$SUPPORT_WEB_API_LANG_TABLE[$type][$sLang]) ){
			return FALSE;
		}
		
		return in_array($tLang, self::$SUPPORT_WEB_API_LANG_TABLE[$type][$sLang]);
	}

	/***************************************************************
	 * 名前:WebAPI
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function WebAPI($apiName = NULL, $apiKey = NULL, $apiSecret = NULL){
		// WebAPI接続情報
		$this->URL_WEBAPI    = Setting::get("URL_WEBAPI");
		$this->USER_WEBAPI   = Setting::get("USER_WEBAPI");
		if (!( $apiName === NULL )){
			$this->USER_WEBAPI = $apiName;
		}
		$this->KEY_WEBAPI    = Setting::get("KEY_WEBAPI");
		if (!( $apiKey === NULL )){
			$this->KEY_WEBAPI = $apiKey;
		}
		$this->SECRET_WEBAPI = Setting::get("SECRET_WEBAPI");
		if (!( $apiSecret === NULL )){
			$this->SECRET_WEBAPI = $apiSecret;
		}
	}
	
	/***************************************************************
	 * 名前:getListBilingual
	 * 説明: 対訳集データ一覧取得
	 * 引数:
	 * 返値:
	****************************************************************/
	function getListBilingual( $lang_s, $lang_t, $limit, $offset ){
		$res = array();
		
		// URL設定
		$apiUrl = $this->URL_WEBAPI . "/list/bilingual/";

		// パラメータ設定
		$params = array(
			"key"    => $this->KEY_WEBAPI,
			"name"   => $this->USER_WEBAPI,
			"type"   => "xml",
			"lang_s" => $lang_s,
			"lang_t" => $lang_t,
			"limit"  => $limit,
			"offset" => $offset,
		);

		////////////////
		// WebAPI実行 //
		////////////////
		$apiRes =  $this->webApiExec($this->KEY_WEBAPI , $this->SECRET_WEBAPI , $apiUrl , $params);
		// 結果判定（１回ループ）
		do {
			if ($apiRes == FALSE){
				$res = FALSE;
				break;
			}
			
			// body部取得
			$bodyText = $apiRes['body'];
			$xml = $this->getXML_fromBody($bodyText);
			// xml抽出結果チェック
			if ($xml === ""){
				$res = FALSE;
				Util::printLog("APIError:Return[" . $bodyText ."]");
				break;
			}
			
			// APIエラーチェック
			if ($xml['code'] !== "0"){
				$res = FALSE;
				Util::printLog("APIError:" . $xml['code'] . "(" . mb_convert_encoding($xml['message'], "SJIS", "UTF-8") . ")");
				break;
			}
			
			// 一覧取得
			if ( $xml['result']['list'] ){
				// 取得件数が１件の場合
				if ( array_key_exists ( 'id' , $xml['result']['list']['item'] ) ){
					$res[] = array("id"              => $xml['result']['list']['item']['id'],
								   "name"            => $xml['result']['list']['item']['name'],
								   "source_language" => $xml['result']['list']['item']['source_language'],
								   "target_language" => $xml['result']['list']['item']['target_language'] );
				// 上記以外の場合
				} else {
					foreach ( $xml['result']['list']['item'] as $item ){
						$res[] = array("id"              => $item['id'],
									   "name"            => $item['name'],
									   "source_language" => $item['source_language'],
									   "target_language" => $item['target_language'] );
					}
				}
			}
		} while (FALSE);
		return $res;
	}

	/***************************************************************
	 * 名前:getSimilarity
	 * 説明: 類似文検索
	 * 引数:
	 * 返値:
	****************************************************************/
	function getSimilarity( $text, $score, $arrBilingualRootID, $limit ){
		$res = array();
		
		// URL設定
		$apiUrl = $this->URL_WEBAPI . "/sim/";
		
		$pid = "";
		foreach ( $arrBilingualRootID as $brID ){
			$pid .= "{$brID},";
		}
		
		// パラメータ設定
		$params = array(
			"key"    => $this->KEY_WEBAPI,
			"name"   => $this->USER_WEBAPI,
			"type"   => "xml",
			"text"   => $text,
			"pid"    => $pid,
			"score"  => $score,
			"limit"  => $limit,
		);

		////////////////
		// WebAPI実行 //
		////////////////
		$apiRes =  $this->webApiExec($this->KEY_WEBAPI , $this->SECRET_WEBAPI , $apiUrl , $params);
		// 結果判定（１回ループ）
		do {
			if ($apiRes == FALSE){
				$res = FALSE;
				break;
			}
			
			// body部取得
			$bodyText = $apiRes['body'];
			$xml = $this->getXML_fromBody($bodyText);
			// xml抽出結果チェック
			if ($xml === ""){
				$res = FALSE;
				Util::printLog("APIError:Return[" . $bodyText ."]");
				break;
			}
			
			// APIエラーチェック
			if ($xml['code'] !== "0"){
				$res = FALSE;
				Util::printLog("APIError:" . $xml['code'] . "(" . mb_convert_encoding($xml['message'], "SJIS", "UTF-8") . ")");
				break;
			}
			
			// 一覧取得
			if ( $xml['result']['sim'] ){
				// 取得件数が１件の場合
				if ( array_key_exists ( 'source' , $xml['result']['sim']['item'] ) ){
					$res[] = array(
						"source"   => $xml['result']['sim']['item']['source'],
						"target"   => $xml['result']['sim']['item']['target'],
						"score"    => $xml['result']['sim']['item']['score'],
						"position" => $xml['result']['sim']['item']['position'],
					);
				// 上記以外の場合
				} else {
					foreach ( $xml['result']['sim']['item'] as $item ){
						$res[] = array(
							"source"   => $item['source'],
							"target"   => $item['target'],
							"score"    => $item['score'],
							"position" => $item['position'],
						);
					}
				}
			}
		} while (FALSE);
		return $res;
	}

	/***************************************************************
	 * 名前:convertGet
	 * 説明: 文書コンバート(Get)
	 * 引数:
	 * 返値:
	****************************************************************/
	function convertGet( $ext , $lang_s , $lang_t , $file , $outputFilePath , $ext_t){
		$res = "";

		// URL設定
		$apiUrl = $this->URL_WEBAPI . "/convert/get/";

		// パラメータ設定
		$params = array(
			"key"    => $this->KEY_WEBAPI,
			"name"   => $this->USER_WEBAPI,
			"type"   => "xml",
			"ext"    => $ext,
			"ext_t"  => $ext_t,
			"lang_s" => $lang_s,
			"lang_t" => $lang_t,
		);
		$fileParams = array("file" => array("file" => $file));

		////////////////
		// WebAPI実行 //
		////////////////
		$apiRes =  $this->webApiExec($this->KEY_WEBAPI , $this->SECRET_WEBAPI , $apiUrl , $params , $fileParams);
		// 結果判定（１回ループ）
		do {
			if ($apiRes == FALSE){
				$res = FALSE;
				break;
			}
			
			// HTTPコード取得
			if (array_key_exists('code', $apiRes)){
				$CODE_HTTP = $apiRes['code'];
			} else {
				$res = FALSE;
				Util::printLog("APIError:HTTPCode Not Exist");
				break;
			}
			
			// HTTPコード判定
			if ($CODE_HTTP !== "200"){
				$res = FALSE;
				Util::printLog("APIError:HTTPError code[$CODE_HTTP]");
				break;
			}
			
			// body部取得
			$xmlBody = "";
			$bodyText = $apiRes['body'];
			$xml = explode("<?xml ", $bodyText);
			if (isset($xml[1])){
				$xmlBody = "<?xml " . $xml[1];
			}
			// xml抽出結果チェック
			if ($xmlBody === ""){
				$res = FALSE;
				Util::printLog("APIError:Return[" . $bodyText ."]");
				break;
			}

			// xml抽出結果出力
			if ( file_put_contents($outputFilePath, $xmlBody) ) {
				$res = TRUE;
				break;
			} else {
				$res = FALSE;
				Util::printLog("APIError: OutPutFile[$outputFilePath]");
				break;
			}
		} while (FALSE);
		return $res;
	}

	/***************************************************************
	 * 名前:autoTranslate
	 * 説明: 自動翻訳
	 * 引数:
	 * 返値:
	****************************************************************/
	function autoTranslate( $engineName, $sLang, $tLang, $text ){
		$res = "";
		
		// 言語方向変換
		$paramLangS = self::convertLang($sLang);
		$paramLangT = self::convertLang($tLang);

		// URL設定
		$apiUrl = $this->URL_WEBAPI . "/mt/" . $engineName . "_" . $paramLangS . "_" . $paramLangT . "/";

		// パラメータ設定
		$params = array(
			"key"  => $this->KEY_WEBAPI,
			"name" => $this->USER_WEBAPI,
			"type" => "xml",
			"text" => $text,
			"split" => 0,     // 文章区切り(0:OFF / 1:ON)
			"data" => "Logging Data",
		);

		////////////////
		// WebAPI実行 //
		////////////////
		$apiRes =  $this->webApiExec($this->KEY_WEBAPI , $this->SECRET_WEBAPI , $apiUrl , $params);
		// 結果判定（１回ループ）
		do {
			if ($apiRes == FALSE){
				$res = FALSE;
				break;
			}
			
			// body部取得
			$bodyText = $apiRes['body'];
			$xml = $this->getXML_fromBody($bodyText);
			// xml抽出結果チェック
			if ($xml === ""){
				$res = FALSE;
				Util::printLog("APIError:Return[" . $bodyText ."]");
				break;
			}
			
			// APIエラーチェック
			if ($xml['code'] !== "0"){
				$res = FALSE;
				Util::printLog("APIError:" . $xml['code'] . "(" . mb_convert_encoding($xml['message'], "SJIS", "UTF-8") . ")");
				break;
			}
			
			// 翻訳値取得
			$res = $xml['result']['text'];
		} while (FALSE);
		return $res;
	}

	/***************************************************************
	 * 名前:exterm
	 * 説明: 用語抽出
	 * 引数:
	 * 返値:
	****************************************************************/
	function exterm( $inputFile , $outputFile , $langS , $langT ){
		$res = "";

		// URL設定
		$apiUrl = $this->URL_WEBAPI . "/exterm/";

		// 言語方向変換
		$paramLangS = self::convertLang($langS);
		$paramLangT = self::convertLang($langT);
		
		// 言語方向が「単言語」の場合
		if ( $paramLangS === $paramLangT ) {
			$paramLangT = NULL;
		}
		
		// パラメータ設定
		$params = NULL;
		if ($paramLangT === NULL){
			$params = array(
				"key"    => $this->KEY_WEBAPI,
				"name"   => $this->USER_WEBAPI,
				"type"   => "xml",
				"lang_s" => $paramLangS,
			);
		} else {
			$params = array(
				"key"    => $this->KEY_WEBAPI,
				"name"   => $this->USER_WEBAPI,
				"type"   => "xml",
				"lang_s" => $paramLangS,
				"lang_t" => $paramLangT,
			);
		}
		$fileParams = array("file" => array("file" => $inputFile));

		////////////////
		// WebAPI実行 //
		////////////////
		$apiRes =  $this->webApiExec($this->KEY_WEBAPI , $this->SECRET_WEBAPI , $apiUrl , $params , $fileParams);
		// 結果判定（１回ループ）
		do {
			if ($apiRes == FALSE){
				$res = FALSE;
				break;
			}
			
			// HTTPコード取得
			if (array_key_exists('code', $apiRes)){
				$CODE_HTTP = $apiRes['code'];
			} else {
				$res = FALSE;
				Util::printLog("APIError:HTTPCode Not Exist");
				break;
			}
			
			// HTTPコード判定
			if ($CODE_HTTP !== "200"){
				$res = FALSE;
				Util::printLog("APIError:HTTPError code[$CODE_HTTP]");
				break;
			}
			
			// body部取得
			$xmlBody = "";
			$bodyText = $apiRes['body'];
			$xml = $this->getXML_fromBody($bodyText);
			// APIエラーチェック
			if ($xml !== ""){
				if ($xml['code'] !== "0"){
					$res = FALSE;
					Util::printLog("APIError:" . $xml['code'] . "(" . mb_convert_encoding($xml['message'], "SJIS", "UTF-8") . ")");
					break;
				}
			}
			
			// 抽出結果出力
			file_put_contents($outputFile, $bodyText);
			$res = TRUE;
			break;
		} while (FALSE);
		return $res;
	}

	/***************************************************************
	 * 名前:getXML_fromBody
	 * 説明: XML抽出(内部メソッド)
	 * 引数:
	 * 返値:
	****************************************************************/
	private function getXML_fromBody($bodyText = NULL) {

		$res = "";
		$xml = explode("<?xml ", $bodyText);
		if (isset($xml[1])){
			$xmlBody = "<?xml " . $xml[1];
			$xmlObject = simplexml_load_string($xmlBody);
			$res = json_decode( json_encode( $xmlObject ), TRUE );
		}
		
		return $res;
	}

	/***************************************************************
	 * 名前:webApiExec
	 * 説明: WebAPI実行(内部メソッド)
	 * 引数:
	 * 返値:
	****************************************************************/
	private function webApiExec($KEY = NULL , $SECRET = NULL , $URL = NULL , $params = NULL , $fileParams = NULL) {

		$method = "POST";
		$options = array("consumer_key" => $KEY, "consumer_secret" => $SECRET);
		OAuthStore::instance("2Leg", $options);

		Util::printLog("ExecuteWebAPI:" . $URL);

		try {
			$request = new OAuthRequester($URL, $method, $params , null , $fileParams);
			$result = $request->doRequest();
			return $result;
		} catch(OAuthException2 $e) {
			Util::printLog($e->getMessage());
			return FALSE;
		}

	}
}
