<?php require_once("class/DifferenceEngine.php"); ?>
<?php

class _DiffSimpleEngine extends _DiffEngine {

    function diff($from_lines, $to_lines) {
        $edits = array();
        if ( is_array($from_lines) && count($from_lines) > 0 ){
            $from_keys = array_keys($from_lines);
            while ($key = array_shift($from_keys)){
                $xline = $from_lines[$key];
                $yline = isset($to_lines[$key]) ? $to_lines[$key] : NULL;
                if ( ! is_null($yline) ){
                    unset($to_lines[$key]);
                }

                if ( $xline === $yline ){
                    $edits[$key] = new _DiffOp_Copy(array($xline));
                }
                else if ( strlen($xline) > 0 && strlen($yline) > 0 ){
                    $edits[$key] = new _DiffOp_Change(array($xline), array($yline));
                } else if ( strlen($xline) > 0 ){
                    $edits[$key] = new _DiffOp_Delete(array($xline));
                } else if ( strlen($yline) > 0 ){
                    $edits[$key] = new _DiffOp_Add(array($yline));
                } else {
                    $edits[$key] = new _DiffOp_Delete(array($xline));
                }
            }
        }
        if ( is_array($to_lines) && count($to_lines) > 0 ){
            foreach ( $to_lines as $key => $yline ){
                $edits[$key] = new _DiffOp_Add(array($yline));
            }
        }

        return $edits;
    }
}

class SimpleDiff extends Diff {

    function __construct($from_lines, $to_lines) {
        $eng = new _DiffSimpleEngine;
        $this->edits = $eng->diff($from_lines, $to_lines);
    }
}

class _HWLDF_XMLAccumulator extends _HWLDF_WordAccumulator {

    function _flushGroup($new_tag) {
        if ($this->_group !== '') {
            if ($this->_tag == 'mark')
                $this->_line .= '<x>'.$this->_escape($this->_group).'</x>';
            elseif ($this->_tag == 'add')
                $this->_line .= '<xadd>'.$this->_escape($this->_group).'</xadd>';
            elseif ($this->_tag == 'del')
                $this->_line .= '<xdel>'.$this->_escape($this->_group).'</xdel>';
            else
                $this->_line .= $this->_escape($this->_group);
        }
        $this->_group = '';
        $this->_tag = $new_tag;
    }
}

class XMLLevelDiff extends WordLevelDiff {

    function orig() {
        $orig = new _HWLDF_XMLAccumulator;

        foreach ($this->edits as $edit) {
            if ($edit->type == 'copy')
                $orig->addWords($edit->orig);
            elseif ($edit->orig)
                $orig->addWords($edit->orig, 'mark');
        }
        return $orig->getLines();
    }

    function closing() {
        $closing = new _HWLDF_XMLAccumulator;

        foreach ($this->edits as $edit) {
            if ($edit->type == 'copy')
                $closing->addWords($edit->closing);
            elseif ($edit->closing)
                $closing->addWords($edit->closing, 'mark');
        }
        return $closing->getLines();
    }
}

class XMLDiffFormatter extends DiffFormatter {
    function __construct() {
        $this->leading_context_lines = 2;
        $this->trailing_context_lines = 2;
    }

    function _block_header($xbeg, $xlen, $ybeg, $ylen) {
        return "";
    }

    function _start_block($header) {
        echo "<root>";
    }

    function _end_block() {
        echo "</root>";
    }

    function _lines($lines, $prefix=' ', $color="white") {
    }

    function line($line,$escaped=false) {
        if (!$escaped){
            $line = $this->_escape($line);
        }
        return '<col>'.$line.'</col>';
    }
    function emptyLine() {
        return '<col></col>';
    }

    function addedLines($lines,$escaped=false) {
        foreach ($lines as $line) {
            print('<row type="add">' . $this->emptyLine() . $this->line($line,$escaped) . "</row>\n");
        }
    }
    function deletedLines($lines,$escaped=false) {
        foreach ($lines as $line) {
            print('<row type="delete">' . $this->line($line,$escaped) . $this->emptyLine() . "</row>\n");
        }
    }
    function contextLines($lines,$escaped=false) {
        foreach ($lines as $line) {
            print('<row type="copy">' . $this->line($line,$escaped) . $this->line($line,$escaped) . "</row>\n");
        }
    }

    function _added($lines) {
        $this->addedLines($lines,false);
    }

    function _deleted($lines) {
        $this->deletedLines($lines,false);
    }

    function _context($lines) {
        $this->contextLines($lines,false);
    }

    function _changed($orig, $closing) {
        $diff = new XMLLevelDiff($orig, $closing);  // this escapes the diff data
        $del = $diff->orig();
        $add = $diff->closing();

        while ($line = array_shift($del)) {
            $aline = array_shift($add);
            print('<row type="change">' . $this->line($line,true) . $this->line($aline,true) . "</row>\n");
        }
        $this->addedLines($add,true); # If any leftovers
    }

    function _escape($str) {
        return htmlspecialchars($str);
    }
}


//Setup VIM: ex: et ts=4 :
