<?php
define("LIFETIME",   60 * 60 * 24 * 30); // 最大30日間はセッションを保持する
define("LOGOUTTIME", 60 * 60 *  3);      // 自動ログアウト時間(指定の時間アクセスがなかった場合、ログアウト処理をする)

// セッション(サーバー側)の有効期限[秒]
ini_set('session.gc_maxlifetime', LIFETIME);
// セッションクッキー(クライアント側)の有効期限[秒]
session_set_cookie_params( LIFETIME );
// セッション開始
session_start();

$lastAccessTime = isset($_SESSION["lastAccessTime"]) ? $_SESSION["lastAccessTime"] : 0;
if ( (time() - $lastAccessTime) > LOGOUTTIME ){
	// ログアウト処理をする
	unset($_SESSION["logged"]);
	
	if ( isset($_SERVER["HTTP_X_REQUESTED_WITH"]) && strtolower($_SERVER["HTTP_X_REQUESTED_WITH"]) === 'xmlhttprequest' ){
		// ajaxでアクセス
		$skipAccessTime = TRUE;
	} else {
		// 再ログイン後戻るページを指定
		$_SESSION["requestUri"] = $_SERVER["REQUEST_URI"];
	}
}
// 最終アクセス時間を更新したくない場合は Common.php 読み込み前に $skipAccessTime = TRUE; とする
if ( isset($skipAccessTime) && $skipAccessTime ){
	$skipAccessTime = FALSE;
} else {
	$_SESSION["lastAccessTime"] = time();
}

// 環境変数
require_once(dirname($_SERVER["DOCUMENT_ROOT"]) . "/setting.ini");
// 定数定義
require_once("common/Define.php");
// Utilクラス
require_once("class/Util.php");
// Toolクラス
require_once("class/Tool.php");
// WebAPIクラス
require_once("class/WebAPI.php");
// Validateクラス
require_once("class/Validate.php");
// データベースクラス
require_once("class/DAO.php");
// ビュークラス
require_once("class/View.php");
// Authクラス
require_once("class/Auth.php");
// ユーザークラス
require_once("class/User.php");
// ヘルプトクラス
require_once("class/Help.php");
// プロジェクトクラス
require_once("class/Project.php");
// 文書クラス
require_once("class/Document.php");
// タスククラス
require_once("class/Task.php");
// 辞書クラス
require_once("class/Term.php");
// 統計クラス
require_once("class/Statistics.php");
// 結果クラス
require_once("class/Result.php");
//システム辞書
require_once("class/QReditDict.php");
// システム設定
require_once(dirname($_SERVER["DOCUMENT_ROOT"]) . "/config.ini");

//------------------------
// グローバル変数
//------------------------
$gResult = new Result();

//------------------------
// データベース接続
//------------------------
DAO::connect();

//------------------------
// ログイン情報取得
//------------------------
{
	// ログインしていない場合
	if ( ($userID = Auth::isLogin()) === FALSE ){
		// ユーザー情報セット
		User::setInitialize(NULL);
		
		if ( isset($_SESSION["DISP_LANG"]) ){
			User::set("disp_lang", $_SESSION["DISP_LANG"]);
		}
	} else {
		if ( ! User::setInitialize($userID) ){
			Auth::logout();
			User::setInitialize(NULL);
		} else {
			// ログイン拒否設定の場合
			if ( User::get("login") === DAO::T_USER_LOGIN_NG ){
				Auth::logout();
				User::setInitialize(NULL);
			}
			// 状態卒業の場合
			if ( User::get("state") === DAO::T_USER_STATE_QUIT ){
				Auth::logout();
				User::setInitialize(NULL);
			}
		}
	}
}

//------------------------
// 表示中のプロジェクト
//------------------------
{
	if ( isset($_REQUEST["q_project_id"]) ){
		$_SESSION["project_id"] = (int)$_REQUEST["q_project_id"];
	}
	
	if ( isset($_SESSION["project_id"]) ){
		
	}
}

//------------------------
// アクセス制御
//------------------------
{
	// array(ブラウザでのアクセス権限)
	$G_HTML_ACCESS_TABLE = array(
		"^/content/etc"                            => array(DAO::T_USER_TYPE_LOGOFF, DAO::T_USER_TYPE_NORMAL, DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_ORGMGR, DAO::T_USER_TYPE_SYSMGR),
		"^/content"                                => array(DAO::T_USER_TYPE_______, DAO::T_USER_TYPE_NORMAL, DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_ORGMGR, DAO::T_USER_TYPE_SYSMGR),
		"^/app"                                    => array(DAO::T_USER_TYPE_______, DAO::T_USER_TYPE_NORMAL, DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_ORGMGR, DAO::T_USER_TYPE_SYSMGR),
		"^/\?q_logout"                             => array(DAO::T_USER_TYPE_LOGOFF, DAO::T_USER_TYPE_NORMAL, DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_ORGMGR, DAO::T_USER_TYPE_SYSMGR),
		"^/$"                                      => array(DAO::T_USER_TYPE_LOGOFF, DAO::T_USER_TYPE_NORMAL, DAO::T_USER_TYPE_INSTTR, DAO::T_USER_TYPE_ORGMGR, DAO::T_USER_TYPE_SYSMGR),
	);
	
	//------------------------
	// アクセス制限検証
	//------------------------
	$isAccess = FALSE;
	
	foreach ( $G_HTML_ACCESS_TABLE as $accessURL => $arrAccess ){
		$accessURL = preg_replace("/\//", "\/", $accessURL);
		
		if ( preg_match("/{$accessURL}/", $_SERVER["REQUEST_URI"], $match) > 0 ){
			if ( in_array(User::get("type"), $arrAccess) ){
				$isAccess = TRUE;
			} else {
				$isAccess = FALSE;
			}
			break;
		}
	}
	
	if ( ! $isAccess ){
		header("Location: /");
		//header("Location: /content/etc/404.html");
		exit();
	}
	
	if ( strcmp($_SERVER["REQUEST_URI"], "/") === 0 ){
		if ( Auth::isLogin() !== FALSE ){
			header("Location: /content/home/");
		}
	}
}

//------------------------
// 言語設定
//------------------------
{
	if ( preg_match("/\/\?hl=([a-z][a-z])$/", $_SERVER["REQUEST_URI"], $matchs) ){
		$lang = $matchs[1];
		if ( Validate::isLang($lang) ){
			$_SESSION["DISP_LANG"] = $lang;
			User::set("disp_lang", $lang);
		}
	}
}

//------------------------
// サイト文章
//------------------------
require_once(Setting::get("PATH_HTDOCS") . "/lang/minhon.word." . User::get("disp_lang") . ".php");

//------------------------
// 共通ヘッダ(多言語)
//------------------------
// 定数定義
require_once("common/DefineString.php");
// 表示クラス
require_once("class/Printer.php");

