(function($) {
	// class $.jqplot.TableMarkerLegendRenderer
	$.jqplot.TableMarkerLegendRenderer = function(){
		$.jqplot.TableLegendRenderer.call(this);
	};
	
	$.jqplot.TableMarkerLegendRenderer.prototype = new $.jqplot.TableLegendRenderer();
	$.jqplot.TableMarkerLegendRenderer.prototype.constructor = $.jqplot.TableMarkerLegendRenderer;
	
	// called with scope of legend.
	$.jqplot.TableMarkerLegendRenderer.prototype.init = function(options) {
		$.extend(true, this, options);
		
		this.markerRender = new $.jqplot.MarkerRenderer(options);
	};
	
	// called with scope of legend
	$.jqplot.TableMarkerLegendRenderer.prototype.addrow = function(label, marker, color, pad, reverse) {
		var rs = (pad) ? this.rowSpacing+'px' : '0px';
		
		var elem = document.createElement('tr');
		var tr = $(elem);
		tr.addClass('jqplot-table-legend');
		elem = null;
		
		if (reverse){
			tr.prependTo(this._table);
		}
		else{
			tr.appendTo(this._table);
		}
		
		if (this.showSwatches) {
			var td = $(document.createElement('td'));
			td.addClass('jqplot-table-legend jqplot-table-legend-swatch');
			td.css({textAlign: 'center', paddingTop: rs});

			var div0 = $(document.createElement('div'));
			div0.addClass('jqplot-table-legend-swatch-outline');
			canvas = $(document.createElement('canvas'));
			canvas.addClass('jqplot-table-legend-swatch');
			
			var canvas = $(document.createElement('canvas'));
			canvas.addClass('jqplot-table-legend-swatch');
			canvas.attr("width", "12");
			canvas.attr("height", "12");
			var constOpt = {
				show : true,
				size : 12,
				shadow : false
			};
			if ( marker.style ){
				if ( marker.style.substring(0, 6) === "filled" ){
					constOpt["size"] = 10;
				} else {
					constOpt["size"] = 8;
				}
			}
			var opt = {};
			$.extend(opt, {'color': color, 'style': 'filledSquare'}, marker, constOpt);
			this.markerRender.init(opt);
			this.markerRender.draw(6, 6, canvas.get(0).getContext('2d'), opt);
			
			tr.append(td.append(div0.append(canvas)));
			
		}
		if (this.showLabels) {
			var td = $(document.createElement('td'));
			td.addClass('jqplot-table-legend jqplot-table-legend-label');
			td.css('paddingTop', rs);
			tr.append(td);
			if (this.escapeHtml) {
				td.text(label);
			}
			else {
				td.html(label);
			}
		}
	};
	
	$.jqplot.TableMarkerLegendRenderer.prototype.draw = function() {
		if (this._elem) {
			this._elem.emptyForce();
			this._elem = null;
		}

		if (this.show) {
			var series = this._series;
			
			var elem = document.createElement('div');
			this._elem = $(elem);
			this._elem.addClass('jqplot-table-legend');
			
			var pad = false, 
				reverse = false,
				s;
			var table,
				group = 0;
			for (var i = 0; i< series.length; i++) {
				s = series[i];
				if ( group === 0 ) {
					table = document.createElement('table');
					this._table = $(table);
					this._table.appendTo(elem);
					group = s.group || 1;
				} else if ( s.group && group !== s.group ){
					table = document.createElement('table');
					this._table = $(table);
					this._table.appendTo(elem);
					group = s.group;
				}
				if (s.show && s.showLabel) {
					var lt = this.labels[i] || s.label.toString();
					if (lt) {
						var color = s.color;
						var marker = s.markerOptions;
						if (reverse && i < series.length - 1){
							pad = true;
						}
						else if (reverse && i == series.length - 1){
							pad = false;
						}
						this.renderer.addrow.call(this, lt, marker, color, pad, reverse);
						pad = true;
					}
					// let plugins add more rows to legend.  Used by trend line plugin.
					for (var j=0; j<$.jqplot.addLegendRowHooks.length; j++) {
						var item = $.jqplot.addLegendRowHooks[j].call(this, s);
						if (item) {
							this.renderer.addrow.call(this, item.label, item.markerOptions || null, item.color, pad);
							pad = true;
						} 
					}
					lt = null;
				}
			}
			
		}
		return this._elem;
	};
	

	
})(jQuery);
