<?php
Class Proc {
	const PROCESS_FILE_NAME_PROCESS = "process.process";
	const PROCESS_FILE_NAME_WAITING = "process.waiting";
	const PROCESS_FILE_NAME_UPDATE  = "process.update";
	const PROCESS_FILE_NAME_FINISH  = "process.finish";
	
	var $rootPath;
	var $intervalTime;
	var $limitTime;
	
	/***************************************************************
	 * 名前:Proc
	 * 説明:コンストラクタ
	 * 引数:
	 * 返値:
	****************************************************************/
	function Proc( $_rootPath, $_intervalTime, $_limitTime ){
		$this->rootPath     = $_rootPath;
		$this->intervalTime = $_intervalTime;
		$this->limitTime    = $_limitTime;
	}
	
	/***************************************************************
	 * 名前:isProcess
	 * 説明:プロセス中か？
	 * 引数:
	 * 返値:
	****************************************************************/
	function isProcess(){
		$isProcess = FALSE;
		
		// プロセス管理ファイル
		$processFile = $this->rootPath . "/" . self::PROCESS_FILE_NAME_PROCESS;
		
		// プロセスファイル存在(処理実行中)の場合
		if ( file_exists($processFile) ){
			echo "[info]process file existed.\n";
			
			// プロセス状態取得
			if ( ($fp = fopen($processFile, "r")) ){
				$time = fgets($fp);
				$time = trim($time);
				
				echo "[info]process file {$time}.\n";
				
				// 制限時間内に終了していない場合
				// (エラーなどで途中で止まったetc)
				if ( $time + $this->limitTime < time() ){
					echo "[info]process time over.\n";
					$isProcess = FALSE;
				} else {
					echo "[info]process continue.\n";
					$isProcess = TRUE;
				}
				fclose($fp);
			} else {
				echo "[error]process failed.\n";
				$isProcess = TRUE;
			}
		} else {
			echo "[info]process new.\n";
			$isProcess = FALSE;
		}
		
		return $isProcess;
	}
	
	/***************************************************************
	 * 名前:makeProcess
	 * 説明:プロセスファイル作成
	 * 引数:
	 * 返値:
	****************************************************************/
	function makeProcess(){
		// プロセス管理ファイル
		$processFile = $this->rootPath . "/" . self::PROCESS_FILE_NAME_PROCESS;
		
		// プロセスファイル作成
		if ( $fp = fopen($processFile, "w") ){
			echo "[info]make process file.\n";
			
			fwrite($fp, time());
			fclose($fp);
		} else {
			return FALSE;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:deleteProcess
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function deleteProcess(){
		// プロセス管理ファイル
		$processFile = $this->rootPath . "/" . self::PROCESS_FILE_NAME_PROCESS;
		
		if ( file_exists($processFile) ){
			echo "[info]delete process file.\n";
			
			if ( ! unlink($processFile) ){
				echo "[error]delete process file.\n";
				return FALSE;
			}
		}
	}
	
	/***************************************************************
	 * 名前:updateWaitTime
	 * 説明:待ち時間更新
	 * 引数:
	 * 返値:
	****************************************************************/
	function updateWaitTime(){
		$waitMin = 0;
		
		// 待ち時間ファイル
		$waitingFile = $this->rootPath . "/" . self::PROCESS_FILE_NAME_WAITING;
		
		// 過去の累計待ち時間を取得
		if ( file_exists($waitingFile) ){
			if ( $fp = fopen($waitingFile, "r") ){
				$waitMin = fread($fp, filesize($waitingFile));
				fclose($fp);
			} else {
				echo "[error]open file {$waitingFile}.\n";
				return FALSE;
			}
		}
		
		// 累計待ち時間更新
		if ( $fp = fopen($waitingFile, "w") ){
			if ( ! fwrite($fp, (int)$waitMin + $this->intervalTime) ){
				echo "[error]write waiting file.\n";
			}
			fclose($fp);
		} else {
			return FALSE;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:getTargetTime
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	function getTargetTime(){
		$time = $this->intervalTime;
		
		// 待ち時間ファイル
		$waitingFile = $this->rootPath . "/" . self::PROCESS_FILE_NAME_WAITING;
		
		if ( file_exists($waitingFile) ){
			if ( $fp = fopen($waitingFile, "r") ){
				$time = fread($fp, filesize($waitingFile));
				$time = (int)$time + $this->intervalTime;
				fclose($fp);
				
				if ( ! unlink($waitingFile) ){
					echo "[error]delete waiting file.\n";
					return FALSE;
				}
			} else {
				echo "[error]open waiting file.\n";
				return FALSE;
			}
		}
		
		return $time;
	}
	
	/***************************************************************
	 * 名前:isExistsUpdate
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static function isExistsUpdate( $dirPath ){
		return file_exists("{$dirPath}/" . self::PROCESS_FILE_NAME_UPDATE);
	}
	
	/***************************************************************
	 * 名前:makeUpdate
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static function makeUpdate( $dirPath ){
		if ( ! ($fp = fopen("{$dirPath}/" .self::PROCESS_FILE_NAME_UPDATE, "w")) ){
			echo "[error]make process update file to {$dirPath}.\n";
			return FALSE;
		}
		fclose($fp);
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:deleteUpdate
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static function deleteUpdate( $dirPath ){
		if ( ! unlink("{$dirPath}/" . self::PROCESS_FILE_NAME_UPDATE) ){
			echo "[error]delete update file in {$dirPath}.\n";
			return FALSE;
		}
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:makeFinish
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static function makeFinish( $dirPath, $date = NULL ){
		if ( ! ($fp = fopen("{$dirPath}/" .self::PROCESS_FILE_NAME_FINISH, "w")) ){
			echo "[error]make process finish file to {$dirPath}.\n";
			return FALSE;
		}
		
		if ( is_null($date) ){
			$date = time();
		}
		fwrite($fp, date("Y/m/d H:i:s", $date));
		
		fclose($fp);
		
		return TRUE;
	}
	
	/***************************************************************
	 * 名前:getFinishTime
	 * 説明:
	 * 引数:
	 * 返値:
	****************************************************************/
	static function getFinishTime( $dirPath ){
		$file = "{$dirPath}/" .self::PROCESS_FILE_NAME_FINISH;
		if ( file_exists($file) ){
			if ( ! ($fp = fopen($file, "r")) ){
				echo "[error]get process finish file to {$dirPath}.\n";
				return FALSE;
			}
			
			$time = fread($fp, 4096);
			
			fclose($fp);
			
			if ( ($time = strtotime($time)) === FALSE ){
				return FALSE;
			}
			return $time;
		}
		
		return FALSE;
	}
}
