<?php require_once(dirname(__FILE__) . "/../setting.ini"); ?>
<?php require_once("class/DocTmx.php"); ?>
<?php
//--------------------------------------------------------------------
// explain  : データファイル作成
// argument : 
//--------------------------------------------------------------------
// タイムアウト解除
set_time_limit(0);

//----------------------------
// define
//----------------------------
define("PATH_PROCESS",  dirname(__FILE__));
define("PATH_LOG_FILE", dirname(__FILE__));

define("DATA_LIMIT", 1000);

//----------------------------
// function
//----------------------------
function main( $kind, $sourceOutputPath, $targetOutputPath, $processPath = NULL ){
	if ( strcmp($kind, "document") !== 0 ){
		return FALSE;
	}
	
	$lasttime = NULL;
	if ( is_null($processPath) ){
		$processPath = PATH_PROCESS;
	}
	$timefile = "{$processPath}/process.time";
	if ( file_exists($timefile)){
		$lasttime = file_get_contents($timefile);
		if ( ! is_numeric($lasttime) ){
			$lasttime = NULL;
		}
	}
	
	$time = time(); // 処理をする前の時間を覚えておく
	
	document($sourceOutputPath, $targetOutputPath, $lasttime);
	
	// 処理開始時間を覚えておく（次回実行時はこの時間より後に更新されたデータのみ処理する）
	file_put_contents($timefile, $time);
}


function document( $sourceOutputPath, $targetOutputPath, $date ){
	DAO::connect();
	
	$objDAODocument = DAO::instance("document");
	
	// *フィールド
	$arrField = array(
		"t_document_id",
		"t_document_state",
		"t_document_lang_s",
		"t_document_lang_t",
		"t_document_text",
	);
	// *条件
	$arrCondition = array();
	if ( is_numeric($date) ){
		$sqlDate = $objDAODocument->esc(date("Y-m-d H:i:s", $date));
		$arrCondition["update"] = array("type" => DAO::TYPE_SQL, "value" => "t_document_update > \"{$sqlDate}\"");
	}
	// *実行
	$offset = 0;
	do {
		$res = $objDAODocument->select($arrField, $arrCondition, NULL, NULL, NULL, DATA_LIMIT, $offset);
		if ( $res["result"] === RES_OK ){
			foreach ( $res["data"] as $dataDocument ){
				outputDocument($dataDocument, $sourceOutputPath, $targetOutputPath);
			}
		} else {
			break;
		}
		$offset += DATA_LIMIT;
	} while ( $res["count"] > $offset );
}

function outputDocument( $dataDocument, $sourceOutputPath, $targetOutputPath ){
	$id = $dataDocument["t_document_id"];
	$state = $dataDocument["t_document_state"];
	$sLang = $dataDocument["t_document_lang_s"];
	if ( strcmp($sLang, LANG_CN) === 0 ){
		$sLang = "zh";
	}
	$tLang = $dataDocument["t_document_lang_t"];
	if ( strcmp($tLang, LANG_CN) === 0 ){
		$tLang = "zh";
	}
	$xml = $dataDocument["t_document_text"];
	
	$sourceFile = "{$sourceOutputPath}/{$id}.{$sLang}";
	$targetFile = "{$targetOutputPath}/{$id}.{$tLang}";
	
	if ( $state === DAO::T_DOCUMENT_STATE_DELETE ){
		// 文書が削除された場合は空ファイルを作成する
		file_put_contents($sourceFile, "");
		file_put_contents($targetFile, "");
		return;
	}
	
	$objDocTmx = new DocTmx();
	$objDocTmx->load($xml);
	
	$arrSentence = $objDocTmx->getSentence();
	
	$arrSourceText = array();
	$arrTargetText = array();
	foreach( $arrSentence as $sentence ){
		$arrSourceText[] = $sentence["source_text"];
		$arrTargetText[] = $sentence["target_text"];
	}
	
	if ( count($arrSourceText) > 0 && count($arrTargetText) > 0 ){
		$sourceText = mb_convert_encoding(implode("\n", $arrSourceText), "UTF-8");
		$targetText = mb_convert_encoding(implode("\n", $arrTargetText), "UTF-8");
		
		file_put_contents($sourceFile, $sourceText);
		file_put_contents($targetFile, $targetText);
		echo "{$id}.{$sLang} - {$id}.{$tLang}\n";
	}
}

//----------------------------
// process
//----------------------------
// 引数チェック
if ( $argc === 4 ){
	main( $argv[1], $argv[2], $argv[3] );
} else if ( $argc === 5 ){
	main( $argv[1], $argv[2], $argv[3], $argv[4] );
} else {
	echo "usage:\n";
	echo "php -f run.php <kind> <opath_s> <opath_t> [<ppath>]\n";
	echo "  <kind>    : \"document\" only\n";
	echo "  <opath_s> : output source file directory\n";
	echo "  <opath_t> : output target file directory\n";
	echo "  <ppath>   : (optional) process file directory\n";
}
