<?php require_once(dirname(__FILE__) . "/../setting.ini"); ?>
<?php require_once(Setting::get("PATH_ROOT") . "/app/class/Tool/ToolSimilarity.php"); ?>
<?php
//--------------------------------------------------------------------
// explain  : 類似文検索用インデックスファイル作成(対訳)
// argument : 
//--------------------------------------------------------------------
// タイムアウト解除
set_time_limit(0);

//----------------------------
// define
//----------------------------
define("PROCESS_INTERVAL_TIME", 1);
define("PROCESS_LIMIT_TIME",  (24*60*60));

//----------------------------
// variable
//----------------------------
//----------------------------
// request parameter
//----------------------------
//----------------------------
// check request parameter
//----------------------------
//----------------------------
// function
//----------------------------
function main(){
	$processFileDir = dirname(__FILE__);
	$logFileDir     = dirname(__FILE__);
	$objProc = new Proc($processFileDir, PROCESS_INTERVAL_TIME, PROCESS_LIMIT_TIME);
	
	//------------------------
	// 処理中か？
	//------------------------
	// 処理中の場合
	if ( $objProc->isProcess() === TRUE ){
		exit();
	} else {
		// プロセスファイル作成
		if ( $objProc->makeProcess() === FALSE ){
			Util::printLog("[ERROR]make process.", $logFileDir);
			exit();
		}
	}

	// 前回以降に更新したデータのみを抽出するため、前回の処理開始時間を取得する
	if ( ($lastDate = $objProc->getFinishTime($processFileDir)) === FALSE ){
		$lastDate = NULL;
	}
	
	//処理開始時間を process.finish に入れる
	$date = time();
	
	//------------------------
	// 処理
	//------------------------
	if ( run($logFileDir, $lastDate) === FALSE ){
		//Util::printLog("[ERROR]similarity run.", $logFileDir);
		exit();
	}
	
	//------------------------
	// プロセス管理ファイル削除
	//------------------------
	if ( $objProc->deleteProcess() === FALSE ){
		Util::printLog("[ERROR]delete process.", $logFileDir);
		exit();
	}
	
	//------------------------
	// 終了時間記録
	//------------------------
	if ( $objProc->makeFinish($processFileDir, $date) === FALSE ){
		Util::printLog("[ERROR]make finish.", $logFileDir);
		exit();
	}
}

function run( $logFileDir, $date = NULL ){
	$result = RES_OK;
	
	Util::printLog("[INFO] *start", $logFileDir);
	
	$simDir = Setting::get("PATH_DATA_SIMILARITY");
	Tool::makeDir($simDir);
	$tsvDataDir = Setting::get("PATH_DATA_TMP")."/bilingual_tsv";
	Tool::makeDir($tsvDataDir);
	
	DAO::connect();
	$objDAOTermRoot = DAO::instance("term_root", "bilingual");
	
	// *条件
	$sqlDate = "NULL"; // ログ用
	$arrCondition = array();
	if ( is_numeric($date) ){
		$sqlDate = $objDAOTermRoot->esc(date("Y-m-d H:i:s", $date));
		$arrCondition["update"] = array("type" => DAO::TYPE_SQL, "value" => "t_bilingual_root_update > \"{$sqlDate}\"");
	}
	
	$result = $objDAOTermRoot->outputTsv( $tsvDataDir, $arrCondition, TRUE );
	if ( $result === RES_NO_DATA ){
		Util::printLog("[INFO]No Bilingual Data(From {$sqlDate})", $logFileDir);
	} else if ( $result === RES_NG ){
		Util::printLog("[ERROR]TSV File Output Error(From {$sqlDate})", $logFileDir);
	}
	
	$arrFile = array();
	if ( $result === RES_OK ){
		// $tsvDataDir下に $id.$slang-$tlang.tsv ファイルが作成される
		$hDir = opendir($tsvDataDir);
		if ( $hDir ){
			while ( $fname = readdir($hDir) ){
				if ( is_file($tsvDataDir."/".$fname) ){
					$arrFile[] = $tsvDataDir."/".$fname;
				}
			}
			closedir($hDir);
		} else {
			Util::printLog("[ERROR]Not Found Bilingual TSV Directory({$tsvDataDir})", $logFileDir);
			return FALSE;
		}
	}
	
	//----------------------------------------
	// インデックスファイル作成
	//----------------------------------------
	if ( $result === RES_OK ){
		$objToolSimilarity = new ToolSimilarity();
		
		foreach ( $arrFile as $filepath ){
			list($bID, $bLang, $bExt) = explode(".", basename($filepath));
			
			if ( ! file_exists($filepath) ){
				Util::printLog("[ERROR]Not Found Bilingual TSV File({$filepath})", $logFileDir);
				return FALSE;
			}
			
			$outputDir = $simDir."/{$bID}";
			Tool::makeDir($outputDir);
			
			if ( ($makeResult = $objToolSimilarity->make($filepath, $outputDir)) === TRUE ){
				// 成功
			} else if ( $makeResult === -1 ){
				// インデックスの作成に失敗
				Util::printLog("[ERROR]Don't Create Similarity Index({$filepath})", $logFileDir);
			} else if ( $makeResult === -2 ){
				// インデックスファイルの移動に失敗
				Util::printLog("[ERROR]Don't Move Similarity Index({$filepath})", $logFileDir);
			}
		}
	}
	
	Tool::removeDir($tsvDataDir);
	
	Util::printLog("[INFO] *finish", $logFileDir);
	
	return ($result !== RES_NG);
}

//----------------------------
// process
//----------------------------
// 引数チェック
if ( $argc === 1 ){
	main();
} else {
	echo "usage:\n";
	echo "php -f main.php\n";
}
